/*
 * Written on June 23, 03 by Thomas Tempelmann.
 *
 * More RB-related stuff can be found here: <http://www.tempel.org/rb/>
 *
 * Enjoy and contribute!
 */

#include <Memory.h>
#include <MixedMode.h>
#include <string.h>
#include "TT's Plugin Support.h"
#include "rb_plugin.h"



extern REALevent eventFilterEvents[];
static void theEventFilterHandler (EventRecord *event, long param);

extern REALclassDefinition eventFilterClass;	// forward declaration

typedef void (*NotificationProc) (REALobject instance);

static EventRecord currentEvent;

typedef struct eventFilterData {
	// store your class properties here
	eventFilterData *next;
	NotificationProc action;
	REALobject theInstance;
} eventFilterData;

static eventFilterData *root, *last;

static Boolean hadEvent;


static void theEventFilterHandler (EventRecord *event, long)
// wird von RB vor Verarbeitung eines Events aufgerufen.
// von hier aus knnen Events in RB-Klassen aufgerufen werden
{
	if (hadEvent) {
		hadEvent = false;
		eventFilterData *me = root;
		if (me != nil) {
			do {
				// hier wird der Action()-Event aufgerufen:
				me->action ((REALobject)me->theInstance);
				me = me->next;
			} while (me != nil);
		}
	}
}

static void eventFilterConstructor (REALobject instance)
{
	ClassData (eventFilterClass, instance, eventFilterData, me);
	
	NotificationProc fp = (NotificationProc) REALGetEventInstance((REALcontrolInstance)instance, &eventFilterEvents[0]);
	me->action = fp;	// might be NULL
	me->theInstance = instance;
	
	if (fp) {
		// add this object instance to the list of to be called action handlers
		if (last != nil) last->next = me;
		last = me;
		if (root == nil) root = me;
	}
}

static void eventFilterDestructor (REALobject instance)
{
	ClassData (eventFilterClass, instance, eventFilterData, me);
	
	// find us in the list and remove us from it
	eventFilterData **p = &root;
	while (*p != nil) {
		if (*p == me) {
			if (last == me) last = me->next;
			*p = me->next;
			break;
		}
		p = &((*p)->next);
	}
}

static REALevent eventFilterEvents[] = {
	{ "Action" },
};


static REALclassDefinition eventFilterClass = {
	kCurrentREALControlVersion,
	"TTsAsyncEventDemo",
	nil,
	sizeof(eventFilterData), 0,
	(REALproc) eventFilterConstructor, (REALproc) eventFilterDestructor,
	nil, 0,	//	Properties
	nil, 0,	//	Methods
	eventFilterEvents, sizeof(eventFilterEvents) / sizeof(REALevent),
	nil, 0,
};


static TMTask tmTask = {0};

static void timerHandler ()
// dies ist ein Bsp fr einen Callback eines async Events.
// von hier aus drfen keine RB-Events aufgerufen werden.
// stattdessen wird hier nur ein Flag gesetzt (oder die Daten
// zu einer Queue hinzugefgt), das dann in theEventFilterHandler()
// ausgewertet wird.
{
	hadEvent = true;

	// damit die Reaktionszeit geringer wird, knnte hier noch
	// ein PostEvent() oder entspr. CabonEvents-Call gemacht
	// werden, um bei allernchster Gelegenheit einen Dummy-Event
	// an RB senden zu lassen, damit dann theEventFilterHandler()
	// zum Zuge kommt.

	PrimeTime ((QElemPtr)&tmTask, 1000); // repeat every 1s
}

// ---------------------------------
// ----- Plugin Initialization -----
// ---------------------------------

void PluginExit (void)
{
	if (tmTask.tmAddr) {
		DisposeRoutineDescriptor (tmTask.tmAddr);
		RmvTime ((QElemPtr)&tmTask);
	}
}

void PluginEntry (void)
{
	InitTTsPluginSupport ();

	root = nil;
	
	REALRegisterClass (&eventFilterClass);

	// install our event filter
	REALRegisterEventFilter (theEventFilterHandler, 0);
	
	// Install a async callback (a OS timer)
	tmTask.tmAddr = NewRoutineDescriptor ((ProcPtr)timerHandler, 0, GetCurrentISA());
	InsTime ((QElemPtr)&tmTask);
	PrimeTime ((QElemPtr)&tmTask, 1000); // every 1s
	hadEvent = false;
}
