#if TARGET_CARBON || TARGET_PPC
#include <iostream>
#include <sstream>
#endif

typedef unsigned char Byte;
extern "C" typedef __typeof__((char*)0-(char*)0) ptrdiff_t;  /*- cc 010411 -*/ 

#include <string>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/******************************************************************************/
/*                            REALbasic Plugin Defs                           */
/******************************************************************************/

#include "zlib.h"

#include "rb_plugin.h"

static Boolean debugOn = true ;

#if TARGET_CARBON || TARGET_PPC
	void myDebugStr(const char *file,const char *method, int line,const std::string& msg) ;

	void myDebugStr(const char *file,const char *method, int line,const std::string& msg)
	{
/*
		std::string       str ;
		std::stringstream buffer ;
		Str255            b ;

		if (!debugOn) return ;
		
		buffer << file << " - " << method << " line:" << line  << " [" << msg << "]" ;

		str = buffer.str() ;
		
	#if TARGET_CARBON || TARGET_PPC
		strncpy((char *)&b[1],&str[0], 255) ;
		if ( strlen(&str[0]) > 255 )
			b[0] = 255 ;
		else
			b[0] = strlen(&str[0]) ;		
	#endif
		
		DebugStr(b) ;
*/
	}

#else
	void myDebugStr(const char *file, const char *method, int line, const char *msg) ;

	void myDebugStr(const char *file, const char *method, int line, const char *msg) {};

#endif


struct gzipStream_data {
	gzFile file;
	Boolean LittleEndian;
};

static void doGZDebug(Boolean setOn)
{
	debugOn = setOn ;
	
	myDebugStr(__FILE__, __func__, __LINE__, "doGZDebug");
	
}

static Boolean doGZGetDebug()
{
	return debugOn;
}

// Ok, lets pre-declare all our class methods
static void gzipStream_open(REALobject instance, REALfolderItem f, Boolean writeMode) ;
static REALstring gzipStream_read(REALobject instance, long len) ;
static REALstring gzipStream_read_55(REALobject instance, long len, REALobject enc);
static long gzipStream_write(REALobject instance, REALstring buffer) ;
static void gzipStream_write2(REALobject instance, REALstring buffer);
static void gzipStream_close(REALobject instance) ;
static Boolean gzipStream_EOF(REALobject instance, long gzf) ;
static long gzipStream_Error(REALobject instance);
static REALstring gzipStream_ErrorString(REALobject instance);
static Boolean gzipStream_ReadWriteError(REALobject instance);
static long gzipStream_Position(REALobject instance);
static void gzipStream_Position2(REALobject instance, long offset);
static void gzipStream_flush(REALobject instance);
static long gzipStream_readbyte(REALobject instance);
static void gzipStream_writebyte(REALobject instance, long c);
static long gzipStream_readlong(REALobject instance);
static void gzipStream_writelong(REALobject instance, long x);
static REALstring gzipStream_readline(REALobject instance);
static REALstring gzipStream_readline_55(REALobject instance, REALobject enc);
static void gzipStream_writeline(REALobject instance, REALstring s);
static void gzipStream_setparams(REALobject instance, long level, long strategy);
static void gzipStream_constructor(REALobject instance);
static void gzipStream_destructor(REALobject instance);
static void gzipStream_default_constructor(REALobject instance);
static void gzipStream_opt_constructor(REALobject instance, REALfolderItem f, Boolean writeMode);

// Class definition goes here, because the methods refer to it
REALmethodDefinition gzipStream_methods[] = {
	{ (REALproc) gzipStream_open, REALnoImplementation, "Open(f as FolderItem, write as Boolean)" },
	{ (REALproc) gzipStream_read, REALnoImplementation, "Read(count as Integer) as String" },
	{ (REALproc) gzipStream_write, REALnoImplementation, "Write(s as String) as Integer" },
	{ (REALproc) gzipStream_write2, REALnoImplementation, "Write(s as String)" },
	{ (REALproc) gzipStream_flush, REALnoImplementation, "Flush()" },
	{ (REALproc) gzipStream_close, REALnoImplementation, "Close()" },
	{ (REALproc) gzipStream_EOF, REALnoImplementation, "EOF as Boolean" },
	{ (REALproc) gzipStream_Error, REALnoImplementation, "Error() as Integer" },
	{ (REALproc) gzipStream_ErrorString, REALnoImplementation, "ErrorString() as String" },
	{ (REALproc) gzipStream_ReadWriteError, REALnoImplementation, "ReadError as Boolean" },
	{ (REALproc) gzipStream_ReadWriteError, REALnoImplementation, "WriteError as Boolean" },
	{ (REALproc) gzipStream_Position, (REALproc) gzipStream_Position2, "Position as Integer" },
	{ (REALproc) gzipStream_readbyte, REALnoImplementation, "ReadByte() as Integer" },
	{ (REALproc) gzipStream_readlong, REALnoImplementation, "ReadLong() as Integer" },
	{ (REALproc) gzipStream_readline, REALnoImplementation, "ReadLine() as String" },
	{ (REALproc) gzipStream_writebyte, REALnoImplementation, "WriteByte(byte as Integer)" },
	{ (REALproc) gzipStream_writelong, REALnoImplementation, "WriteLong(long as Integer)" },
	{ (REALproc) gzipStream_writeline, REALnoImplementation, "WriteLine(line as String)" },
	{ (REALproc) gzipStream_setparams, REALnoImplementation, "SetParameters(level as Integer, strategy as Integer)" },
	{ (REALproc) gzipStream_default_constructor, REALnoImplementation, "GZipStream()" },
	{ (REALproc) gzipStream_opt_constructor, REALnoImplementation, "GZipStream(f as FolderItem, write as Boolean)" }
};

REALmethodDefinition gzipStream_methods_55[] = {
	{ (REALproc) gzipStream_open, REALnoImplementation, "Open(f as FolderItem, write as Boolean)" },
	{ (REALproc) gzipStream_read_55, REALnoImplementation, "Read(count as Integer, enc as TextEncoding = nil) as String" },
	{ (REALproc) gzipStream_write, REALnoImplementation, "Write(s as String) as Integer" },
	{ (REALproc) gzipStream_write2, REALnoImplementation, "Write(s as String)" },
	{ (REALproc) gzipStream_flush, REALnoImplementation, "Flush()" },
	{ (REALproc) gzipStream_close, REALnoImplementation, "Close()" },
	{ (REALproc) gzipStream_EOF, REALnoImplementation, "EOF as Boolean" },
	{ (REALproc) gzipStream_Error, REALnoImplementation, "Error() as Integer" },
	{ (REALproc) gzipStream_ErrorString, REALnoImplementation, "ErrorString() as String" },
	{ (REALproc) gzipStream_ReadWriteError, REALnoImplementation, "ReadError as Boolean" },
	{ (REALproc) gzipStream_ReadWriteError, REALnoImplementation, "WriteError as Boolean" },
	{ (REALproc) gzipStream_Position, (REALproc) gzipStream_Position2, "Position as Integer" },
	{ (REALproc) gzipStream_readbyte, REALnoImplementation, "ReadByte() as Integer" },
	{ (REALproc) gzipStream_readlong, REALnoImplementation, "ReadLong() as Integer" },
	{ (REALproc) gzipStream_readline_55, REALnoImplementation, "ReadLine(enc as TextEncoding = nil) as String" },
	{ (REALproc) gzipStream_writebyte, REALnoImplementation, "WriteByte(byte as Integer)" },
	{ (REALproc) gzipStream_writelong, REALnoImplementation, "WriteLong(long as Integer)" },
	{ (REALproc) gzipStream_writeline, REALnoImplementation, "WriteLine(line as String)" },
	{ (REALproc) gzipStream_setparams, REALnoImplementation, "SetParameters(level as Integer, strategy as Integer)" },
	{ (REALproc) gzipStream_default_constructor, REALnoImplementation, "GZipStream()" },
	{ (REALproc) gzipStream_opt_constructor, REALnoImplementation, "GZipStream(f as FolderItem, write as Boolean)" }
};

REALproperty gzipStream_properties[] = {
	{ nil, "LittleEndian", "Boolean", REALpropRuntimeOnly,
	  REALstandardGetter, REALstandardSetter, FieldOffset(gzipStream_data, LittleEndian) }
};

#pragma mark - Class definition
// We have to use 2 versions of the class because in v5.5 we want to support the Readable/Writeable interfaces
// as well as optional Encoding parameters, which requires dynamic access.
REALclassDefinition gzipStream_class = {
	kCurrentREALControlVersion,
	"GZipStream",
	nil,
	sizeof(gzipStream_data),
	0,
	(REALproc) gzipStream_constructor,
	(REALproc) gzipStream_destructor,
	gzipStream_properties,
	sizeof(gzipStream_properties) / sizeof(REALproperty),
	gzipStream_methods,
	sizeof(gzipStream_methods) / sizeof(REALmethodDefinition),
};

REALclassDefinition gzipStream_class_55 = {
	kCurrentREALControlVersion,
	"GZipStream",
	nil,
	sizeof(gzipStream_data),
	0,
	(REALproc) gzipStream_constructor,
	(REALproc) gzipStream_destructor,
	gzipStream_properties,
	sizeof(gzipStream_properties) / sizeof(REALproperty),
	gzipStream_methods_55,
	sizeof(gzipStream_methods_55) / sizeof(REALmethodDefinition),
	nil,
	0,
	nil,
	0,
	"Readable,Writable",
};

REALclassDefinition gzipStream_class_reg;

// Constructor/Destructor
static void gzipStream_constructor(REALobject instance)
{
	ClassData (gzipStream_class_reg, instance, gzipStream_data, data);
	
	myDebugStr(__FILE__, __func__, __LINE__, "constructor");
	
	data->file = NULL;
	data->LittleEndian = 0;
}

static void gzipStream_destructor(REALobject instance)
{
	myDebugStr(__FILE__, __func__, __LINE__, "destructor");
	gzipStream_close(instance);
}

static void gzipStream_default_constructor(REALobject instance)
{
	// There's nothing to do here - we already zero the data in the class-defined constructor
}

static void gzipStream_opt_constructor(REALobject instance, REALfolderItem f, Boolean writeMode)
{
	gzipStream_open(instance, f, writeMode);
}

// Class methods
static void gzipStream_open(REALobject instance, REALfolderItem f, Boolean writeMode) 
{
	ClassData (gzipStream_class_reg, instance, gzipStream_data, data);
	
	REALstring fullPath ;
	
	fullPath = REALConvertString(REALpathFromFolderItem(f), REALGetFontEncoding("System"));
	
	if ( fullPath == NULL )
	{
#if !WIN32
		DebugStr("\pgzipStream_open REALpathFromFolderItem FAILED");
#endif
		return;
	}
	
	myDebugStr((const char *)__FILE__, (const char *)__func__, __LINE__, "open gzip file" );
	myDebugStr((const char *)__FILE__, (const char *)__func__, __LINE__, fullPath->CString() );
	
	if ( writeMode )
		data->file = gzopen(fullPath->CString(), "wb");
	else
		data->file = gzopen(fullPath->CString(), "rb");
	
	if ( data->file == NULL )
		myDebugStr(__FILE__, __func__, __LINE__, "open FAILED" );
}

static REALstring gzipStream_read(REALobject instance, long len) 
{
	ClassData (gzipStream_class_reg, instance, gzipStream_data, data);
	
	REALstring toReturn;
	
	char *buffer = (char *)calloc(len+1, sizeof(char));
	
	if ( buffer == NULL )
	{
		myDebugStr(__FILE__, __func__, __LINE__,"gzipStream_read alloc buffer FAILED");
		toReturn = REALBuildString("",0);
		REALSetStringEncoding(toReturn, REALGetFontEncoding("System"));
		return toReturn;
	}
	int actual = gzread(data->file, buffer, len);
	myDebugStr(__FILE__, __func__, __LINE__, "read" );
	myDebugStr(__FILE__, __func__, __LINE__, buffer );	 
	
	toReturn = REALBuildString(buffer,actual);
	REALSetStringEncoding(toReturn, REALGetFontEncoding("System"));
	
	free(buffer);
	
	return toReturn;
}

static REALstring gzipStream_read_55(REALobject instance, long len, REALobject enc)
{
	REALstring toReturn;
	long code;
	
	toReturn = gzipStream_read(instance, len);
	if (REALGetPropValue(enc, "Code", &code))
		REALSetStringEncoding(toReturn, code);
	return toReturn;
}

static long gzipStream_write(REALobject instance, REALstring buffer) 
{
	ClassData (gzipStream_class_reg, instance, gzipStream_data, data);
	
	myDebugStr(__FILE__, __func__, __LINE__, "write" );
	myDebugStr(__FILE__, __func__, __LINE__, buffer->CString() );
	
	return gzwrite(data->file, (char *)buffer->CString(), buffer->Length());
}

static void gzipStream_write2(REALobject instance, REALstring buffer)
{
	gzipStream_write(instance, buffer);
}

static void gzipStream_close(REALobject instance)
{
	ClassData (gzipStream_class_reg, instance, gzipStream_data, data);
	
	myDebugStr(__FILE__, __func__, __LINE__, "close" );
	
	if (data->file != 0) {
		gzclose(data->file);
		data->file = 0;
	}
}

static Boolean gzipStream_EOF(REALobject instance, long gzf)
{
	int isEOF ;
	
	ClassData (gzipStream_class_reg, instance, gzipStream_data, data);
	
	myDebugStr(__FILE__, __func__, __LINE__, "test EOF" );
	
	isEOF = gzeof(data->file) ;	
	
	if (isEOF)
		myDebugStr(__FILE__, __func__, __LINE__, "EOF" );
	else
		myDebugStr(__FILE__, __func__, __LINE__, "NOT EOF" );
	
	return isEOF;
}

static long gzipStream_Error(REALobject instance)
{
	int errnum;
	
	ClassData (gzipStream_class_reg, instance, gzipStream_data, data);
	
	gzerror(data->file, &errnum);
	
	myDebugStr(__FILE__, __func__, __LINE__, "Error" );
	
	return errnum;	 
}

static REALstring gzipStream_ErrorString(REALobject instance)
{
	const char *errStr;
	int errnum;
	REALstring rbErrStr;
	
	ClassData (gzipStream_class_reg, instance, gzipStream_data, data);
	
	errStr = gzerror(data->file, &errnum);
	
	rbErrStr = REALBuildString(errStr, strlen(errStr) );
	REALSetStringEncoding(rbErrStr, REALGetFontEncoding("System"));

	myDebugStr((const char *)__FILE__, (const char *)__func__, __LINE__, "ErrorString" );
	myDebugStr(__FILE__, __func__, __LINE__, rbErrStr->CString() );
	
	return rbErrStr;
}

static Boolean gzipStream_ReadWriteError(REALobject instance)
{
	int errnum;
		
	ClassData (gzipStream_class_reg, instance, gzipStream_data, data);
	
	myDebugStr(__FILE__, __func__, __LINE__, "ReadWriteError" );
	
	gzerror(data->file, &errnum);
	
	return (errnum != Z_OK);
}

static long gzipStream_Position(REALobject instance)
{
	ClassData (gzipStream_class_reg, instance, gzipStream_data, data);
	
	myDebugStr(__FILE__, __func__, __LINE__, "position" );

	return gztell(data->file);
}

static void gzipStream_Position2(REALobject instance, long offset)
{
	ClassData (gzipStream_class_reg, instance, gzipStream_data, data);
	
	myDebugStr(__FILE__, __func__, __LINE__, "position" );
	
	gzseek(data->file, offset, SEEK_SET);
}

static void gzipStream_flush(REALobject instance)
{
	ClassData (gzipStream_class_reg, instance, gzipStream_data, data);
	
	myDebugStr(__FILE__, __func__, __LINE__, "flush" );
	
	gzflush(data->file, Z_SYNC_FLUSH);
}

static long gzipStream_readbyte(REALobject instance)
{
	ClassData (gzipStream_class_reg, instance, gzipStream_data, data);
	
	myDebugStr(__FILE__, __func__, __LINE__, "readbyte" );
	
	return gzgetc(data->file);
}

static void gzipStream_writebyte(REALobject instance, long c)
{
	ClassData (gzipStream_class_reg, instance, gzipStream_data, data);
	
	myDebugStr(__FILE__, __func__, __LINE__, "writebyte" );
	
	gzputc(data->file, c);
}

static long gzipStream_readlong(REALobject instance)
{
	ClassData (gzipStream_class_reg, instance, gzipStream_data, data);
	
	myDebugStr(__FILE__, __func__, __LINE__, "readlong" );
	
	long result;
	
	// Check the endian value
	if (data->LittleEndian) {
		result = (long)gzgetc(data->file);
		result |= ((long)gzgetc(data->file))<<8;
		result |= ((long)gzgetc(data->file))<<16;
		result |= ((long)gzgetc(data->file))<<24;
	} else {
		gzread(data->file, &result, sizeof(long));
	}
	
	return result;
}

static void gzipStream_writelong(REALobject instance, long x)
{
	ClassData (gzipStream_class_reg, instance, gzipStream_data, data);
	
	myDebugStr(__FILE__, __func__, __LINE__, "writelong" );
	
	// Check the endian value
	if (data->LittleEndian) {
		for (int n = 0; n < 4; n++) {
			gzputc(data->file, (int)(x & 0xFF));
			x >>= 8;
		}
	} else {
		gzwrite(data->file, &x, sizeof(long));
	}
}

static REALstring gzipStream_readline(REALobject instance)
{
	REALstring toReturn;
	
	ClassData (gzipStream_class_reg, instance, gzipStream_data, data);
	
	// We can't use gzgets() because that requires a length, and we want
	// to keep going until EOF or newline. So lets copy the gzgets() code
	// and modify it here
	const int origLen = 1024;
	int len = origLen;
	char *buf = (char *)calloc(len, sizeof(char));
	char *b = buf;

	while (gzread(data->file, buf, 1) == 1 && *buf++ != '\n') {
		if (--len == 0) {
			len = (int)(buf-b);
			buf = (char *)calloc(len+origLen, sizeof(char));
			memcpy(buf, b, len);
			b = buf;
			buf = b + len;
			len = origLen;
		}
	}
	*buf = '\0';
	if (b == buf) {
		toReturn = REALBuildString("",0);
		REALSetStringEncoding(toReturn, REALGetFontEncoding("System"));
	} else {
		toReturn = REALBuildString(b, (int)(buf-b));
		REALSetStringEncoding(toReturn, REALGetFontEncoding("System"));
	}
	
	free(b);
	
	myDebugStr(__FILE__, __func__, __LINE__, "readline" );
	myDebugStr(__FILE__, __func__, __LINE__, toReturn->CString() );
	
	return toReturn;
}

static REALstring gzipStream_readline_55(REALobject instance, REALobject enc)
{
	REALstring toReturn;
	long code;
	
	toReturn = gzipStream_readline(instance);
	if (REALGetPropValue(enc, "Code", &code))
		REALSetStringEncoding(toReturn, code);
	
	return toReturn;
}

static void gzipStream_writeline(REALobject instance, REALstring s)
{
	ClassData (gzipStream_class_reg, instance, gzipStream_data, data);
	
	char *buffer = (char *)calloc(s->Length()+2, sizeof(char));
	memcpy(buffer, s->CString(), s->Length());
	buffer[s->Length()] = '\n';
	buffer[s->Length()+1] = 0;
	
	gzwrite(data->file, buffer, s->Length()+1);
	
	myDebugStr(__FILE__, __func__, __LINE__, "writeline" );
	myDebugStr(__FILE__, __func__, __LINE__, buffer );
	
	free(buffer);
}

static void gzipStream_setparams(REALobject instance, long level, long strategy)
{
	ClassData (gzipStream_class_reg, instance, gzipStream_data, data);
	
	myDebugStr(__FILE__, __func__, __LINE__, "setparams" );
	
	gzsetparams(data->file, level, strategy);
}

static REALstring doGZVersion()
{
	REALstring toReturn;
	const char *vers = zlibVersion();
	toReturn = REALBuildString(vers, strlen(vers));
	REALSetStringEncoding(toReturn, REALGetFontEncoding("System"));
	return toReturn;
}

static int compressError;

static long doGZCompressError()
{
	return compressError;
}
	
static REALstring doGZCompress(REALstring source)
{
	REALstring toReturn ;
	uLong bufferLen =  1.01*source->Length()+12 ;
	char *buffer = (char *)calloc(bufferLen, sizeof(char) ) ;
	
	if ( buffer == NULL )
	{
		myDebugStr(__FILE__, __func__, __LINE__, "Calloc FAILED" ) ;
		compressError = Z_MEM_ERROR ;
		return REALBuildString("",0) ;
	}

 	compressError = compress ((unsigned char *)buffer, &bufferLen, (const unsigned char *)source->CString(), source->Length() );

	if (compressError == Z_OK)
	{
		myDebugStr(__FILE__, __func__, __LINE__, "compress returned Z_OK" ) ;
		toReturn = REALBuildString(	(char *)buffer, bufferLen) ;
	}
	else
	{
		myDebugStr(__FILE__, __func__, __LINE__, "compress FAILED (not Z_OK)" ) ;
		toReturn = REALBuildString("",0) ;
	}
		
	free(buffer) ;

	return toReturn ;
}

static REALstring doGZCompress2(REALstring source, long level)
{
	REALstring toReturn ;
	uLong bufferLen =  1.01*source->Length()+12 ;
	char *buffer = (char *)calloc(bufferLen, sizeof(char) ) ;

	if ( buffer == NULL )
	{
		myDebugStr(__FILE__, __func__, __LINE__, "Calloc FAILED" ) ;
		compressError = Z_MEM_ERROR ;
		return REALBuildString("",0) ;
	}

 	compressError = compress2 ((unsigned char *)buffer, &bufferLen, (const unsigned char *)source->CString(),source->Length(), level);
 	
	if (compressError == Z_OK)
		toReturn = REALBuildString(	buffer, bufferLen) ;
	else
		toReturn = REALBuildString("",0) ;
		
	free(buffer) ;

	return toReturn ;
}

static REALstring doGZUncompress(REALstring source, long uncompressedSize)
{
	REALstring toReturn ;
	uLong bufferLen =  uncompressedSize ;
	char *buffer = (char *)calloc(bufferLen, sizeof(char) ) ;

	if ( buffer == NULL )
	{
		myDebugStr(__FILE__, __func__, __LINE__, "Calloc FAILED" ) ;
		compressError = Z_MEM_ERROR ;
		return REALBuildString("",0) ;
	}

 	compressError = uncompress ((unsigned char *)buffer, &bufferLen, (const unsigned char *)source->CString(), source->Length());

	if (compressError == Z_OK)
		toReturn = REALBuildString(	buffer, bufferLen) ;
	else
		toReturn = REALBuildString("",0) ;
		
	free(buffer) ;

	return toReturn ;
}

// Earlier than RB 5.5 - methods
REALmethodDefinition gzCompressErr_defn = { (REALproc) doGZCompressError, REALnoImplementation, "gzCompressError() as Integer" }; 
REALmethodDefinition gzCompress_defn    = { (REALproc) doGZCompress     , REALnoImplementation, "gzCompress(toCompress as String) as String" }; 
REALmethodDefinition gzCompress2_defn   = { (REALproc) doGZCompress2    , REALnoImplementation, "gzCompress(toCompress as String, compressionLevel as Integer) as String" }; 
REALmethodDefinition gzUncompress_defn  = { (REALproc) doGZUncompress   , REALnoImplementation, "gzUncompress(toUncompress as String, uncompressedSize as Integer) as String" }; 
REALmethodDefinition gzDebug_defn       = { (REALproc) doGZDebug        , REALnoImplementation, "gzDebug(debugOn as Boolean)" }; 
REALmethodDefinition gzVersion_defn     = { (REALproc) doGZVersion      , REALnoImplementation, "gzVersion() as String" }; 

// RB 5.5 or greater - module
REALmethodDefinition gzModule_methods[] = {
	{ (REALproc) doGZDebug, REALnoImplementation, "Debug(assigns onOff as boolean)" },
	{ (REALproc) doGZGetDebug, REALnoImplementation, "Debug() as Boolean" },
	{ (REALproc) doGZVersion, REALnoImplementation, "Version() as String" },
	{ (REALproc) doGZCompressError, REALnoImplementation, "Error() as Integer" },
	{ (REALproc) doGZCompress, REALnoImplementation, "Compress(source as String) as String" },
	{ (REALproc) doGZCompress2, REALnoImplementation, "Compress(source as String, level as Integer) as String" },
	{ (REALproc) doGZUncompress, REALnoImplementation, "Uncompress(source as String, uncompressedSize as Integer) as String" }
};

REALconstant gzModule_constants[] = {
	{ "Z_OK = 0" },
	{ "Z_ERRNO = -1" },
	{ "Z_STREAM_ERROR = -2" },
	{ "Z_DATA_ERROR = -3" },
	{ "Z_MEM_ERROR = -4" },
	{ "Z_BUF_ERROR = -5" },
	{ "Z_NO_COMPRESSION = 0" },
	{ "Z_BEST_SPEED = 1" },
	{ "Z_BEST_COMPRESSION = 9" },
	{ "Z_DEFAULT_COMPRESSION = -1" },
	{ "Z_FILTERED = 1" },
	{ "Z_HUFFMAN_ONLY = 2" },
	{ "Z_DEFAULT_STRATEGY = 0" }
};	

REALmoduleDefinition gzipModule_defn = {
	kCurrentREALControlVersion,
	"GZip",
	gzModule_methods,
	sizeof(gzModule_methods) / sizeof(REALmethodDefinition),
	gzModule_constants,
	sizeof(gzModule_constants) / sizeof(REALconstant)
};

// Earlier than RB 5.5 - pseudo-constants
static long gzConst_Z_OK() { return 0; };
static long gzConst_Z_ERRNO() { return -1; };
static long gzConst_Z_STREAM_ERROR() { return -2; };
static long gzConst_Z_DATA_ERROR() { return -3; };
static long gzConst_Z_MEM_ERROR() { return -4; };
static long gzConst_Z_BUF_ERROR() { return -5; };
static long gzConst_Z_NO_COMPRESSION() { return 0; };
static long gzConst_Z_BEST_SPEED() { return 1; };
static long gzConst_Z_BEST_COMPRESSION() { return 9; };
static long gzConst_Z_DEFAULT_COMPRESSION() { return -1; };
static long gzConst_Z_FILTERED() { return 1; };
static long gzConst_Z_HUFFMAN_ONLY() { return 2; };
static long gzConst_Z_DEFAULT_STRATEGY() { return 0; };
REALmethodDefinition gzConst_Z_OK_defn = { (REALproc) gzConst_Z_OK, REALnoImplementation, "Z_OK as Integer" };
REALmethodDefinition gzConst_Z_ERRNO_defn = { (REALproc) gzConst_Z_ERRNO, REALnoImplementation, "Z_ERRNO as Integer" };
REALmethodDefinition gzConst_Z_STREAM_ERROR_defn = { (REALproc) gzConst_Z_STREAM_ERROR, REALnoImplementation, "Z_STREAM_ERROR as Integer" };
REALmethodDefinition gzConst_Z_DATA_ERROR_defn = { (REALproc) gzConst_Z_DATA_ERROR, REALnoImplementation, "Z_DATA_ERROR as Integer" };
REALmethodDefinition gzConst_Z_MEM_ERROR_defn = { (REALproc) gzConst_Z_MEM_ERROR, REALnoImplementation, "Z_MEM_ERROR as Integer" };
REALmethodDefinition gzConst_Z_BUF_ERROR_defn = { (REALproc) gzConst_Z_BUF_ERROR, REALnoImplementation, "Z_BUF_ERROR as Integer" };
REALmethodDefinition gzConst_Z_NO_COMPRESSION_defn = { (REALproc) gzConst_Z_NO_COMPRESSION, REALnoImplementation, "Z_NO_COMPRESSION as Integer" };
REALmethodDefinition gzConst_Z_BEST_SPEED_defn = { (REALproc) gzConst_Z_BEST_SPEED, REALnoImplementation, "Z_BEST_SPEED as Integer" };
REALmethodDefinition gzConst_Z_BEST_COMPRESSION_defn = { (REALproc) gzConst_Z_BEST_COMPRESSION, REALnoImplementation, "Z_BEST_COMPRESSION as Integer" };
REALmethodDefinition gzConst_Z_DEFAULT_COMPRESSION_defn = { (REALproc) gzConst_Z_DEFAULT_COMPRESSION, REALnoImplementation, "Z_DEFAULT_COMPRESSION as Integer" };
REALmethodDefinition gzConst_Z_FILTERED_defn = { (REALproc) gzConst_Z_FILTERED, REALnoImplementation, "Z_FILTERED as Integer" };
REALmethodDefinition gzConst_Z_HUFFMAN_ONLY_defn = { (REALproc) gzConst_Z_HUFFMAN_ONLY, REALnoImplementation, "Z_HUFFMAN_ONLY as Integer" };
REALmethodDefinition gzConst_Z_DEFAULT_STRATEGY_defn = { (REALproc) gzConst_Z_DEFAULT_STRATEGY, REALnoImplementation, "Z_DEFAULT_STRATEGY as Integer" };


void PluginEntry(void)
{
	compressError = Z_OK ;

	myDebugStr(__FILE__, __func__, __LINE__, "plugin entry");
	
	// Lets register!
	// If this is RB 5.5 or greater, we're going to register a module namespace for our global methods
	// In versions earlier than RB 5.5 lets just use global methods
	if (REALGetRBVersion() >= 5.5) 
	{
		myDebugStr(__FILE__, __func__, __LINE__, "version >= 5.5");
		
		// Register the module
		REALRegisterModule(&gzipModule_defn);

		myDebugStr(__FILE__, __func__, __LINE__, "setup to register stream");

		// Select the v5.5 class
		gzipStream_class_reg = gzipStream_class_55;
	} 
	else 
	{
		// Register the global methods
		REALRegisterMethod(&gzCompressErr_defn );
		REALRegisterMethod(&gzCompress_defn );
		REALRegisterMethod(&gzCompress2_defn );
		REALRegisterMethod(&gzUncompress_defn );
		// Register the pseudo-constants
		REALRegisterMethod(&gzConst_Z_OK_defn);
		REALRegisterMethod(&gzConst_Z_ERRNO_defn);
		REALRegisterMethod(&gzConst_Z_STREAM_ERROR_defn);
		REALRegisterMethod(&gzConst_Z_DATA_ERROR_defn);
		REALRegisterMethod(&gzConst_Z_MEM_ERROR_defn);
		REALRegisterMethod(&gzConst_Z_BUF_ERROR_defn);
		REALRegisterMethod(&gzConst_Z_NO_COMPRESSION_defn);
		REALRegisterMethod(&gzConst_Z_BEST_SPEED_defn);
		REALRegisterMethod(&gzConst_Z_BEST_COMPRESSION_defn);
		REALRegisterMethod(&gzConst_Z_DEFAULT_COMPRESSION_defn);
		REALRegisterMethod(&gzConst_Z_FILTERED_defn);
		REALRegisterMethod(&gzConst_Z_HUFFMAN_ONLY_defn);
		REALRegisterMethod(&gzConst_Z_DEFAULT_STRATEGY_defn);
		REALRegisterMethod(&gzDebug_defn);
		REALRegisterMethod(&gzVersion_defn);

		myDebugStr(__FILE__, __func__, __LINE__, "setup to register stream");

		// Select the pre-5.5 class
		gzipStream_class_reg = gzipStream_class;
	}

	myDebugStr(__FILE__, __func__, __LINE__, "register stream");
	
	// Now register the class
	REALRegisterClass(&gzipStream_class_reg);
}

extern "C" FILE* fdopen (int, const char*);
extern "C" FILE* fdopen (int, const char*)
{ return 0; }
