/*
 * Collection extension plugin for REALbasic 2.1
 *
 * The accompanying project file was created using CodeWarrior Pro 4
 * with the latest updates from Metrowerks' FTP Site (including IDE 3.3).
 *
 * This code is freeware, you may use and extend it as you like.
 * It was created on Mar 21, 2000 by Thomas Tempelmann.
 * Method "Exists" was added Mar 11, 2001 by Thomas Tempelmann.
 *
 * More RB-related stuff can be found here: <http://www.tempel.org/rb/>
 *
 * Enjoy and contribute!
 */


//--------------------
// Your includes here
//--------------------

#include <string.h>	// this and any other includes must come _before_ the "pragma mpwc on"!
#ifndef WIN32
	#include <MacMemory.h>
#else
	static void Debugger() {}
	static void DebugStr(unsigned char* p) {}
#endif

//-------------------------
// RB header includes here
//-------------------------

#if TARGET_CPU_68K
	#pragma mpwc on
#endif

#include "rb_plugin.h"	// this include must come _after_ the "pragma mpwc on"!

typedef long RBinteger;

typedef REALstring *REALstringPtr;
typedef REALstringPtr StrArrayHdl[];
typedef struct {
	StrArrayHdl **str;
	long elems;
} StrArrayDesc;
typedef struct {
	long refCount;
	long l1, l2, l3, l4, l5;
	StrArrayDesc *keys;
} CollectionDesc;

/*
 * The following routine is the heart of this plugin.
 * It find the Key with the given index in a Collection object,
 * and returns a copy of the key's string value.
 *
 * (If you are adventurous, you may also return a reference
 * to the key itself, so that you could even modify it, but
 * this is not a good idea for future compatibility, because
 * RS might later decide that keys will be stored in a different
 * format that's optimized for key searching, and then the
 * key names may can be modified any more without bringing the
 * search table out of order. In case you want to get the
 * reference anyway, do not forget to call "REALLockObject"
 * on the key string object you're getting).
 */
static REALstring collKey (CollectionDesc* c, RBinteger idx)
{
	const char* s = "";
	if (idx > 0) { // index is 1-based. If index is < 1, then return empty string
		StrArrayDesc *strArr = c->keys;
		if (idx <= strArr->elems) {	// if index exceeds array boundary, then return empty string
			REALstring rs;
			rs = *((**strArr->str)[idx-1]);
			s = (const char*) rs->PString();
		}
	}
	return REALBuildString (s+1, s[0]);
}

/*
 * The following function has been added March 11, 2001.
 * It tests if a certain key exists.
 */

typedef long (*REALStringCompareProc) (REALstring, REALstring, long); // !TT: not sure if the 3rd parm is used at all
static REALStringCompareProc pREALStringCompare;

static long collTst (CollectionDesc* c, REALstring keyName)
{
	if (keyName) {
		const unsigned char* kn = keyName->PString();
		StrArrayDesc *strArr = c->keys;
		for (int idx = 0; idx < strArr->elems; ++idx) {
			REALstring rs = *((**strArr->str)[idx]);
			const unsigned char* s = rs->PString();
			if (s[0] == kn[0]) {
				if (pREALStringCompare (keyName, rs, 1) == 0) {
					return true;
				}
			}
		}
	}
	return false;
}


static REALmethodDefinition sampleClassMethods[] = {
	{ (REALproc) collKey, REALnoImplementation, "Key(index as Integer) as String"},
	{ (REALproc) collTst, REALnoImplementation, "Exists(name as String) as Boolean"},
};

static REALclassDefinition sampleClass = {
	kCurrentREALControlVersion,
	"Collection",
	nil,
	0,
	0,
	nil, nil,
	nil,
	0,
	sampleClassMethods,
	sizeof(sampleClassMethods) / sizeof(REALmethodDefinition),
	nil,
	0
};


// for this to work, the "static" keyword has to be removed from the "gResolver" declarations in "PluginMain.cpp":
#ifdef powerc
	extern void *(*gResolverPPC)(const char *entryName);
	#define CallResolver(a)gResolverPPC (a)
#else
	extern void *(*gResolver)(const char *entryName);
	#define CallResolver(a) gResolver(a)
#endif

void PluginEntry (void)
{
	pREALStringCompare = (REALStringCompareProc) CallResolver("RuntimeStringCompare");

	REALRegisterClassExtension (&sampleClass);
}
