/*
 * Attention! There is still a bug in RB 3.0 around using BYREF parameters:
 * You can not use BYREF _and_ return a value from a function. That is,
 * If you use BYREF parameters, you must declare the C function to not
 * return anything (void) until this bug is fixed in RB!
 */

//--------------------
// Your includes here
//--------------------

#include <map>

#include <string.h>
#if TARGET_OS_WIN32
	static void inline DebugStr(unsigned char* p) {}
	#define log(x) OutputDebugString(x)
	#define PrepareCallback() 
	#define EnterCallback()
	#define ExitCallback()
#else
	#if TARGET_CPU_68K
		#include <SetupA4.h>
	#endif
	#include <MacMemory.h>
	#include "dcon.h"
	#define log(x) dprintf(x)
#endif

//-------------------------
// RB header includes here
//-------------------------

#include "TT's Plugin Support.h"
#include "rb_plugin.h"


//-----------------------
// Some type definitions
//-----------------------

typedef long RBinteger;
typedef Boolean	RBboolVar;	// use this for Boolean variables stored in this plugin
typedef Boolean	RBboolPar;	// use this to receive Boolean parameters (both by value and by reference)
typedef long	RBboolRet;	// use this when returning a Boolean as a function result

#if TARGET_CPU_68K
	typedef Float80 RBdouble;	// Attention: for this to work, "Floating Point" in the "68K Processor" options must be set to "SANE".
#else
	typedef double RBdouble;
#endif

//-----------------------
// Add global RB methods
//-----------------------

static RBinteger do_alloc (RBinteger a)
{
	return (long)malloc(a);
}

static void do_free (RBinteger a)
{
	free((char*)a);
}

static REALmethodDefinition pluginMethodArray[] = {
	{ (REALproc) do_alloc, REALnoImplementation, "Malloc(n as Integer) as Integer" },
	{ (REALproc) do_free, REALnoImplementation, "Mfree(x as Integer)" },
};

//------------------------------------------------------
// Create a new RB class or extend an existing RB class
//------------------------------------------------------

extern REALclassDefinition sampleClass;	// forward declaration

struct sampleClassData
{
};

static void sampleClassConstructor (REALobject instance)
{
	ClassData (sampleClass, instance, sampleClassData, me);
}

static void sampleClassDestructor (REALobject instance)
{
	ClassData (sampleClass, instance, sampleClassData, me);
}

static RBinteger sampleIntGetter (REALobject instance, long param)
// This is called when RB reads a property of your class
{
	ClassData (sampleClass, instance, sampleClassData, me);
	return 0;
}

static void sampleIntSetter (REALobject instance, long param, RBinteger value)
// This is called when RB assigns a value to a property of your class
{
	ClassData (sampleClass, instance, sampleClassData, me);
}

static void sampleClassMethod (REALobject instance, RBinteger aValue)
{
	ClassData (sampleClass, instance, sampleClassData, me);
}

/*
static REALproperty sampleClassProperties[] = {
	{ nil, "SampleInt", "Integer",         0, (REALproc) sampleIntGetter,    (REALproc) sampleIntSetter,    0 },
	{ nil, "SamplePassiveBool", "Boolean", 0, REALstandardGetter, REALstandardSetter, FieldOffset(sampleClassData, aBoolean) },
};
*/

static REALmethodDefinition sampleClassMethods[] = {
	{ (REALproc) sampleClassMethod, REALnoImplementation, "SampleMethod(aValue as Integer)"},
};


static REALclassDefinition sampleClass = {
	kCurrentREALControlVersion,
	"SampleClass",
	nil,
	sizeof(sampleClassData),
	0,
	(REALproc) sampleClassConstructor, (REALproc) sampleClassDestructor,
	nil, 0, // sampleClassProperties, sizeof(sampleClassProperties) / sizeof(REALproperty),
	sampleClassMethods, sizeof(sampleClassMethods) / sizeof(REALmethodDefinition),
	nil, 0, //sampleClassEvents, sizeof(sampleClassEvents) / sizeof(REALevent),
	nil, 0
};


// ---------------------------------
// ----- Plugin Initialization -----
// ---------------------------------

void PluginInit (void)
{
}

void PluginExit (void)
{
}

void PluginEntry (void)
{
	InitTTsPluginSupport ();

	#if TARGET_CPU_68K
		PrepareCallback ();
	#endif
	
//	REALRegisterClass (&sampleClass);

	for (int i = 0; i < sizeof(pluginMethodArray) / sizeof(REALmethodDefinition); ++i) {
		REALRegisterMethod (&pluginMethodArray[i]);
	}
}
