/*
 * TT's Debugging Plugin
 *
 * Provides debugging functions (logging) for Mac OS and Windows.
 *
 * For REALBasic 2.1.2
 *
 * v1.2 (1.2 supports OS X by writing to the Console)
 */

//--------------------
// Your includes here
//--------------------

#ifndef WIN32
	#include <Gestalt.h>
	#include "dcon.h"
#endif

//-------------------------
// RB header includes here
//-------------------------

#include "rb_plugin.h"

//-------------------
// global RB methods
//-------------------

static int isOSX = 0;
static int gLogEnabled = 0;

static void logStr (REALstring msg)
{
	if (gLogEnabled) {
		#ifdef WIN32
			OutputDebugString (msg->CString());
			OutputDebugString ("\n");
		#else
			if (isOSX) {
				DebugStr (msg->PString());
			} else {
				dprintf (msg->CString());
				dprintf ("\n");
			}
		#endif
	}
}

static void logOpen (REALstring fn)
{
	#ifndef WIN32
		if (not isOSX) {
			dopen (fn->CString());
		}
	#endif
}

static void logEna (Boolean b)
{
	gLogEnabled = b;
}

static long freeMem ()
{
	#ifdef WIN32
		return 0;
	#else
		if (isOSX) {
			return 0;
		} else {
			return FreeMem ();
		}
	#endif
}

static long getVers ()
{
	return 0x010200;
}

static REALmethodDefinition pluginMethodArray[] = {
	{ (REALproc) getVers, REALnoImplementation, "TTsDebuggingVersion()" },
	{ (REALproc) freeMem, REALnoImplementation, "AvailableMemory() as Integer" },
	{ (REALproc) logStr, REALnoImplementation, "Log(msg as String)" },
	{ (REALproc) logEna, REALnoImplementation, "LogEnable(activate as Boolean)" },
	{ (REALproc) logOpen, REALnoImplementation, "LogToFile(filename as String)" },
};


// ---------------------------------
// ----- Plugin Initialization -----
// ---------------------------------


void PluginEntry (void)
{
	#ifndef WIN32
		long response;
		int err = Gestalt (gestaltSystemVersion, &response);
		isOSX = not err and (response >= 0x01000);
	#endif

	for (int i = 0; i < sizeof(pluginMethodArray) / sizeof(REALmethodDefinition); ++i) {
		REALRegisterMethod (&pluginMethodArray[i]);
	}

	gLogEnabled = 1;
}
