/*
 * MountVol Interceptor.cp
 *
 * Freeware, by Thomas Tempelmann, http://www.tempel.org/rb/
 */

#include <MixedMode.h>
#include "rb_plugin.h"
#include "MountVol Interceptor.h"


typedef	long RBinteger;
typedef	Boolean RBboolPar;


#define useWNE 0

#if useWNE

	// intercept WNE calls, look for diskEvt events
	
	const ProcInfoType wneProcInfo =
		kPascalStackBased |
		RESULT_SIZE(SIZE_CODE(sizeof(Boolean))) |
		STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(short))) |
		STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(EventRecord*))) |
		STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(UInt32))) |
		STACK_ROUTINE_PARAMETER(4, SIZE_CODE(sizeof(RgnHandle)));
		
	#ifdef powerc
		typedef UniversalProcPtr wneProc;
	#else
		typedef pascal Boolean (*wneProc) (short eventMask, EventRecord* theEvent, UInt32 sleep, RgnHandle mouseRgn);
	#endif

	static	UniversalProcPtr gNewWNE;
	static	wneProc gOldWNE;

#else

	// intercept MountVol calls
	
	const ProcInfoType mountVolProcInfo =
		kRegisterBased |
		RESULT_SIZE(SIZE_CODE(sizeof(OSErr))) |
		REGISTER_ROUTINE_PARAMETER(1, kRegisterA0, SIZE_CODE(sizeof(void*)));

	#ifdef powerc
		typedef UniversalProcPtr mountVolProc;
	#else
		// causes compiler error: #pragma parameter __D0 (*mountVolProc)(__A0)
		typedef pascal OSErr (*mountVolProc) (VolumeParam *);
	#endif

	static	UniversalProcPtr gNewMountVol;
	static	mountVolProc gOldMountVol;

#endif

static	Boolean gHooksInstalled;

#ifdef powerc
	#define __far
#else
	#define __far far
#endif

#define cMaxDrv 32
static	__far	int gDiskInserted;
static	__far	short gDriveStack[cMaxDrv+1];


#if useWNE
	static pascal Boolean wneHook (EventMask eventMask, EventRecord* theEvent, UInt32 sleep, RgnHandle mouseRgn)
	{
		Boolean b = CALL_FOUR_PARAMETER_UPP (gOldWNE, wneProcInfo, eventMask, theEvent, sleep, mouseRgn);
		if (theEvent->what == diskEvt) {
			// a disk was inserted
			if (gDiskInserted < cMaxDrv) {
				gDriveStack[gDiskInserted++] = theEvent->what & 0xFFFF;
			}
		}
		return b;
	}
#else
	#ifndef powerc
		#pragma parameter __D0 mountVolHook(__A0)
	#endif
	static pascal OSErr mountVolHook (VolumeParam *pb)
	{
		// MountVol is called
		if (gDiskInserted < cMaxDrv) {
			gDriveStack[gDiskInserted++] = pb->ioVRefNum;
		}
		return extFSErr;
	}
#endif

static void removeHook ()
{
	if (gHooksInstalled) {
		#if useWNE
			if (gOldWNE) SetToolTrapAddress ((UniversalProcPtr)gOldWNE, _WaitNextEvent);
			gOldWNE = nil;
		#else
			if (gOldMountVol) SetOSTrapAddress ((UniversalProcPtr)gOldMountVol, _MountVol);
			gOldMountVol = nil;
		#endif
		gHooksInstalled = false;
	}
}

static void installHook ()
{
	if (not gHooksInstalled) {
		// install the WNE hook
		#if useWNE
			gOldWNE = (wneProc) NGetTrapAddress (_WaitNextEvent, 1);
			SetToolTrapAddress (gNewWNE, _WaitNextEvent);
		#else
			gOldMountVol = (mountVolProc) NGetTrapAddress (_MountVol, 0);
			SetOSTrapAddress (gNewMountVol, _MountVol);
		#endif
		gHooksInstalled = true;
	}
	gDiskInserted = 0;
}

static RBinteger getInsert ()
{
	if (not gHooksInstalled or gDiskInserted <= 0) return 0;
	short drv = gDriveStack[--gDiskInserted];
	return drv;
}

static void intercept (RBboolPar active)
{
	if (active) {
		installHook ();
	} else {
		removeHook ();
	}
}

static REALmethodDefinition PluginMethodArray[] = {
	{ (REALproc)getInsert, REALnoImplementation, "GetInsertedDriveNum() as Integer)" },
	{ (REALproc)intercept, REALnoImplementation, "InterceptDiskInsertionEvents(active as Boolean)" },
};

void ExitMountVolInterceptor ()
{
	removeHook ();
}

void InitMountVolInterceptor ()
{
	// allocate the ProcPtrs
	#if useWNE
		gNewWNE = NewRoutineDescriptor ((ProcPtr)wneHook, wneProcInfo, GetCurrentISA());
		gOldWNE = (wneProc)nil;
	#else
		gNewMountVol = NewRoutineDescriptor ((ProcPtr)mountVolHook, mountVolProcInfo, GetCurrentISA());
		gOldMountVol = (mountVolProc)nil;
	#endif

	gHooksInstalled = false;

	for (int i = 0; i < sizeof(PluginMethodArray) / sizeof(REALmethodDefinition); ++i) {
		REALRegisterMethod (&PluginMethodArray[i]);
	}
}

// EOF
