/*
 * Plugin for REALbasic 2
 *
 * This code is freeware, you may use and extend it as you like.
 *
 * The accompanying project file was created using CodeWarrior Pro 3.
 *
 * It was written on Oct 23, 99 by Thomas Tempelmann.
 *
 * More RB-related stuff can be found here: <http://www.tempel.org/rb/>
 *
 * Enjoy and contribute!
 */

#include <Disks.h>
#include <DriverGestalt.h>
#include <Devices.h>
#include <string.h>
#include <stdio.h>
#include <stddef.h>
#include <SetupA4.h>
#include "FileLib.h"
#include "MoreFilesExtras.h"
//#include "MountVol Interceptor.h"
#include "PartInfoRec Utils.h"
// we do this all outselves better: #include "MoreDisks.h"
#include "rb_plugin.h"


typedef	long RBinteger;
typedef	long RBboolean;


//-----------------------
//   support functions
//-----------------------

typedef unsigned short Word;

static Boolean gHasIconServices = false, gTestedIconServices = false, gMustTermIconServices = false;

static void MyInitIconServices ();

static void pstrcpy (unsigned char* d, const unsigned char* s)
{
	memcpy (d, s, ((unsigned short)s[0])+1);
}


static short FindVolByDrive (short drvNum)
{
	HVolumeParam pb;
	short err;

/* kein Loop ntig hier, glaube ich:
	short idx = 1;
	do {
		pb.ioVolIndex = idx;
		pb.ioNamePtr = NULL;
		err = PBHGetVInfoSync ((HParamBlockRec*)&pb);
		if (err) break;
		if (pb.ioVDrvInfo == drvNum) {
			return pb.ioVRefNum;
		}
		++idx;
	} while (true);
*/
// stattdessen:
	pb.ioVolIndex = 0;
	pb.ioNamePtr = NULL;
	pb.ioVRefNum = drvNum;
	err = PBHGetVInfoSync ((HParamBlockRec*)&pb);
	if (!err) {
		return pb.ioVRefNum;
	}

	return 0;
}

static Boolean driverSupportsDriverGestalt (short dRefNum)
{
	DCtlHandle dCtlHdl = GetDCtlEntry (dRefNum);
	return dCtlHdl and ((*dCtlHdl)->dCtlFlags & kmDriverGestaltEnableMask) != 0;
}

static Boolean getMediaName (short driveNum, Str255 where)
// Returns the "where" string for a drive
{
	OSErr err;
	CntrlParam pb = {0};
	pb.ioCRefNum = DriveDRefNum (driveNum);
	pb.ioVRefNum = driveNum;
	where[0] = 0;
	if (driverSupportsDriverGestalt (pb.ioCRefNum)) {
		pb.csCode = kDriverGestaltCode;
		((DriverGestaltParam *)(&pb))->driverGestaltSelector = kdgMediaName;
		err = PBStatusSync ((ParamBlockRec *)&pb);
		if (!err) {
			pstrcpy (where, (StringPtr)((DriverGestaltParam *)(&pb))->driverGestaltResponse);
		}
	} else {
		err = -1;
	}
	if (err) {
		pb.csCode = 21;
		err = PBControlSync((ParamBlockRec *)&pb);
		if (!err) {
			// csParam contains pointer to b+w icon followed by pascal string
			StringPtr s = *(StringPtr*)pb.csParam + 256;
			pstrcpy (where, s);
		}
	}
	return !err;
}

static OSErr driverDeviceID (short dRefNum, OSType *interface, UInt32 *devRef)
{
	*interface = 0; *devRef = 0;
	CntrlParam params;
	memset (&params, 0, sizeof(params));
	params.csCode = kDriverGestaltCode;
	params.ioCRefNum = dRefNum;
	params.ioVRefNum = 0;

	*(OSType *)&params.csParam[0] = kdgInterface;
	OSErr err = PBStatusSync((ParamBlockRec *)&params);
	if (!err) {
		*interface = *(OSType*)(&params.csParam[2]);
	}

	*(OSType *)&params.csParam[0] = kdgDeviceReference;
	err = PBStatusSync((ParamBlockRec *)&params);
	if (!err) {
		*devRef = ((DriverGestaltDeviceReferenceResponse *)(&params.csParam[2]))->devRef;
	}
	
	return err;
}

static OSErr driverDeviceType(short dRefNum, OSType *devType)
{
	*devType = 0;
	CntrlParam params;
	memset (&params, 0, sizeof(params));
	params.csCode = kDriverGestaltCode;
	params.ioCRefNum = dRefNum;
	params.ioVRefNum = 0;
	*(OSType *)&params.csParam[0] = kdgDeviceType;
	OSErr err = PBStatusSync((ParamBlockRec *)&params);
	if (!err) {
		*devType = *(OSType*)(&params.csParam[2]);
	}
	return err;
}

static Boolean getDriveMediaIcon (short driveNum, IconSuiteRef *iconHdl)
// Attention: caller must call "DisposeIconSuite (iconHdl, true)" when done with the icon
{
	Boolean success = false;
	OSErr err;

	MyInitIconServices ();
	
	if (gHasIconServices) {

			// use Icon Services to get the volume icon.

		short vRefNum = FindVolByDrive (driveNum);
		if (vRefNum) {
			short label;
			IconRef iconRef;
			FSSpec spec = {vRefNum, fsRtDirID, "\p"};
			err = GetIconRefFromFile (&spec, &iconRef, &label);
			if (!err) {
				IconFamilyHandle ih = nil;
				err = IconRefToIconFamily (iconRef, svAllLargeData, &ih);
				if (!err) {
					*iconHdl = nil;
					err = IconFamilyToIconSuite (ih, svAllLargeData, iconHdl);
					if (!err && *iconHdl) {
						success = true;
					}
				}
				ReleaseIconRef(iconRef);
			}
		}
	}

	if (!success) {

			// If Icon Services failed (either too old MacOS or volume not mounted),
			// then get the driver's icons (I could also try to get the custom icon
			// from the volume, but I want to finish this all one day ...)

		DrvQElPtr dqe = FindDrvQ (driveNum);

		{
			// try to get a new color icon ('icns')
			DriverGestaltParam pb = {0};
			pb.ioCRefNum = dqe->dQRefNum;
			pb.ioVRefNum = dqe->dQDrive;
			pb.csCode = kDriverGestaltCode;
			pb.driverGestaltSelector = kdgMediaIconSuite;
			err = PBStatusSync ((ParmBlkPtr)&pb);
			if (!err) {
				IconFamilyPtr src = (IconFamilyPtr)(Ptr)pb.driverGestaltResponse;
				long l = src->resourceSize;
				IconFamilyHandle familyHdl;
				err = PtrToHand ((Ptr)src, &(Handle)familyHdl, l);
				if (!err) {
					*iconHdl = nil;
					err = IconFamilyToIconSuite (familyHdl, svAllLargeData, iconHdl);
					if (!err && *iconHdl) {
						success = true;
					}
					DisposeHandle ((Handle)familyHdl);
				}
			}
		}
	
		if (!success) {

				// last resort: get the driver's b&w icon

			DrvQElPtr dqe = FindDrvQ (driveNum);
			CntrlParam cntrlParms = {0};
			cntrlParms.ioCRefNum = dqe->dQRefNum;
			cntrlParms.ioVRefNum = dqe->dQDrive;
			cntrlParms.csCode = 22;		// 21: phys icon, 22: media icon
			err = PBControlSync ((ParmBlkPtr)&cntrlParms);
			if (!err) {
				Handle hdl;
				err = PtrToHand (*(Ptr*)&cntrlParms.csParam, &hdl, kLargeIconSize);
				if (!err) {
					err = NewIconSuite ((Handle*)iconHdl);
					if (!err) {
						err = AddIconToSuite (hdl, *(Handle*)iconHdl, 'ICN#');
						if (!err) {
							success = true;
						}
					}
				}
			}
		}
	}
	
	return success;
}

//-----------------------
// global plugin methods
//-----------------------

static RBinteger GetDriveNumFromFolderItem (REALfolderItem item)
{
	OSErr err;
	FSSpec spec;
	if (REALFSSpecFromFolderItem (&spec, item)) {
		HVolumeParam pb;
		pb.ioVolIndex = 0;
		pb.ioNamePtr = nil;
		pb.ioVRefNum = spec.vRefNum;
		err = PBHGetVInfoSync ((HParmBlkPtr)&pb);
		if (!err) {
			return pb.ioVDrvInfo;
		}
	}
	return 0;
}

static RBinteger GetDriveCount ()
{
	long n = 0;
	DrvQElPtr qep = (DrvQElPtr)(GetDrvQHdr()->qHead);
	while (qep) {
		++n;
		qep = (DrvQElPtr)qep->qLink;
	}
	return n;
}

static RBinteger GetDriveNumByIndex (RBinteger index)
{
	DrvQElPtr qep = (DrvQElPtr)(GetDrvQHdr()->qHead);
	if (index >= 0) {
		while (qep) {
			if (index == 0) return qep->dQDrive;
			--index;
			qep = (DrvQElPtr)qep->qLink;
		}
	}
	return 0;
}

static RBinteger GetBlkCount (RBinteger driveNum)
{
	DrvQElPtr qep = FindDrvQ (driveNum);
	if (qep) {
		return (long)(UInt16)qep->dQDrvSz + (qep->qType==1 ? ((long)(UInt16)qep->dQDrvSz2)<<16 : 0);
	}
	return 0;
}

static RBboolean DriveIsLocked (RBinteger driveNum)
{
	DrvQElPtr qep = FindDrvQ (driveNum);
	if (qep) {
		DrvSts2* drvStat = (DrvSts2*) ((Ptr)qep - offsetof(DrvSts2,qLink));
		return (drvStat->writeProt & 0x0080) != 0;
	}
	return false;
}

static RBinteger ReadBlksRaw (RBinteger driveNum, RBinteger blkSize, RBinteger startBlk, RBinteger blks, RBinteger dataPtr)
{
	DrvQElPtr qep = FindDrvQ (driveNum);
	if (qep) {
		CntrlParam pb = {0};
		pb.ioCRefNum = DriveDRefNum (driveNum);
		pb.ioVRefNum = driveNum;
		pb.csCode = 127;
		*(long*)&pb.csParam[0] = 0;	// block offset
		*(long*)&pb.csParam[2] = blkSize;
		*(long*)&pb.csParam[4] = startBlk;
		*(long*)&pb.csParam[6] = blks;
		*(long*)&pb.csParam[8] = dataPtr;
		PBControlSync ((ParamBlockRec *)&pb);
		return pb.ioResult;
	}
	return nsDrvErr;
}


static Boolean hasWideSupport (short dRefNum)
{
	DriverGestaltParam pb;
	pb.ioVRefNum = 0;
	pb.ioCRefNum = dRefNum;
	pb.csCode = kDriverGestaltCode;
	pb.driverGestaltSelector = kdgWide;
	OSErr err = PBStatusSync ((ParmBlkPtr)&pb);
	return !err && *(Boolean*)&pb.driverGestaltResponse;
}

static XIOParam	xiop;
static IOParam	&iop = (IOParam&)xiop;

static OSErr DeviceRW (short mode, short cacheOptions, short drive, short refnum, long startBlk512, long bytes, Ptr buffer)
{
	if (iop.ioResult == 1) return fsmBusyFFSErr;
	OSErr err = paramErr;
	memset (&xiop, 0, sizeof (xiop));
	iop.ioVRefNum = drive;
	iop.ioRefNum = refnum;
	iop.ioBuffer = buffer;
	iop.ioReqCount = bytes;
	iop.ioPosMode = (short)(fsFromStart | cacheOptions);
	iop.ioPosOffset = (long)(startBlk512 << 9);
	if (startBlk512 >= 0x00400000) {	// >= 2GB?
		if (hasWideSupport (refnum)) {
			iop.ioPosMode |= (1 << kWidePosOffsetBit);
			SInt64 start = (SInt64)startBlk512 << 9;
			xiop.ioWPosOffset.lo = (long)start;
			xiop.ioWPosOffset.hi = (long)(start >> 32);
		} else if (startBlk512 < kMaximumBlocksIn4GB) {
			// let's just hope that the driver sees the offset as an _unsigned_ long
		} else {
			// start offset overflow
			return posErr;
		}
	}
	switch (mode) {
	case 0:
		err = PBReadSync ((ParamBlockRec*)&iop);
		break;
	case 1:
		err = PBReadAsync ((ParamBlockRec*)&iop);
		break;
	case 2:
		err = PBWriteSync ((ParamBlockRec*)&iop);
		break;
	case 3:
		err = PBWriteAsync ((ParamBlockRec*)&iop);
		break;
	}
	err = iop.ioResult;
	if (!err && iop.ioActCount < iop.ioReqCount) {
		err = ioErr;
	}
	return err;
}

static RBinteger ReadBlks (RBinteger driveNum, RBinteger startBlk, RBinteger blks512, Ptr dataPtr, RBboolean noCache)
{
	DrvQElPtr qep = FindDrvQ (driveNum);
	if (qep) {
		return DeviceRW (0, noCache?noCacheMask:0, driveNum, qep->dQRefNum, startBlk, blks512<<9, dataPtr);
	}
	return nsDrvErr;
}

static RBinteger ReadBlksAsync (RBinteger driveNum, RBinteger startBlk, RBinteger blks512, Ptr dataPtr, RBboolean noCache)
{
	DrvQElPtr qep = FindDrvQ (driveNum);
	if (qep) {
		return DeviceRW (1, noCache?noCacheMask:0, driveNum, qep->dQRefNum, startBlk, blks512<<9, dataPtr);
	}
	return nsDrvErr;
}

static RBinteger WriteBlks (RBinteger driveNum, RBinteger startBlk, RBinteger blks512, Ptr dataPtr, RBboolean noCache)
{
	DrvQElPtr qep = FindDrvQ (driveNum);
	if (qep) {
		return DeviceRW (2, noCache?noCacheMask:0, driveNum, qep->dQRefNum, startBlk, blks512<<9, dataPtr);
	}
	return nsDrvErr;
}

static RBinteger WriteBlksAsync (RBinteger driveNum, RBinteger startBlk, RBinteger blks512, Ptr dataPtr, RBboolean noCache)
{
	DrvQElPtr qep = FindDrvQ (driveNum);
	if (qep) {
		return DeviceRW (3, noCache?noCacheMask:0, driveNum, qep->dQRefNum, startBlk, blks512<<9, dataPtr);
	}
	return nsDrvErr;
}

static RBinteger DriveIOResult (RBinteger driveNum)
{
	DrvQElPtr qep = FindDrvQ (driveNum);
	if (qep) {
		return iop.ioResult;
	}
	return nsDrvErr;
}

static RBinteger driveEject (RBinteger driveNum)
{
	DrvQElPtr qep = FindDrvQ (driveNum);
	if (qep) {
		qep->dQFSID = 0;
		return Eject (nil, driveNum);
	}
	return nsDrvErr;
}

static RBinteger fDriveFlags (RBinteger driveNum)
{
	DrvQElPtr qep = FindDrvQ (driveNum);
	if (qep) {
		return *(long*)((Ptr)qep-4);
	}
	return 0;
}

static RBinteger DriveFSID (RBinteger driveNum)
{
	DrvQElPtr qep = FindDrvQ (driveNum);
	if (qep) {
		return (long)qep->dQFSID & 0x0000FFFF;
	}
	return -1;
}

static DrvQElPtr DriveQElPtr (RBinteger driveNum)
{
	return FindDrvQ (driveNum);
}

static RBboolean myDriveSupportsFileExchange (RBinteger drv)
{
	return DoDriverGestalt (drv, kdgAPI, nil) == noErr;
}

static RBinteger driveDRefNum (RBinteger driveNum)
{
	return DriveDRefNum (driveNum);
}

static OSErr SafeHoldMemory (void *start, ByteCount length)
{
	OSErr err;
	err = noErr;
	if (GetOSTrapAddress(_HWPriv) != GetToolTrapAddress(_Unimplemented)) {
		err = HoldMemory(start, length);
	}
	if (err == paramErr) {
		err = noErr;
	}
	return err;
}

static OSErr SafeUnholdMemory (void *start, ByteCount length)
{
	OSErr err;
	err = noErr;
	if (GetOSTrapAddress(_HWPriv) != GetToolTrapAddress(_Unimplemented)) {
		err = UnholdMemory(start, length);
	}
	if (err == paramErr) {
		err = noErr;
	}
	return err;
}

static RBinteger CloneDrive (RBinteger driveToClone, RBinteger firstBlock, RBinteger sizeInBlocks)
// note: this function apparently is not supported by the Apple CD driver :(
{
/* alternative:
	SInt16 newDrive2;
	OSErr err = MoreCreateNewDriveQueueElement(driveToClone, firstBlock, sizeInBlocks, &newDrive2);
	if (err) return err;
	return newDrive2;
*/

	OSErr err;
	CntrlParam pb;
	DrvQElPtr drvQEl;
	short newDrive, dRefNum;

	err = noErr;
	if (!DriveSupportsFileExchange (driveToClone)) {
		err = controlErr;
	}

	if (!err) {
		drvQEl = FindDrvQ(driveToClone);
		if (!drvQEl) err = nsDrvErr;
	}

	// clone the driveQElem
	if (!err) {
		dRefNum = drvQEl->dQRefNum;
		err = SafeHoldMemory (&drvQEl, sizeof(drvQEl));
		if (!err) {
			pb.ioVRefNum = driveToClone;
			pb.ioCRefNum = dRefNum;
			pb.csCode = kGetADrive;
			*((DrvQElPtr **) &pb.csParam[0]) = &drvQEl;
			err = PBControlSync ((ParmBlkPtr) &pb);
			if (!err) {
				newDrive = drvQEl->dQDrive;
			}
			(void) SafeUnholdMemory (&drvQEl, sizeof(drvQEl));
		}
	}

	// Now retarget the new drive to the partition on the
	// disk specified by firstBlock and sizeInBlocks.
	if (!err) {
		err = TTsSetDrivePartition (newDrive, false, firstBlock, sizeInBlocks);
	}

	if (err) return err;
	return newDrive;
}

static RBinteger SetDrvPartition (RBinteger drive, RBinteger firstBlock, RBinteger sizeInBlocks)
{
	return TTsSetDrivePartition (drive, false, firstBlock, sizeInBlocks);
}

static RBinteger PartStart (RBinteger drive)
{
	partInfoRec part = {0};
	OSErr err = TTsGetPartInfoRec (drive, false, &part);
	if (err) return err;
	return part.physPartitionLoc;
}

static RBinteger DeviceID (RBinteger drive)
{
	partInfoRec part = {0};
	OSErr err = TTsGetPartInfoRec (drive, false, &part);
	if (err) return -1;
	return *(RBinteger*)&part.SCSIID;
}

static REALstring DriverName (RBinteger drv)
{
	Str255 name;
	if (GetDriverName (DriveDRefNum(drv), name) != noErr) {
		sprintf ((char*)name, "Driver %d", (int)DriveDRefNum(drv));
		c2pstr ((char*)name);
	}
	return REALBuildString ((char*)&name[1], name[0]);
}

static REALstring MediaName (RBinteger drv)
{
	Str255 where;
	if (!getMediaName (drv, where)) {
		return DriverName (drv);
	}
	return REALBuildString ((char*)&where[1], where[0]);
}


static RBboolean DrawIcon (RBinteger drv, REALgraphics gr)
{
	IconSuiteRef iconHdl;
	if (getDriveMediaIcon (drv, &iconHdl)) {
		REALSelectGraphics (gr);
		Rect r;
		SetRect (&r, 0, 0, 32, 32);
		/*
			CGrafPtr origWorld;
			GDHandle origDev;
			GetGWorld (&origWorld, &origDev);
			SetGWorld (world, nil);
		*/
		PlotIconSuite (&r, atNone, ttNone, iconHdl);
		/*
			p = REALBuildPictureFromGWorld (world, true);
			SetGWorld (origWorld, origDev);
		*/
		DisposeIconSuite (iconHdl, true);
		return true;
	}
	return false;
}

static REALmethodDefinition PluginMethodArray[] = {
	{ (REALproc)GetDriveNumFromFolderItem, REALnoImplementation, "DriveRefNumFromVol(v as FolderItem) as Integer" },
	{ (REALproc)GetDriveCount, REALnoImplementation, "DriveCount() as Integer" },
	{ (REALproc)GetDriveNumByIndex, REALnoImplementation, "DriveRefNumFromIndex(index as Integer) as Integer" },
	{ (REALproc)DriveIsLocked, REALnoImplementation, "DriveIsLocked(index as Integer) as Boolean" },
	{ (REALproc)GetBlkCount, REALnoImplementation, "DriveBlockCount(driveNum as Integer) as Integer" },
	{ (REALproc)ReadBlks, REALnoImplementation, "DriveReadBlocks(driveNum as Integer, startBlk512 as Integer, blk512Cnt as Integer, dataPtr as Integer, noNeedToCache as Boolean) as Integer" },
	{ (REALproc)ReadBlksAsync, REALnoImplementation, "DriveReadBlocksAsync(driveNum as Integer, startBlk512 as Integer, blk512Cnt as Integer, dataPtr as Integer, noNeedToCache as Boolean) as Integer" },
	{ (REALproc)ReadBlksRaw, REALnoImplementation, "DriveReadBlocksRaw(driveNum as Integer, blkSize as Integer, startBlk as Integer, blkCnt as Integer, dataPtr as Integer) as Integer" },
	{ (REALproc)WriteBlks, REALnoImplementation, "DriveWriteBlocks(driveNum as Integer, startBlk512 as Integer, blk512Cnt as Integer, dataPtr as Integer, noNeedToCache as Boolean) as Integer" },
	{ (REALproc)WriteBlksAsync, REALnoImplementation, "DriveWriteBlocksAsync(driveNum as Integer, startBlk512 as Integer, blk512Cnt as Integer, dataPtr as Integer, noNeedToCache as Boolean) as Integer" },
	{ (REALproc)DriveIOResult, REALnoImplementation, "DriveIOResult(driveNum as Integer) as Integer" },
	{ (REALproc)driveEject, REALnoImplementation, "DriveEject(driveNum as Integer) as Integer" },
	{ (REALproc)fDriveFlags, REALnoImplementation, "DriveFlags(driveNum as Integer) as Integer" },
	{ (REALproc)DriveFSID, REALnoImplementation, "DriveFSID(driveNum as Integer) as Integer" },
	{ (REALproc)driveDRefNum, REALnoImplementation, "DriveDRefNum(driveNum as Integer) as Integer" },
	{ (REALproc)DriveQElPtr, REALnoImplementation, "DriveQElemPtr(driveNum as Integer) as Integer" },
	{ (REALproc)CloneDrive, REALnoImplementation, "DriveClone(driveNum as Integer, startBlk512 as Integer, blk512Cnt as Integer) as Integer" },
	{ (REALproc)SetDrvPartition, REALnoImplementation, "DriveSetPartition(driveNum as Integer, startBlk512 as Integer, blk512Cnt as Integer) as Integer" },
	{ (REALproc)PartStart, REALnoImplementation, "DrivePartitionStart(driveNum as Integer) as Integer" },
	{ (REALproc)DeviceID, REALnoImplementation, "DriveDeviceIdent(driveNum as Integer) as Integer" },
	{ (REALproc)MediaName, REALnoImplementation, "DriveMediaName(driveNum as Integer) as String" },
	{ (REALproc)myDriveSupportsFileExchange, REALnoImplementation, "DriveSupportsFileExchange(driveNum as Integer) as Boolean" },
	{ (REALproc)DriverName, REALnoImplementation, "DriveDriverName(driveNum as Integer) as String" },
	{ (REALproc)DrawIcon, REALnoImplementation, "DriveDrawIcon(driveNum as Integer, into as Graphics) as Boolean" },
};

static UniversalProcPtr gOldHandler;

static pascal void CleanupAndExit (void)
{
	EnterCallback();
//	ExitMountVolInterceptor ();
	SetToolTrapAddress (gOldHandler, _ExitToShell);
	#if TARGET_CPU_68K
		if (gMustTermIconServices) {
			gMustTermIconServices = false;
			IconServicesTerminate ();
		}
	#endif
	ExitCallback();
	ExitToShell ();
}

static void InitQuit ()
{
	gOldHandler = NGetTrapAddress (_ExitToShell, 1);
	SetToolTrapAddress (NewRoutineDescriptor ((ProcPtr)CleanupAndExit, kPascalStackBased, GetCurrentISA()), _ExitToShell);
}

static void MyInitIconServices ()
{
	if (!gTestedIconServices) {
		gTestedIconServices = true;
		long l = 0;
		Gestalt (gestaltIconUtilitiesAttr, &l);
		gHasIconServices = (l & (1<<gestaltIconUtilitiesHasIconServices)) != 0;
		#if TARGET_CPU_68K
			if (gHasIconServices) {
				static CFragSystem7InitBlock ISFrag = {0};
				IconServicesInit (&ISFrag);
				gMustTermIconServices = true;
				InitQuit ();
			}
		#endif
	}
}

void PluginEntry (void)
{
	// Save the global A4 register for the hook/filter functions
	PrepareCallback ();

	InitQuit ();
	
	for (int i = 0; i < sizeof(PluginMethodArray) / sizeof(REALmethodDefinition); ++i) {
		REALRegisterMethod (&PluginMethodArray[i]);
	}
	
	iop.ioResult = 0;
	
//	InitMountVolInterceptor ();
}
