/*
 * TT's EventMonitor Plugin for REALbasic 2
 *
 * This code is freeware, you may use and extend it as you like.
 *
 * The accompanying project file was created using CodeWarrior Pro 3.
 *
 * It was written on July 15, 99 by Thomas Tempelmann.
 *
 * More RB-related stuff can be found here: <http://www.tempel.org/rb/>
 *
 * Enjoy and contribute!
 */

#if TARGET_CPU_68K
	#include <SetupA4.h>
#endif
#include <MixedMode.h>
#include <string.h>

#include "rb_plugin.h"


#define UseRBsFilter 1	// 0 -> patch the WNE trap

//----------------------
// forward declarations
//----------------------

#if TARGET_WIN32
	typedef unsigned short UInt16;
	typedef unsigned long UInt32;
	typedef long SInt32;
	typedef struct EventRecord {
		short what;
		long message;
		long when;
		Point where;
		short modifiers;
	} EventRecord;
#endif

extern REALevent eventFilterEvents[];
static void theEventFilterHandler (EventRecord *event, long param);

//-------------------
// support functions
//-------------------

#if !UseRBsFilter

	/* this part is still under construction and may be used later for supporting global Event Filters:
	*/

	const ProcInfoType wneProcInfo =
		kPascalStackBased |
		RESULT_SIZE(SIZE_CODE(sizeof(Boolean))) |
		STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(short))) |
		STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(EventRecord*))) |
		STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(UInt32))) |
		STACK_ROUTINE_PARAMETER(4, SIZE_CODE(sizeof(RgnHandle)));
		
	#ifdef powerc
		typedef UniversalProcPtr wneProc;
	#else
		typedef pascal Boolean (*wneProc) (short eventMask, EventRecord* theEvent, UInt32 sleep, RgnHandle mouseRgn);
	#endif

	far static	UniversalProcPtr gOldExitToShell, gNewExitToShell, gNewWNE;
	far static	wneProc gOldWNE;

	static pascal Boolean wneHook (EventMask eventMask, EventRecord* theEvent, UInt32 sleep, RgnHandle mouseRgn)
	{
		// +++ I'm not sure, but I might need to put this call _behind_ the gOldWNE call:
		theEventFilterHandler (theEvent, 0);
		
		Boolean b = CALL_FOUR_PARAMETER_UPP (gOldWNE, wneProcInfo, eventMask, theEvent, sleep, mouseRgn);
		return b;
	}

	static void deInstall ()
	{
		if (gOldExitToShell) SetToolTrapAddress (gOldExitToShell, _ExitToShell);
		gOldExitToShell = nil;
		if (gOldWNE) SetToolTrapAddress ((UniversalProcPtr)gOldWNE, _WaitNextEvent);
		gOldWNE = nil;
	}

	static pascal void quitApp (void)
	{
		#if TARGET_68K
			EnterCallback();
		#endif
		deInstall ();
		#if TARGET_68K
			ExitCallback();
		#endif
		ExitToShell ();
	}

	static void installWNEHook ()
	{
		// allocate the ProcPtrs
		gNewExitToShell = NewRoutineDescriptor ((ProcPtr)quitApp, kPascalStackBased, GetCurrentISA());
		gNewWNE = NewRoutineDescriptor ((ProcPtr)wneHook, wneProcInfo, GetCurrentISA());
		gOldWNE = (wneProc)nil;
		gOldExitToShell = (UniversalProcPtr)nil;

		// install the WNE hook
		gOldWNE = (wneProc) NGetTrapAddress (_WaitNextEvent, 1);
		SetToolTrapAddress (gNewWNE, _WaitNextEvent);
		
		// install a hook onto ExitToShell() in order to restore everything in case the app crashes
		gOldExitToShell = NGetTrapAddress (_ExitToShell, 1);
		SetToolTrapAddress (gNewExitToShell, _ExitToShell);
	}

#endif

//--------------------------
// plugin classes & methods
//--------------------------

extern REALclassDefinition eventFilterClass;	// forward declaration

typedef void (*NotificationProc) (REALobject instance);

static EventRecord currentEvent;

typedef struct eventFilterData {
	// store your class properties here
	eventFilterData *next;
	NotificationProc action;
	REALobject theInstance;
	long eventMask;
} eventFilterData;

static eventFilterData *root, *last;

static void theEventFilterHandler (EventRecord *event, long)
{
	#if TARGET_68K
		EnterCallback();
	#endif

	currentEvent = *event;
	
	eventFilterData *me = root;
	if (me != nil) {
		do {
			if (((1L << event->what) & me->eventMask) != 0) {
				me->action ((REALobject)me->theInstance);
			}
			me = me->next;
		} while (me != nil);
		*event = currentEvent;
	}
	
	#if TARGET_68K
		ExitCallback();
	#endif
}

static void eventFilterConstructor (REALobject instance)
{
	ClassData (eventFilterClass, instance, eventFilterData, me);
	
	NotificationProc fp = (NotificationProc) REALGetEventInstance((REALcontrolInstance)instance, &eventFilterEvents[0]);
	me->action = fp;	// might be NULL
	me->eventMask = 0;	// let thru no events by default
	me->theInstance = instance;
	
	if (fp) {
		// add this object instance to the list of to be called action handlers
		if (last != nil) last->next = me;
		last = me;
		if (root == nil) root = me;
	}
}

static void eventFilterDestructor (REALobject instance)
{
	ClassData (eventFilterClass, instance, eventFilterData, me);
	
	// find us in the list and remove us from it
	eventFilterData **p = &root;
	while (*p != nil) {
		if (*p == me) {
			if (last == me) last = me->next;
			*p = me->next;
			break;
		}
		p = &((*p)->next);
	}
}

/* not supported yet:
static void eventFilterConstructorWithBoolean (REALobject instance, Boolean global)
{
	ClassData (eventFilterClass, instance, eventFilterData, me);
	me->isGlobal = global;
}

static long eventFilterIsGlobal (REALobject instance, long param)
{
	ClassData (eventFilterClass, instance, eventFilterData, me);
	return me->isGlobal;
}
*/

static long eventWhatGet (REALobject instance, long param)
{
	ClassData (eventFilterClass, instance, eventFilterData, me);
	return (UInt16)currentEvent.what;
}

static void eventWhatSet (REALobject instance, long param, long v)
{
	ClassData (eventFilterClass, instance, eventFilterData, me);
	currentEvent.what = (UInt16)v;
}

static long eventMsgGet (REALobject instance, long param)
{
	ClassData (eventFilterClass, instance, eventFilterData, me);
	return (SInt32)currentEvent.message;
}

static void eventMsgSet (REALobject instance, long param, long v)
{
	ClassData (eventFilterClass, instance, eventFilterData, me);
	currentEvent.message = (UInt32)v;
}

static long eventWhenGet (REALobject instance, long param)
{
	ClassData (eventFilterClass, instance, eventFilterData, me);
	return (SInt32)currentEvent.when;
}

static long eventWhereXGet (REALobject instance, long param)
{
	ClassData (eventFilterClass, instance, eventFilterData, me);
	return currentEvent.where.h;
}

static long eventWhereYGet (REALobject instance, long param)
{
	ClassData (eventFilterClass, instance, eventFilterData, me);
	return currentEvent.where.v;
}

static long eventModifiersGet (REALobject instance, long param)
{
	ClassData (eventFilterClass, instance, eventFilterData, me);
	return (UInt16)currentEvent.modifiers;
}

static void eventModifiersSet (REALobject instance, long param, long v)
{
	ClassData (eventFilterClass, instance, eventFilterData, me);
	currentEvent.modifiers = (UInt16)v;
}

static long eventMaskGet (REALobject instance, long param)
{
	ClassData (eventFilterClass, instance, eventFilterData, me);
	return me->eventMask;
}

static void eventMaskSet (REALobject instance, long param, long v)
{
	ClassData (eventFilterClass, instance, eventFilterData, me);
	me->eventMask = v;
}

static REALproperty eventFilterProperties[] = {
	//{ nil, "Global", "Boolean", 0, (REALproc) eventFilterIsGlobal },
	{ nil, "what", "Integer", 0, (REALproc) eventWhatGet, (REALproc) eventWhatSet },
	{ nil, "message", "Integer", 0, (REALproc) eventMsgGet, (REALproc) eventMsgSet },
	{ nil, "modifiers", "Integer", 0, (REALproc) eventModifiersGet, (REALproc) eventModifiersSet },
	{ nil, "when", "Integer", 0, (REALproc) eventWhenGet },
	{ nil, "whereX", "Integer", 0, (REALproc) eventWhereXGet },
	{ nil, "whereY", "Integer", 0, (REALproc) eventWhereYGet },
	{ nil, "actionEventMask", "Integer", 0, (REALproc) eventMaskGet, (REALproc) eventMaskSet },
};

/*
static REALmethodDefinition eventFilterMethods[] = {
	// not supported yet: { (REALproc) eventFilterConstructorWithBoolean, REALnoImplementation, "TTsEventMonitor(global as Boolean)"},
};
*/

static REALevent eventFilterEvents[] = {
	{ "Action" },
};


static REALclassDefinition eventFilterClass = {
	kCurrentREALControlVersion,
	"TTsEventMonitor",
	nil,
	sizeof(eventFilterData),
	0,
	(REALproc) eventFilterConstructor,
	(REALproc) eventFilterDestructor,
	eventFilterProperties,
	sizeof(eventFilterProperties) / sizeof(REALproperty),
	nil,	//	eventFilterMethods,
	0,		//	sizeof(eventFilterMethods) / sizeof(REALmethodDefinition),
	eventFilterEvents,
	sizeof(eventFilterEvents) / sizeof(REALevent),
	nil,
	0,
	nil
};

void PluginEntry (void)
{
	root = nil;

	REALRegisterClass (&eventFilterClass);

	// Save the global A4 register for the hook/filter functions
	#if TARGET_68K
		PrepareCallback ();
	#endif

	// install our event filter
	#if UseRBsFilter
		#if not TARGET_WIN32
			REALRegisterEventFilter (theEventFilterHandler, 0);
		#endif
	#else
		installWNEHook ();
	#endif
}
