#ifndef RB_PLUGIN_H
#define RB_PLUGIN_H

#include "REALplugin.h"

// routines to register entities with REALbasic
void REALRegisterMethod(REALmethodDefinition *defn);
void REALRegisterControl(REALcontrol *defn);
void REALRegisterClass(REALclassDefinition *defn);
void REALRegisterClassExtension(REALclassDefinition *defn);
void REALRegisterDBEngine(REALdbEngineDefinition *defn);
void REALRegisterDBTable(REALdbTableDefinition *defn);
void REALRegisterDBCursor(REALdbCursorDefinition *defn);
#ifndef WIN32
void REALRegisterEventFilter(void (*callback)(EventRecord *event, long param), long param);
#endif
void REALMarkSocketUsage(void);

// Mac plugin necessities
#ifndef WIN32
QDGlobals *REALQDGlobals(void);
int REALallocateMenuID(void);
void REALreleaseMenuID(int id);
#endif

// information
Boolean REALinRuntime(void);			// returns false while in IDE, true in built app

// general object routines
REALobject REALnewInstance(const char *className);
void REALLockObject(REALobject obj);
void REALUnlockObject(REALobject obj);
REALproc REALInterfaceRoutine(REALobject obj, const char *interfaceName, const char *methodName);

// string routines
void REALLockString(REALstring str);
void REALUnlockString(REALstring str);

const char *REALCString(REALstring str);
const unsigned char *REALPString(REALstring str);

REALstring REALBuildString(const char *contents, int length);
REALstring REALInterpretConstantValue(REALstring value);

// picture routines
enum {
	pictureUnknown,
	pictureMacintoshPICT,
	pictureMacintoshCICN,
	pictureMacintoshIconSuite,
	pictureMacintoshGWorld
};

struct REALpictureDescription
{
	long pictureType;
	void *pictureData;
	long width, height;
};

#ifndef WIN32
REALpicture REALBuildPictureFromPicHandle(PicHandle pic, Boolean bPassOwnership);
REALpicture REALBuildPictureFromGWorld(GWorldPtr world, Boolean bPassOwnership);
#endif
REALpicture REALBuildPictureFromPictureDescription(REALpictureDescription *description, Boolean bPassOwnership);
void REALLockPictureDescription(REALpicture pic, REALpictureDescription *description);
void REALUnlockPictureDescription(REALpicture pic);
void REALPictureClearCache(REALpicture pic);
#ifdef WIN32
void REALDrawPicturePrimitive(HDC hDC, REALpicture pic, const Rect *rBounds, int bTransparent);
#else
void REALDrawPicturePrimitive(REALpicture pic, const Rect *rBounds, int bTransparent);
#endif

// database routines
REALdbCursor REALdbCursorFromDBCursor(dbCursor *cursor, REALdbCursorDefinition *defn);
REALdbDatabase REALdbDatabaseFromDBDatabase(dbDatabase *database, REALdbEngineDefinition *defn);

// FolderItem routines
#ifdef WIN32
REALfolderItem REALFolderItemFromPath(const char *path);
#else
REALfolderItem REALFolderItemFromFSSpec(const FSSpec *spec);
Boolean REALFSSpecFromFolderItem(FSSpec *spec, REALfolderItem item);
#endif
REALstring REALpathFromFolderItem(REALfolderItem item);

// Sound routines

enum {
	soundUnknown,
	soundMacintoshSnd
};

struct REALsoundDescription
{
	long soundType;
	void *soundData;
};

void REALLockSoundDescription(REALsound, REALsoundDescription *description);
void REALUnlockSoundDescription(REALsound);

#ifndef WIN32
REALsound REALBuildSoundFromHandle(Handle sound, Boolean bPassOwnership);


// AppleEvent routines
REALappleEvent REALBuildAppleEvent(const AppleEvent *event, Boolean bPassOwnership);
REALappleEvent REALBuildAEDescList(const AppleEvent *event, Boolean bPassOwnership);
REALappleEvent REALBuildAEObjSpecifier(const AppleEvent *event, Boolean bPassOwnership);
AppleEvent *REALAccessAppleEvent(REALappleEvent event);
AppleEvent *REALAccessAppleEventReply(REALappleEvent event);
#endif

// low level access
#ifdef WIN32
HWND REALGetControlHWND(REALcontrolInstance control);
#else
WindowPtr REALGetWindowHandle(REALwindow window);
ControlHandle REALGetControlHandle(REALcontrolInstance control);
MenuHandle REALGetPopupMenuHandle(REALpopupMenu popup);
#endif

#ifndef WIN32
// Movie accessor functions
int REALenterMovies(void);
MovieController REALgetMoviePlayerController(REALmoviePlayer instance);
Movie REALgetMovieMovie(REALmovie instance);
REALmovie REALbuildMovie(Movie movie, short resRefNum, Boolean bNew);
void REALmarkMovieDirty(REALmovie movie);
#endif

void GraphicsDrawLine(Ptr graphicsObject, int x1, int y1, int x2, int y2);

void *REALGetControlData(REALcontrolInstance instance, REALcontrol *defn);
void *REALGetClassData(REALobject instance, REALclassDefinition *defn);

// Control instance routines
REALstring REALDefaultControlCaption(void);
REALstring REALDefaultControlFont(void);
int REALDefaultControlFontSize(void);
void REALGetControlBounds(REALcontrolInstance instance, Rect *rBounds);
REALwindow REALGetControlWindow(REALcontrolInstance instance);
void REALInvalidateControl(REALcontrolInstance instance);
void REALInvalidateControlRect(REALcontrolInstance instance, int left, int top, int right, int bottom);
void *REALGetEventInstance(REALcontrolInstance instance, REALevent *event);
Boolean REALGetControlVisible(REALcontrolInstance instance);
Boolean REALGetControlEnabled(REALcontrolInstance instance);
void REALSetControlVisible(REALcontrolInstance instance, Boolean visible);
#ifdef WIN32
void REALSetSpecialBackground(REALcontrolInstance instance, COLORREF *pcolor);
#else
void REALSetSpecialBackground(REALcontrolInstance instance);
#endif
REALgraphics REALGetControlGraphics(REALcontrolInstance instance);

#ifdef WIN32
HDC REALGraphicsDC(REALgraphics context);
int REALGetWin32Charset(void);
#else
void REALSelectGraphics(REALgraphics context);
void REALGraphicsDrawOffscreenMacControl(REALgraphics, ControlHandle mh);
#endif

#define ControlData(defn, instance, typeName, data) typeName *data = (typeName *) REALGetControlData(instance, &defn)
#define ClassData(defn, instance, typeName, data) typeName *data = (typeName *) REALGetClassData(instance, &defn)

// socket routines
#define socketEventConnect 1
#define socketEventError 2
#define socketEventDataReceived 4
#define socketEventSendComplete 8

void REALSocketConnect(REALsocket, REALstring address, int port);
void REALSocketClose(REALsocket);
REALstring REALSocketReadAll(REALsocket);
REALstring REALSocketRead(REALsocket, int count);
void REALSocketWrite(REALsocket, REALstring data);
int REALSocketLastErrorCode(REALsocket);
REALstring REALSocketLookahead(REALsocket);
REALstring REALSocketLocalAddressGetter(REALsocket);
void REALSocketPoll(REALsocket);
int REALSocketGetEvents(REALsocket);

// database interface
void REALRegisterDataSourceInterface(const char *szMenuName, void (*proc)(void));
void REALRegisterDataSource(const char *szDatasourceName, REALdbDatabase (*proc)(Ptr data, int dataLen));
void REALDesignAddDataSource(const char *baseName, const char *szDataSourceName, Ptr data, int dataLen);

// utility functions
void REALMessageBox(REALstring text);
long REALstringToOSType(REALstring id);

#endif
