/*
 * MountVol Interceptor.cp
 *
 * Plugin for REALbasic 2.1 and later
 *
 * Written on May 22, 2002 by Thomas Tempelmann for the public domain
 *
 * More RB-related stuff can be found here: <http://www.tempel.org/rb/>
 *
 * Enjoy and contribute!
 */

#include <MixedMode.h>
#include "rb_plugin.h"
#include "MountVol Interceptor.h"


#if CARBON

	/* A macro which creates a static instance of a non-dispatched routine descriptor */
	#define BUILD_ROUTINE_DESCRIPTOR(procInfo, procedure)                               \
	    {                                                                               \
	        _MixedModeMagic,                            /* Mixed Mode A-Trap */         \
	        kRoutineDescriptorVersion,                  /* version */                   \
	        kSelectorsAreNotIndexable,                  /* RD Flags - not dispatched */ \
	        0,                                          /* reserved 1 */                \
	        0,                                          /* reserved 2 */                \
	        0,                                          /* selector info */             \
	        0,                                          /* number of routines */        \
	        {                                           /* Its an array */             \
	            {                                       /* Its a struct */             \
	                (procInfo),                         /* the ProcInfo */              \
	                0,                                  /* reserved */                  \
	                GetCurrentArchitecture(),           /* ISA and RTA */               \
	                kProcDescriptorIsAbsolute |         /* Flags - its absolute addr */\
	                kFragmentIsPrepared |               /* Its prepared */             \
	                kUseNativeISA,                      /* Always use native ISA */     \
	                (ProcPtr)(procedure),               /* the procedure */             \
	                0,                                  /* reserved */                  \
	                0                                   /* Not dispatched */            \
	            }                                                                       \
	        }                                                                           \
	    }

	static UniversalProcPtr
	NewRoutineDescriptor(
	  ProcPtr        theProc,
	  ProcInfoType   theProcInfo,
	  ISAType        theISA)
    {
    	struct RoutineDescriptor xx = \
	    {                                                                               \
	        _MixedModeMagic,                            /* Mixed Mode A-Trap */         \
	        kRoutineDescriptorVersion,                  /* version */                   \
	        kSelectorsAreNotIndexable,                  /* RD Flags - not dispatched */ \
	        0,                                          /* reserved 1 */                \
	        0,                                          /* reserved 2 */                \
	        0,                                          /* selector info */             \
	        0,                                          /* number of routines */        \
	        {                                           /* Its an array */             \
	            {                                       /* Its a struct */             \
	                theProcInfo,                        /* the ProcInfo */              \
	                0,                                  /* reserved */                  \
	                theISA,                             /* ISA and RTA */               \
	                kProcDescriptorIsAbsolute |         /* Flags - its absolute addr */\
	                kFragmentIsPrepared |               /* Its prepared */             \
	                kUseNativeISA,                      /* Always use native ISA */     \
	                (ProcPtr)(theProc),                 /* the procedure */             \
	                0,                                  /* reserved */                  \
	                0                                   /* Not dispatched */            \
	            }                                                                       \
	        }                                                                           \
	    };
    	RoutineDescriptor *rd = (RoutineDescriptor *) NewPtr (sizeof(RoutineDescriptor));
    	if (not rd) { DebugStr ("\poops - NewPtr failed in MountVol Interceptor Plugin"); return nil; }
	    *rd = xx;
	    return rd;
    }

	static UniversalProcPtr
	GetOSTrapAddress(UInt16 trapNum)
	{
		long v = 0;
		Gestalt (gestaltOSTable, &v);
		UniversalProcPtr *p = (UniversalProcPtr*) (v + 4 * (trapNum & 0x3FF));
		return *p;
	}

	static void SetOSTrapAddress(
	  UniversalProcPtr   trapAddr,
	  UInt16             trapNum)
	{
		long v = 0;
		Gestalt (gestaltOSTable, &v);
		UniversalProcPtr *p = (UniversalProcPtr*) (v + 4 * (trapNum & 0x3FF));
		*p = trapAddr;
	}

	static UniversalProcPtr
	GetToolTrapAddress(UInt16 trapNum)
	{
		long v = 0;
		Gestalt (gestaltToolboxTable, &v);
		UniversalProcPtr *p = (UniversalProcPtr*) (v + 4 * (trapNum & 0x3FF));
		return *p;
	}

	static void
	SetToolTrapAddress(
	  UniversalProcPtr   trapAddr,
	  UInt16             trapNum)
	{
		long v = 0;
		Gestalt (gestaltToolboxTable, &v);
		UniversalProcPtr *p = (UniversalProcPtr*) (v + 4 * (trapNum & 0x3FF));
		*p = trapAddr;
	}

#endif


typedef	long RBinteger;
typedef	Boolean RBboolPar;

enum {
  xWaitNextEvent                = 0xA860,
  xMountVol                     = 0xA00F
};

#define useWNE 0

#if useWNE

	// intercept WNE calls, look for diskEvt events
	
	const ProcInfoType wneProcInfo =
		kPascalStackBased |
		RESULT_SIZE(SIZE_CODE(sizeof(Boolean))) |
		STACK_ROUTINE_PARAMETER(1, SIZE_CODE(sizeof(short))) |
		STACK_ROUTINE_PARAMETER(2, SIZE_CODE(sizeof(EventRecord*))) |
		STACK_ROUTINE_PARAMETER(3, SIZE_CODE(sizeof(UInt32))) |
		STACK_ROUTINE_PARAMETER(4, SIZE_CODE(sizeof(RgnHandle)));
		
	#ifdef powerc
		typedef UniversalProcPtr wneProc;
	#else
		typedef pascal Boolean (*wneProc) (short eventMask, EventRecord* theEvent, UInt32 sleep, RgnHandle mouseRgn);
	#endif

	static	UniversalProcPtr gNewWNE;
	static	wneProc gOldWNE;

#else

	// intercept MountVol calls
	
	const ProcInfoType mountVolProcInfo =
		kRegisterBased |
		RESULT_SIZE(SIZE_CODE(sizeof(OSErr))) |
		REGISTER_ROUTINE_PARAMETER(1, kRegisterA0, SIZE_CODE(sizeof(void*)));

	#ifdef powerc
		typedef UniversalProcPtr mountVolProc;
	#else
		// causes compiler error: #pragma parameter __D0 (*mountVolProc)(__A0)
		typedef pascal OSErr (*mountVolProc) (VolumeParam *);
	#endif

	static	UniversalProcPtr gNewMountVol;
	static	mountVolProc gOldMountVol;

#endif

static	Boolean gHooksInstalled;

#ifdef powerc
	#define __far
#else
	#define __far far
#endif

#define cMaxDrv 32
static	__far	int gDiskInserted;
static	__far	short gDriveStack[cMaxDrv+1];


#if useWNE
	static pascal Boolean wneHook (EventMask eventMask, EventRecord* theEvent, UInt32 sleep, RgnHandle mouseRgn)
	{
		Boolean b = CALL_FOUR_PARAMETER_UPP (gOldWNE, wneProcInfo, eventMask, theEvent, sleep, mouseRgn);
		if (theEvent->what == diskEvt) {
			// a disk was inserted
			if (gDiskInserted < cMaxDrv) {
				gDriveStack[gDiskInserted++] = theEvent->what & 0xFFFF;
				theEvent->what = 0;
				b = false;
			}
		}
		return b;
	}
#else
	#ifndef powerc
		#pragma parameter __D0 mountVolHook(__A0)
	#endif
	static pascal OSErr mountVolHook (VolumeParam *pb)
	{
		// MountVol is called
		if (gDiskInserted < cMaxDrv) {
			gDriveStack[gDiskInserted++] = pb->ioVRefNum;
		}
		return noErr;	// pretend the disk got mounted even though it didn't
	}
#endif

static void removeHook ()
{
	if (gHooksInstalled) {
		#if useWNE
			if (gOldWNE) SetToolTrapAddress ((UniversalProcPtr)gOldWNE, xWaitNextEvent);
			gOldWNE = nil;
		#else
			if (gOldMountVol) SetOSTrapAddress ((UniversalProcPtr)gOldMountVol, xMountVol);
			gOldMountVol = nil;
		#endif
		gHooksInstalled = false;
	}
}

static void installHook ()
{
	if (not gHooksInstalled) {
		// install the WNE hook
		#if useWNE
			gOldWNE = (wneProc) GetToolTrapAddress (xWaitNextEvent);
			SetToolTrapAddress (gNewWNE, xWaitNextEvent);
		#else
			gOldMountVol = (mountVolProc) GetOSTrapAddress (xMountVol);
			SetOSTrapAddress (gNewMountVol, xMountVol);
		#endif
		gHooksInstalled = true;
	}
	gDiskInserted = 0;
}

static RBinteger getInsert ()
{
	if (not gHooksInstalled or gDiskInserted <= 0) return 0;
	short drv = gDriveStack[--gDiskInserted];
	return drv;
}

static void intercept (RBboolPar active)
{
	if (active) {
		installHook ();
	} else {
		removeHook ();
	}
}

static REALmethodDefinition PluginMethodArray[] = {
	{ (REALproc)getInsert, REALnoImplementation, "GetInsertedDriveNum() as Integer)" },
	{ (REALproc)intercept, REALnoImplementation, "InterceptDiskInsertionEvents(active as Boolean)" },
};

void ExitMountVolInterceptor ()
{
	removeHook ();
}

void InitMountVolInterceptor ()
{
	gHooksInstalled = false;	// preset

	// allocate the ProcPtrs
	long v = 0;
	Gestalt (gestaltSystemVersion, &v);
	if (v < 0x0A00) {
		#if useWNE
			gNewWNE = NewRoutineDescriptor ((ProcPtr)wneHook, wneProcInfo, GetCurrentISA());
			gOldWNE = (wneProc)nil;
		#else
			gNewMountVol = NewRoutineDescriptor ((ProcPtr)mountVolHook, mountVolProcInfo, GetCurrentISA());
			gOldMountVol = (mountVolProc)nil;
		#endif
	}

	for (int i = 0; i < sizeof(PluginMethodArray) / sizeof(REALmethodDefinition); ++i) {
		REALRegisterMethod (&PluginMethodArray[i]);
	}
}

// EOF
