/*
 * "MountVol Interceptor" Plugin for REALbasic 2.1 and later
 *
 * Written on May 22, 2002 by Thomas Tempelmann for the public domain
 *
 * Built with CW Pro 5
 *
 * User documention is available in the "docs.rsrc". Read and edit it using
 * the "RB Plugin Plunger" (http://homepage.mac.com/vanhoek/#pluginplunger)
 *
 * More RB-related stuff can be found here: <http://www.tempel.org/rb/>
 *
 * Enjoy and contribute!
 */

#ifdef TARGET_68K
	#include <SetupA4.h>
#endif
#include "MountVol Interceptor.h"
#include "rb_plugin.h"

enum {
  xExitToShell                  = 0xA9F4
};


#if CARBON

	/* A macro which creates a static instance of a non-dispatched routine descriptor */
	#define BUILD_ROUTINE_DESCRIPTOR(procInfo, procedure)                               \
	    {                                                                               \
	        _MixedModeMagic,                            /* Mixed Mode A-Trap */         \
	        kRoutineDescriptorVersion,                  /* version */                   \
	        kSelectorsAreNotIndexable,                  /* RD Flags - not dispatched */ \
	        0,                                          /* reserved 1 */                \
	        0,                                          /* reserved 2 */                \
	        0,                                          /* selector info */             \
	        0,                                          /* number of routines */        \
	        {                                           /* Its an array */             \
	            {                                       /* Its a struct */             \
	                (procInfo),                         /* the ProcInfo */              \
	                0,                                  /* reserved */                  \
	                GetCurrentArchitecture(),           /* ISA and RTA */               \
	                kProcDescriptorIsAbsolute |         /* Flags - its absolute addr */\
	                kFragmentIsPrepared |               /* Its prepared */             \
	                kUseNativeISA,                      /* Always use native ISA */     \
	                (ProcPtr)(procedure),               /* the procedure */             \
	                0,                                  /* reserved */                  \
	                0                                   /* Not dispatched */            \
	            }                                                                       \
	        }                                                                           \
	    }

	static UniversalProcPtr
	NewRoutineDescriptor(
	  ProcPtr        theProc,
	  ProcInfoType   theProcInfo,
	  ISAType        theISA)
    {
    	struct RoutineDescriptor xx = \
	    {                                                                               \
	        _MixedModeMagic,                            /* Mixed Mode A-Trap */         \
	        kRoutineDescriptorVersion,                  /* version */                   \
	        kSelectorsAreNotIndexable,                  /* RD Flags - not dispatched */ \
	        0,                                          /* reserved 1 */                \
	        0,                                          /* reserved 2 */                \
	        0,                                          /* selector info */             \
	        0,                                          /* number of routines */        \
	        {                                           /* Its an array */             \
	            {                                       /* Its a struct */             \
	                theProcInfo,                        /* the ProcInfo */              \
	                0,                                  /* reserved */                  \
	                theISA,                             /* ISA and RTA */               \
	                kProcDescriptorIsAbsolute |         /* Flags - its absolute addr */\
	                kFragmentIsPrepared |               /* Its prepared */             \
	                kUseNativeISA,                      /* Always use native ISA */     \
	                (ProcPtr)(theProc),                 /* the procedure */             \
	                0,                                  /* reserved */                  \
	                0                                   /* Not dispatched */            \
	            }                                                                       \
	        }                                                                           \
	    };
    	RoutineDescriptor *rd = (RoutineDescriptor *) NewPtr (sizeof(RoutineDescriptor));
    	if (not rd) { DebugStr ("\poops - NewPtr failed in MountVol Interceptor Plugin"); return nil; }
	    *rd = xx;
	    return rd;
    }

	static UniversalProcPtr
	GetOSTrapAddress(UInt16 trapNum)
	{
		long v = 0;
		Gestalt (gestaltOSTable, &v);
		UniversalProcPtr *p = (UniversalProcPtr*) (v + 4 * (trapNum & 0x3FF));
		return *p;
	}

	static void SetOSTrapAddress(
	  UniversalProcPtr   trapAddr,
	  UInt16             trapNum)
	{
		long v = 0;
		Gestalt (gestaltOSTable, &v);
		UniversalProcPtr *p = (UniversalProcPtr*) (v + 4 * (trapNum & 0x3FF));
		*p = trapAddr;
	}

	static UniversalProcPtr
	GetToolTrapAddress(UInt16 trapNum)
	{
		long v = 0;
		Gestalt (gestaltToolboxTable, &v);
		UniversalProcPtr *p = (UniversalProcPtr*) (v + 4 * (trapNum & 0x3FF));
		return *p;
	}

	static void
	SetToolTrapAddress(
	  UniversalProcPtr   trapAddr,
	  UInt16             trapNum)
	{
		long v = 0;
		Gestalt (gestaltToolboxTable, &v);
		UniversalProcPtr *p = (UniversalProcPtr*) (v + 4 * (trapNum & 0x3FF));
		*p = trapAddr;
	}

#endif



static UniversalProcPtr gOldHandler;

static pascal void CleanupAndExit (void)
{
	#ifdef TARGET_68K
		EnterCallback();
	#endif
	ExitMountVolInterceptor ();
	SetToolTrapAddress (gOldHandler, xExitToShell);
	#ifdef TARGET_68K
		ExitCallback();
	#endif
	ExitToShell ();
}

static void InitQuit ()
{
	gOldHandler = GetToolTrapAddress (xExitToShell);
	SetToolTrapAddress (NewRoutineDescriptor ((ProcPtr)CleanupAndExit, kPascalStackBased, GetCurrentISA()), xExitToShell);
}

void PluginEntry (void)
{
	long v = 0;
	Gestalt (gestaltSystemVersion, &v);
	if (v < 0x0A00) {
		#ifdef TARGET_68K
			PrepareCallback ();
		#endif
		InitQuit ();
	}

	InitMountVolInterceptor ();
}
