/*
 * Plugin for REALbasic 3 to get some properties of any REALbasic object
 *
 * This code is freeware, you may use and extend it as you like.
 *
 * The accompanying project file was created using CodeWarrior Pro 6.
 *
 * More RB-related stuff can be found here: <http://www.tempel.org/rb/>
 *
 * Enjoy and contribute!
 *
 * ---------
 *  History
 * ---------
 *
 *  Jan 15, 2001, v1.0
 *     Initially written by Thomas Tempelmann after a request from Frank Esselbach
 *  Feb 7, 2001, v1.1
 *     Added String accessors and TTsObjectPeekPluginVersion()
 *  Sep 8, 2002, v1.2
 *     Lock count can now be set, too
 *  Jul 19, 2004, v2.0
 *     ObjLockCount() supports new object layout of RB 5 and 5.5;
 *     Disabled ObjClassPtr() because I don't know the ofs for RB 5.5
 *  Jul 3, 2005, v2.1
 *     Made work with RB 2005
 *  Jan 28, 2006, v3.0
 *     Added more functions, tested with RB 5.5.5, 2005r4 and 2006r1
 */

#include <stdarg.h>
#include "REALplugin.h"
#include "rb_plugin.h"

//------------------
// exported routines
//------------------

static long countOfs = 0;
static long strCntOfs = 0;
static long addVal = 1;

static long getLockCount (REALobject obj)
{
	if (!obj) return 0;
	return *(long*)((Ptr)obj+countOfs) - addVal;	// we subtract one here for not counting the additional passed-in reference
}

static void setLockCount (REALobject obj, long l)
{
	if (obj) {
		*(long*)((Ptr)obj+countOfs) = l + addVal;	// we add one here for the additional passed-in reference which will be dec'd after this call
	}
}

static long getLockStr (REALstring obj)
{
	if (!obj) return 0;
	return *(long*)((Ptr)obj+strCntOfs) - addVal;	// we subtract one here for not counting the additional passed-in reference
}

static void setLockStr (REALstring obj, long l)
{
	if (obj) {
		*(long*)((Ptr)obj+strCntOfs) = l + addVal;	// we add one here for the additional passed-in reference which will be dec'd after this call
	}
}

static REALmemoryBlock getClassPtr (REALobject obj)
{
	if (!obj) return nil;
	void* cl = *(void**)((Ptr)obj+0x04);
	return REALPtrToMemoryBlock (cl);
}

static REALmemoryBlock getObjPtr (REALobject obj)
{
	if (!obj) return nil;
	return REALPtrToMemoryBlock (obj);
}

static void* getStackPtr (int dummy, ...)
{
	void *ptr;
	va_list vl;
	va_start (vl, dummy);
	ptr = (void*) vl;
	va_end (vl);
	static long x = (long) ptr;
	return ptr;
}

static long getVers ()
{
	return 0x030000;
}

static REALmethodDefinition pluginMethodArray[] = {
	{ (REALproc) getVers, REALnoImplementation, "TTsObjectPeekVersion() as Integer" },
	{ (REALproc) getObjPtr, REALnoImplementation, "TTsObjBasePtr(obj as Object) as MemoryBlock" },
	{ (REALproc) getObjPtr, REALnoImplementation, "TTsStrBasePtr(str as String) as MemoryBlock" },
	{ (REALproc) getLockCount, REALnoImplementation, "TTsObjLockCount(obj as Object) as Integer" },
	{ (REALproc) setLockCount, REALnoImplementation, "TTsObjLockCountSet(obj as Object, c as Integer)" },
	{ (REALproc) getLockStr, REALnoImplementation, "TTsStrLockCount(str as String) as Integer" },
	{ (REALproc) setLockStr, REALnoImplementation, "TTsStrLockCountSet(str as String, c as Integer)" },
	{ (REALproc) getClassPtr, REALnoImplementation, "TTsObjClassPtr(obj as Object) as MemoryBlock" },
	{ (REALproc) getStackPtr, REALnoImplementation, "TTsStackPointer() as Integer" },
};

void PluginEntry (void)
{
	for (int i = 0; i < sizeof(pluginMethodArray) / sizeof(REALmethodDefinition); ++i) {
		REALRegisterMethod (&pluginMethodArray[i]);
	}
	if (REALGetRBVersion() >= 5 and REALGetRBVersion() < 6) {
		addVal = 0;
	}
	if (REALGetRBVersion() >= 5.5) {
		countOfs = 12;
	}
}
