
/*
 * Plugin for RB 2.1
 *
 * By Thomas Tempelmann (rb@tempel.org), 13 July 2000
 * Enhanced July 14-20 by Alfred Van Hoek and Bjrn Eirksson
 */

/*
 * How it works
 *--------------
 *
 *  Well, it's not a secret, but we haven't taken the effort to
 *  document it here, either. Perhaps you can understand it by
 *  looking at this source code. Start with browsePlugin() and
 *  loadPlugin().
 *
 *  Oh, and although there's also a target to build a 68K version
 *  of this plugin, it has never been tested and is likely not to
 *  work without modifications. So, current you can only rely on
 *  the PPC version to work as intended.
 */

//----------------------
// common includes here
//----------------------

#include <string.h>
#include <setjmp.h>
#include <CodeFragments.h>
#include <MacMemory.h>
#include <SetupA4.h>

//-------------------------
// RB header includes here
//-------------------------

#include "rb_plugin.h"


//-----------------------
// Some type definitions
//-----------------------

typedef long RBinteger;
typedef Boolean RBboolean;


//*********************************************************************************

enum ClassDefnTypes {
	classType, controlType, dbEngineType, dbTableType, dbCursorType
};

enum {
	kVersion1 = 1,
	kVersion2,
	kVersion3,
	kVersion4,
	kVersion5
};


//------------------------------------------------------------
// Class Analyser Class (also supports Control Analyer Class)
//------------------------------------------------------------

extern REALclassDefinition classAndCtrlAnalClass;
extern REALclassDefinition ctrlAnalClass;

struct classAndCtrlAnalClassData
{
	union {
		long				*version;	// the version is always the fist field in all plugin-defined RB structs
		REALcontrol			*rbcntl;
		REALclassDefinition	*rbclass;
	};
	ClassDefnTypes	type;
	RBboolean		isExt;
};


//------------------------------------------------------------

static RBinteger getStructVersion (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	return *me->version;
}

static REALstring getSuper (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	const char* name = nil;	
	if (me->type == classType) {
		name = me->rbclass->superName;
	}
	return REALBuildString (name, name?strlen(name):0);
}

static RBinteger methodCnt (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	int rbint = 0;
	if (me->type == classType) {
		rbint = me->rbclass->methodCount;
	} else if (me->type == controlType) {
		rbint = me->rbcntl->methodCount;
	}
	return rbint;
}

static REALstring getMethod (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	const char* name = nil;	
	if (me->type == classType) {
		name = me->rbclass->methods[idx-1].declaration;
	} else if (me->type == controlType) {
		name = me->rbcntl->methods[idx-1].declaration;
	}
	return REALBuildString (name, name?strlen(name):0);
}

static RBinteger propertyCnt (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	int rbint = 0;
	if (me->type == classType) {
		rbint = me->rbclass->propertyCount;
	} else if (me->type == controlType) {
		rbint = me->rbcntl->propertyCount;
	}
	return rbint;
}

static REALstring getPropertyName (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	const char* name = nil;
	if (me->type == classType) {
		name = me->rbclass->properties[idx-1].name;
	} else if (me->type == controlType) {
		name = me->rbcntl->properties[idx-1].name;
	}
	return REALBuildString (name, name?strlen(name):0);
}

static REALstring getPropertyType (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	const char* name = nil;
	if (me->type == classType) {
		name = me->rbclass->properties[idx-1].type;
	} else if (me->type == controlType) {
		name = me->rbcntl->properties[idx-1].type;
	}
	return REALBuildString (name, name?strlen(name):0);
}

static RBinteger eventCnt (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	int rbint = 0;
	if (me->type == classType) {
		rbint = me->rbclass->eventCount;
	} else if (me->type == controlType) {
		rbint = me->rbcntl->eventCount;
	}
	return rbint;
}

static REALstring getEvent (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	const char* name;	
	if (me->type == classType) {
		name = me->rbclass->events[idx-1].declaration;
	} else if (me->type == controlType) {
		name = me->rbcntl->events[idx-1].declaration;
	}
	return REALBuildString (name, name?strlen(name):0);
}

static RBinteger eventInstanceCnt (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	int rbint = 0;
	if (me->type == classType) {
		if (me->rbclass->version >= kVersion3) {
			rbint = me->rbclass->eventInstanceCount;
		}
	} else if (me->type == controlType) {
		if (me->rbcntl->version >= kVersion4) {
			rbint = me->rbcntl->eventInstanceCount;
		}
	}
	return rbint;
}

static REALstring getEventInstance (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	const char* name = nil;
	if (me->type == classType) {
		if (me->rbclass->version >= kVersion3) {
			name = me->rbclass->eventInstances[idx-1].name;
		}
	} else if (me->type == controlType) {
		if (me->rbcntl->version >= kVersion4) {
			name = me->rbcntl->eventInstances[idx-1].name;
		}
	}
	return REALBuildString (name, name?strlen(name):0);
}


static RBinteger bindDescriptionCnt (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	int rbint = 0;
	if (me->type == classType) {
		if (me->rbclass->version >= kVersion4) {
			rbint = me->rbclass->bindDescriptionCount;
		}
	} else if (me->type == controlType) {
		if (me->rbcntl->version >= kVersion4) {
			rbint = me->rbcntl->bindDescriptionCount;
		}
	}
	return rbint;
}

static REALstring getbindInterface (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	const char* name = nil;
	if (me->type == classType) {
		if (me->rbclass->version >= kVersion5) {
			name = me->rbclass->bindDescriptions[idx-1].szInterface;
		}
	} else if (me->type == controlType) {
		if (me->rbcntl->version >= kVersion5) {
			name = me->rbcntl->bindDescriptions[idx-1].szInterface;
		}
	}
	return REALBuildString (name, name?strlen(name):0);
}

static REALstring getbindDescription (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	const char* name = nil;
	if (me->type == classType) {
		if (me->rbclass->version >= kVersion5) {
			name = me->rbclass->bindDescriptions[idx-1].szDescription;
		}
	} else if (me->type == controlType) {
		if (me->rbcntl->version >= kVersion5) {
			name = me->rbcntl->bindDescriptions[idx-1].szDescription;
		}
	}
	return REALBuildString (name, name?strlen(name):0);
}

static REALstring getbindPart (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	const char* name = nil;
	if (me->type == classType) {
		if (me->rbclass->version >= kVersion5) {
			name = me->rbclass->bindDescriptions[idx-1].szPart;
		}
	} else if (me->type == controlType) {
		if (me->rbcntl->version >= kVersion5) {
			name = me->rbcntl->bindDescriptions[idx-1].szPart;
		}
	}
	return REALBuildString (name, name?strlen(name):0);
}

static RBinteger propertyFlags (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	int rbint = 0;
	if (me->type == classType) {
		rbint = me->rbclass->properties[idx-1].flags;
	} else if (me->type == controlType) {
		rbint = me->rbcntl->properties[idx-1].flags;
	}
	return rbint;
}

static REALstring getInterfaces (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	const char* name = nil;
	if (me->type == classType && me->rbclass->version >= kVersion3) {
		name = me->rbclass->interfaces;
	} else if (me->type == controlType && me->rbcntl->version >= kVersion5) {
		name = me->rbcntl->interfaces;
	}
	return REALBuildString (name, name?strlen(name):0);
}

static RBboolean tstConstructor (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	REALproc proc = nil;	
	if (me->type == controlType && me->rbcntl->behaviour != nil) {
		proc = (REALproc) me->rbcntl->behaviour->constructorFunction;
	} else if (me->type == classType) {
		proc = me->rbclass->constructor;
	}
	return proc != nil;
}

static RBboolean tstDestructor (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	REALproc proc = nil;	
	if (me->type == controlType && me->rbcntl->behaviour != nil) {
		proc = (REALproc) me->rbcntl->behaviour->destructorFunction;
	} else if (me->type == classType) {
		proc = me->rbclass->destructor;
	}
	return proc != nil;
}

//------------------------------------------------------------

static RBinteger controlFlags (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	int rbint = 0;
	if (me->type == controlType) {
		rbint = me->rbcntl->flags;
	}
	return rbint;
}

static RBboolean tstLongAsBoolean (REALobject instance, long param)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	REALproc proc = nil;	
	if (me->type == controlType && me->rbcntl->behaviour != nil) {
		proc = (REALproc) *((char*)me->rbcntl->behaviour + param);
	}
	return proc != nil;
}

//------------------------------------------------------------

//  --- methods for PluginClassAnalyser and PluginControlAnalyser ---
static REALmethodDefinition analClMethods[] = {
	{ (REALproc) getStructVersion, REALnoImplementation, "Version() as Integer" },
	{ (REALproc) getSuper, REALnoImplementation, "Super() as String" },
	{ (REALproc) methodCnt, REALnoImplementation, "MethodCount() as Integer" },
	{ (REALproc) getMethod, REALnoImplementation, "Method(idx as Integer) as String" },
	{ (REALproc) propertyCnt, REALnoImplementation, "PropertyCount() as Integer" },
	{ (REALproc) getPropertyName, REALnoImplementation, "PropertyName(idx as Integer) as String" },
	{ (REALproc) getPropertyType, REALnoImplementation, "PropertyType(idx as Integer) as String" },
	{ (REALproc) propertyFlags, REALnoImplementation, "PropertyFlags(idx as Integer) as Integer" },
	{ (REALproc) eventCnt, REALnoImplementation, "EventCount() as Integer" },
	{ (REALproc) getEvent, REALnoImplementation, "Event(idx as Integer) as String" },
	{ (REALproc) eventInstanceCnt, REALnoImplementation, "EventInstanceCount() as Integer" },
	{ (REALproc) getEventInstance, REALnoImplementation, "EventInstance(idx as Integer) as String" },
	{ (REALproc) getInterfaces, REALnoImplementation, "Interfaces() as String" },
	{ (REALproc) bindDescriptionCnt, REALnoImplementation, "BindDescriptionCount() as Integer" },
	{ (REALproc) getbindInterface, REALnoImplementation, "BindInterface(idx as Integer) as String" },
	{ (REALproc) getbindDescription, REALnoImplementation, "BindDescription(idx as Integer) as String" },
	{ (REALproc) getbindPart, REALnoImplementation, "BindPart(idx as Integer) as String" },

	{ (REALproc) tstConstructor, REALnoImplementation, "ImplementsConstructor() as Boolean" },
	{ (REALproc) tstDestructor, REALnoImplementation, "ImplementsDestructor() as Boolean" },
};

//  --- properties for PluginClassAnalyser and PluginControlAnalyser ---
static REALproperty analClProperties[] = {
	{ nil, "IsClassExtension", "Boolean", 0, REALstandardGetter, REALstandardSetter, FieldOffset(classAndCtrlAnalClassData, isExt) },
};

//  --- methods for PluginControlAnalyser only ---
static REALmethodDefinition analCtMethods[] = {
	{ (REALproc) controlFlags, REALnoImplementation, "ControlFlags() as Integer" },
};

//  --- properties for PluginControlAnalyser only ---
static REALproperty analCtProperties[] = {
	{ nil, "ImplementsConstructorFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, redrawFunction) },
	{ nil, "ImplementsDestructorFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, redrawFunction) },
	{ nil, "ImplementsRedrawFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, redrawFunction) },
	{ nil, "ImplementsClickFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, clickFunction) },
	{ nil, "ImplementsMouseDragFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, mouseDragFunction) },
	{ nil, "ImplementsMouseUpFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, mouseUpFunction) },
	{ nil, "ImplementsGainedFocusFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, gainedFocusFunction) },
	{ nil, "ImplementsLostFocusFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, lostFocusFunction) },
	{ nil, "ImplementsKeyDownFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, keyDownFunction) },
	{ nil, "ImplementsOpenFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, openFunction) },
	{ nil, "ImplementsCloseFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, closeFunction) },
	{ nil, "ImplementsBackgroundIdleFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, backgroundIdleFunction) },
	{ nil, "ImplementsDrawOffscreenFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, drawOffscreenFunction) },
	{ nil, "ImplementsSetSpecialBackground", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, setSpecialBackground) },
	{ nil, "ImplementsConstantChanging", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, constantChanging) },
	{ nil, "ImplementsDroppedNewInstance", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, droppedNewInstance) },
};

static REALclassDefinition classAndCtrlAnalClass = {
	kCurrentREALControlVersion,
	"PluginClassAnalyser", nil,
	sizeof(classAndCtrlAnalClassData), 0,
	REALnoImplementation, REALnoImplementation,
	analClProperties, sizeof(analClProperties) / sizeof(REALproperty),
	analClMethods, sizeof(analClMethods) / sizeof(REALmethodDefinition),
};

static REALclassDefinition ctrlAnalClass = {
	kCurrentREALControlVersion,
	"PluginControlAnalyser", "PluginClassAnalyser",
	nil, 0,
	REALnoImplementation, REALnoImplementation,
	analCtProperties, sizeof(analCtProperties) / sizeof(REALproperty),
	analCtMethods, sizeof(analCtMethods) / sizeof(REALmethodDefinition),
};


//*********************************************************************************

//---------------------
// DB Analyser Classes
//---------------------

extern REALclassDefinition databaseAnalClass;

struct databaseAnalClassData
{
	union {
		long					*version;	// the version is always the fist field in all plugin-defined RB structs
		char					*structData;
		REALdbEngineDefinition	*rbDBngn;
		REALdbTableDefinition	*rbDBtbl;
		REALdbCursorDefinition	*rbDBcrsr;
	};
	ClassDefnTypes	type;
};

static RBinteger getDBStructVersion (REALobject instance)
{
	ClassData (databaseAnalClass, instance, databaseAnalClassData, me);
	return *me->version;
}

static RBboolean dbLongAsBoolean (REALobject instance, long param)
{
	ClassData (databaseAnalClass, instance, databaseAnalClassData, me);
	REALproc proc = (REALproc) *(me->structData + param);
	return proc != nil;
}

//----------------------------------------------------------------

// methods common to all DB classes
static REALmethodDefinition analDBMethods[] = {
	{ (REALproc) getDBStructVersion, REALnoImplementation, "Version() as Integer" },
};

// properties of PluginDBEngineAnalyser class
static REALproperty analDBEngineProperties[] = {
	{ nil, "ImplementsCloseDatabase", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,closeDatabase) },
	{ nil, "ImplementsGetTableSchemaCrsr", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,getTableSchemaCursor) },
	{ nil, "ImplementsGetFieldSchemaCrsr", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,getFieldSchemaCursor) },
	{ nil, "ImplementsDirectSQLSelect", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,directSQLSelect) },
	{ nil, "ImplementsDirectSQLExecute", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,directSQLExecute) },
	{ nil, "ImplementsCreateTable", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,createTable) },
	{ nil, "ImplementsAddTableRecord", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,addTableRecord) },
	{ nil, "ImplementsGetTableCursor", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,getTableCursor) },
	{ nil, "ImplementsUpdateFields", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,updateFields) },
	{ nil, "ImplementsAddTableColumn", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,addTableColumn) },
	{ nil, "ImplementsGetDatabaseIndexes", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,getDatabaseIndexes) },
	{ nil, "ImplementsGetLastErrorCode", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,getLastErrorCode) },
	{ nil, "ImplementsGetLastErrorString", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,getLastErrorString) },
	{ nil, "ImplementsCommit", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,commit) },
	{ nil, "ImplementsRollback", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,rollback) },
	{ nil, "ImplementsGetProperty", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,getProperty) },
};

// properties of PluginDBTableAnalyser class
static REALproperty analDBTableProperties[] = {
	{ nil, "ImplementsCloseTable", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbTableDefinition,closeTable) },
	{ nil, "ImplementsTableCursor", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbTableDefinition,tableCursor) },
};

// properties of PluginDBCursorAnalyser class
static REALproperty analDBCursorProperties[] = {
	{ nil, "ImplementsCloseCursor", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,closeCursor) },
	{ nil, "ImplementsColumnCount", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorColumnCount) },
	{ nil, "ImplementsColumnName", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorColumnName) },
	{ nil, "ImplementsRowCount", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorRowCount) },
	{ nil, "ImplementsColumnValue", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorColumnValue) },
	{ nil, "ImplementsReleaseValue", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorReleaseValue) },
	{ nil, "ImplementsNextRow", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorNextRow) },
	{ nil, "ImplementsDelete", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorDelete) },
	{ nil, "ImplementsDeleteAll", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorDeleteAll) },
	{ nil, "ImplementsFieldKey", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorFieldKey) },
	{ nil, "ImplementsUpdate", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorUpdate) },
	{ nil, "ImplementsEdit", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorEdit) },
};


static REALclassDefinition databaseAnalClass = {
	kCurrentREALControlVersion,
	"PluginDatabaseAnalyser", nil,
	sizeof(databaseAnalClassData), 0,
	REALnoImplementation, REALnoImplementation,
	nil, 0,
	analDBMethods, sizeof(analDBMethods) / sizeof(REALmethodDefinition),
};

static REALclassDefinition dbEngineAnalClass = {
	kCurrentREALControlVersion,
	"PluginDBEngineAnalyser", "PluginDatabaseAnalyser",
	nil, 0,
	REALnoImplementation, REALnoImplementation,
	analDBEngineProperties, sizeof(analDBEngineProperties) / sizeof(REALproperty),
	nil, 0,
};

static REALclassDefinition dbTableAnalClass = {
	kCurrentREALControlVersion,
	"PluginDBTableAnalyser", "PluginDatabaseAnalyser",
	nil, 0,
	REALnoImplementation, REALnoImplementation,
	analDBTableProperties, sizeof(analDBTableProperties) / sizeof(REALproperty),
	nil, 0,
};

static REALclassDefinition dbCursorAnalClass = {
	kCurrentREALControlVersion,
	"PluginDBCursorAnalyser", "PluginDatabaseAnalyser",
	nil, 0,
	REALnoImplementation, REALnoImplementation,
	analDBCursorProperties, sizeof(analDBCursorProperties) / sizeof(REALproperty),
	nil, 0,
};


//*********************************************************************************

//---------------------------------------
// The Plugin Analyser Interface (class)
//---------------------------------------

#define UnknownRequestName "UnknownRequest"
#define RegisterVersionName "RegisterVersion"
#define RegisterMethodName "RegisterMethod"
#define RegisterClassName "RegisterClass"
#define RegisterControlName "RegisterControl"
#define RegisterDBEngineName "RegisterDBEngine"
#define RegisterDBTableName "RegisterDBTable"
#define RegisterDBCursorName "RegisterDBCursor"
#define PluginLoadErrName "LoadError"
#define PluginOpenName "Open"
#define PluginCloseName "Close"

static REALevent analyserEvents[] = {
	{ UnknownRequestName"(name as String)" },
	{ RegisterVersionName"(v as Integer)" },
	{ RegisterMethodName"(defn as String)" },
	{ RegisterClassName"(name as String, defn as PluginClassAnalyser)" },
	{ RegisterControlName"(name as String, defn as PluginControlAnalyser)" },
	{ RegisterDBEngineName"(defn as PluginDBEngineAnalyser)" },
	{ RegisterDBTableName"(defn as PluginDBTableAnalyser)" },
	{ RegisterDBCursorName"(defn as PluginDBCursorAnalyser)" },
	{ PluginLoadErrName"(rsrcName as String, rsrcID as Integer, errMsg as String)" },
	{ PluginOpenName"(rsrcName as String, rsrcID as Integer, cFragConnID as Integer)" },
	{ PluginCloseName"(successful as Boolean)" },
};

static REALclassDefinition analClass = {
	kCurrentREALControlVersion,
	"PluginAnalyser", nil,
	0, 0,
	REALnoImplementation, REALnoImplementation,
	nil, 0,
	nil, 0,
	analyserEvents, sizeof(analyserEvents) / sizeof(REALevent),
};

//---------------------------------------------------------
// definitions for the (now obsolete) plugin API of RB 1.x
//---------------------------------------------------------

struct REALmethodDefinitionV1
{
	int exportIndex;
	const char *declaration;
};

struct REALexportV1
{
	const char *entryName;	// this is not set up yet when registerPluginExports() gets called!
	void *proc;
	int flags;
};


//-----------------------
// global RB methods
//-----------------------

typedef void (*StringProc) (REALobject instance, REALstring str);
typedef void (*IntProc) (REALobject instance, RBinteger value);
typedef void (*ClassProc) (REALobject instance, REALstring name, REALobject classAnalyser);
typedef void (*ControlProc) (REALobject instance, REALstring name, REALobject classAnalyser);
typedef void (*DBEngineProc) (REALobject instance, REALobject classAnalyser);
typedef void (*DBTableProc) (REALobject instance, REALobject classAnalyser);
typedef void (*DBCursorProc) (REALobject instance, REALobject classAnalyser);
typedef void (*LoadErrProc) (REALobject instance, REALstring rsrcName, RBinteger rsrcID, REALstring errMsg);
typedef void (*OpenProc) (REALobject instance, REALstring rsrcName, RBinteger rsrcID, RBinteger cFragConnID);
typedef void (*BoolProc) (REALobject instance, RBboolean successful);


struct browserData
{
	REALobject	instance;
	long		pluginVersion;

	StringProc		unknownRequestEvent;
	IntProc			registerVersionEvent;
	StringProc		registerMethodEvent;
	ClassProc		registerClassEvent;
	ControlProc		registerControlEvent;
	DBEngineProc 	registerDBEngineEvent;
	DBTableProc 	registerDBTableEvent;
	DBCursorProc 	registerDBCursorEvent;
	LoadErrProc		pluginLoadErr;
	OpenProc		pluginOpen;
	BoolProc		pluginClose;
};

static browserData *currentBrowser;

static void registerMethod (REALmethodDefinition *defn)
{
	EnterCallback();
	const char *name;
	if (currentBrowser->pluginVersion <= 1) {
		name = ((REALmethodDefinitionV1*)defn)->declaration;
	} else {
		name = defn->declaration;
	}
	REALstring s = REALBuildString (name, strlen(name));
	if (currentBrowser->registerMethodEvent) currentBrowser->registerMethodEvent (currentBrowser->instance, s);
	REALUnlockString (s);
	ExitCallback();
}

static void registerClass (REALclassDefinition *defn)
{
	EnterCallback();
	REALstring s = REALBuildString (defn->name, strlen(defn->name));
	REALobject classAnal = REALnewInstance (classAndCtrlAnalClass.name);
	if (classAnal) {
		ClassData (classAndCtrlAnalClass, classAnal, classAndCtrlAnalClassData, ca);
		ca->rbclass = defn;
		ca->type = classType;
		ca->isExt = false;
		if (currentBrowser->registerClassEvent) currentBrowser->registerClassEvent (currentBrowser->instance, s, classAnal);
		REALUnlockObject (classAnal);
	}
	if (s) REALUnlockString (s);
	ExitCallback();
}

static void registerClassExtension (REALclassDefinition *defn)
{
	EnterCallback();
	REALstring s = REALBuildString (defn->name, strlen(defn->name));
	REALobject classAnal = REALnewInstance (classAndCtrlAnalClass.name);
	if (classAnal) {
		ClassData (classAndCtrlAnalClass, classAnal, classAndCtrlAnalClassData, ca);
		ca->rbclass = defn;
		ca->type = classType;
		ca->isExt = true;
		if (currentBrowser->registerClassEvent) currentBrowser->registerClassEvent (currentBrowser->instance, s, classAnal);
		REALUnlockObject (classAnal);
	}
	if (s) REALUnlockString (s);
	ExitCallback();
}

static void registerControl (REALcontrol *defn)
{
	EnterCallback();
	REALstring s = REALBuildString (defn->name, strlen(defn->name));
	REALobject ctrlAnal = REALnewInstance (ctrlAnalClass.name);
	if (ctrlAnal) {
		ClassData (classAndCtrlAnalClass, ctrlAnal, classAndCtrlAnalClassData, ca);
		ca->rbcntl = defn;
		ca->type = controlType;
		ca->isExt = false;
		if (currentBrowser->registerControlEvent) currentBrowser->registerControlEvent (currentBrowser->instance, s, ctrlAnal);
		REALUnlockObject (ctrlAnal);
	}
	if (s) REALUnlockString (s);
	ExitCallback();
}

static void registerDBEngine (REALdbEngineDefinition *defn)
{
	EnterCallback();
	REALstring s = REALBuildString ((char*)defn->forSystemUse, strlen((char*)defn->forSystemUse));
	REALobject dbAnal = REALnewInstance (dbEngineAnalClass.name);
	if (dbAnal) {
		ClassData (databaseAnalClass, dbAnal, databaseAnalClassData, db);
		db->type = dbEngineType;
		db->rbDBngn = defn;
		if (currentBrowser->registerDBEngineEvent) currentBrowser->registerDBEngineEvent (currentBrowser->instance, dbAnal);
		REALUnlockObject (dbAnal);
	}
	if (s) REALUnlockString (s);
	ExitCallback();
}

static void registerDBTable (REALdbTableDefinition *defn)
{
	EnterCallback();
	REALobject dbAnal = REALnewInstance (dbTableAnalClass.name);
	if (dbAnal) {
		ClassData (databaseAnalClass, dbAnal, databaseAnalClassData, db);
		db->type = dbTableType;
		db->rbDBtbl = defn;
		if (currentBrowser->registerDBTableEvent) currentBrowser->registerDBTableEvent (currentBrowser->instance, dbAnal);
		REALUnlockObject (dbAnal);
	}
	ExitCallback();
}

static void registerDBCursor (REALdbCursorDefinition *defn)
{
	EnterCallback();
	REALobject dbAnal = REALnewInstance (dbCursorAnalClass.name);
	if (dbAnal) {
		ClassData (databaseAnalClass, dbAnal, databaseAnalClassData, db);
		db->type = dbCursorType;
		db->rbDBcrsr = defn;
		if (currentBrowser->registerDBCursorEvent) currentBrowser->registerDBCursorEvent (currentBrowser->instance, dbAnal);
		REALUnlockObject (dbAnal);
	}
	ExitCallback();
}

static void registerPluginExports (REALexportV1 *defn)
{
	// we can ignore this one (called by RB v1.0 plugins to report the method entry point table)
}

static void registerPluginVersion (long v)
{
	EnterCallback();
	currentBrowser->pluginVersion = v;
	if (currentBrowser->registerVersionEvent) currentBrowser->registerVersionEvent (currentBrowser->instance, v);
	ExitCallback();
}

static void	registerEventFilter (REALEventCallback callback, long param)
{
	// there's no interesting user information in this call
}

static void	registerPluginMarkSocketUsage (void)
{
	// there's no interesting user information in this call
}

static void	registerDataSourceInterface(const char *szMenuName, REALDataSourceInterfaceProc proc)
{
	// is there something in this that we want to tell the user?
}

static void	registerDataSource(const char *szDatasourceName, REALDataSourceProc proc)
{
	// is there something in this that we want to tell the user?
}

static int registerREALenterMovies (void)	
{
	// there's no interesting user information in this call
	return 0;
}

static jmp_buf gJmpBuf;

static void abortPlugin ()
{
	longjmp (gJmpBuf, 1);
}

static void* resolver (const char *entryName)
{
	EnterCallback();
	if (strcmp (entryName, "ResolverPPC") == 0) {
		ExitCallback();
		return (void*) resolver;
	}

	if (strcmp (entryName, "RegisterPluginVersion") == 0) {
		ExitCallback();
		return (void*) registerPluginVersion;
	}
	if (strcmp (entryName, "PluginRegisterMethod") == 0) {
		ExitCallback();
		return (void*) registerMethod;
	}
	if (strcmp (entryName, "PluginRegisterClass") == 0) {
		ExitCallback();
		return (void*) registerClass;
	}
	if (strcmp (entryName, "PluginRegisterClassExtension") == 0) {
		ExitCallback();
		return (void*) registerClassExtension;
	}
	if (strcmp (entryName, "PluginRegisterControl") == 0) {
		ExitCallback();
		return (void*) registerControl;
	}
	if (strcmp (entryName, "PluginRegisterDBEngine") == 0) {
		ExitCallback();
		return (void*) registerDBEngine;
	}
	if (strcmp (entryName, "PluginRegisterDBTable") == 0) {
		ExitCallback();
		return (void*) registerDBTable;
	}
	if (strcmp (entryName, "PluginRegisterDBCursor") == 0) {
		ExitCallback();
		return (void*) registerDBCursor;
	}
	if (strcmp (entryName, "RegisterPluginExports") == 0) {
		ExitCallback();
		return (void*) registerPluginExports;
	}	
	if (strcmp (entryName, "PluginRegisterEventFilterPPC") == 0) {
		ExitCallback();
		return (void*) registerEventFilter;
	}	
	if (strcmp (entryName, "PluginMarkSocketUsage") == 0) {
		ExitCallback();
		return (void*) registerPluginMarkSocketUsage;
	}	
	if (strcmp (entryName, "RegisterDataSourceInterface") == 0) {
		ExitCallback();
		return (void*) registerDataSourceInterface;
	}	
	if (strcmp (entryName, "RegisterDataSource") == 0) {
		ExitCallback();
		return (void*) registerDataSource;
	}	
	if (strcmp (entryName, "REALenterMovies") == 0) {
		ExitCallback();
		return (void*) registerREALenterMovies;
	}	

	// Error - we got an unhandled entryName!
	{
		REALstring s = REALBuildString (entryName, strlen(entryName));
		if (currentBrowser->unknownRequestEvent) currentBrowser->unknownRequestEvent (currentBrowser->instance, s);
		REALUnlockString (s);
	}
	ExitCallback();
	return (void*) abortPlugin;	// will return from the plugin when this requested routine is called
}

static UniversalProcPtr gOldExitToShell, gNewExitToShell;
static Boolean gSimulatingExitToShell;

static void deInstallExitToShellInterceptor ()
{
	if (gOldExitToShell) SetToolTrapAddress (gOldExitToShell, _ExitToShell);
	gOldExitToShell = nil;
}

static pascal void interceptExitToShell (void)
{
	EnterCallback();
	if (!gSimulatingExitToShell) {
		deInstallExitToShellInterceptor ();
		ExitCallback();
		ExitToShell ();
	}
	ExitCallback();
}

static void installExitToShellInterceptor ()
{
	gSimulatingExitToShell = false;
	gNewExitToShell = NewRoutineDescriptor ((ProcPtr)interceptExitToShell, kPascalStackBased, GetCurrentISA());
	gOldExitToShell = NGetTrapAddress (_ExitToShell, 1);
	SetToolTrapAddress (gNewExitToShell, _ExitToShell);
}

typedef void (*PluginMainProcPtr) (RoutineDescriptor*);

static void loadPlugin (Handle plHdl)
{
	HLockHi (plHdl);
	CFragConnectionID myConnID;
	Str255 errMsg;
	PluginMainProcPtr plMain;
	Boolean ok = false;

	short rsrcID;
	Str255 rsrcName2;
	OSType rsrcType;
	GetResInfo (plHdl, &rsrcID, &rsrcType, rsrcName2);
	REALstring rsrcName = REALBuildString ((Ptr)&rsrcName2[1], rsrcName2[0]);

	long ofs = *(long*)(*plHdl+0x14);
	OSErr err = GetMemFragment (*plHdl+ofs, GetHandleSize(plHdl)-ofs, "\ploaded plugin", kPrivateCFragCopy, &myConnID, (Ptr*)&plMain, errMsg);
	if (!err) {
		if (currentBrowser->pluginOpen) currentBrowser->pluginOpen (currentBrowser->instance, rsrcName, rsrcID, (long)myConnID);

		RoutineDescriptor resolverPPC = BUILD_ROUTINE_DESCRIPTOR (kThinkCStackBased | RESULT_SIZE(SIZE_CODE(4)) | STACK_ROUTINE_PARAMETER(1, SIZE_CODE(4)), resolver);
		installExitToShellInterceptor ();
		if (setjmp (gJmpBuf) == 0) {
			// call the plugin's main() function
			plMain (&resolverPPC);
			ok = true;
		} else {
			// we get here if the plugin called a resolved function that was unknown to us.
			ok = false;
		}
		// It is possible that the plugin mainEntry code has installed a trap patch to ExitToShell in order to
		// let go of some resources it might have permanently opened. To make the plugin release these resources,
		// we simulate a ExitToShell call here now:
		gSimulatingExitToShell = true;
		ExitToShell ();
		deInstallExitToShellInterceptor ();
		CloseConnection (&myConnID);

		if (currentBrowser->pluginClose) currentBrowser->pluginClose (currentBrowser->instance, ok);
	} else {
		REALstring s = REALBuildString ((Ptr)&errMsg[1], errMsg[0]);
		if (currentBrowser->pluginLoadErr) currentBrowser->pluginLoadErr (currentBrowser->instance, rsrcName, rsrcID, s);
		REALUnlockString (s);
	}

	REALUnlockString (rsrcName);
}


static long browsePlugin (REALfolderItem plugFile, REALobject browserInstance)
{
	FSSpec		plSpec;
	Boolean		ok;
	
	PrepareCallback();
	
	ok = REALFSSpecFromFolderItem(&plSpec, plugFile);

	if (ok) {
		browserData data;
		
		data.pluginVersion = 0;
		data.instance = browserInstance;
		data.unknownRequestEvent = (StringProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[0]);
		data.registerVersionEvent = (IntProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[1]);
		data.registerMethodEvent = (StringProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[2]);
		data.registerClassEvent = (ClassProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[3]);
		data.registerControlEvent = (ControlProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[4]);
		data.registerDBEngineEvent = (DBEngineProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[5]);
		data.registerDBTableEvent = (DBTableProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[6]);
		data.registerDBCursorEvent = (DBCursorProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[7]);
		data.pluginLoadErr = (LoadErrProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[8]);
		data.pluginOpen = (OpenProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[9]);
		data.pluginClose = (BoolProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[10]);

		short prevRF = CurResFile ();
		short f = FSpOpenResFile (&plSpec, fsRdPerm);
		if (f == -1) {
			// failed to open file
			ok = false;
		} else {
			UseResFile (f);
			OSType type = 'PLPC';	// we're looking for PPC plugins only
			for (int i = 1; i <= Count1Resources(type); ++i) {
				Handle h = Get1IndResource (type, i);
				if (h) {
					browserData *prevBrowser = currentBrowser;	// this provides for recursions into this method
					currentBrowser = &data;
					loadPlugin (h);
					currentBrowser = prevBrowser;
					ReleaseResource (h);
				} else {
					// failed to load a resource that's supposed to be available - this could happen if we're low on memory
					ok = false;
				}
			}
			CloseResFile (f);
		}
		UseResFile (prevRF);
	}
	return ok;
}

static REALmethodDefinition pluginMethodArray[] = {
	{ (REALproc) browsePlugin, REALnoImplementation, "BrowsePlugin(plugin as FolderItem, browser as PluginAnalyser) as Boolean"},
};


// ---------------------------------
// ----- Plugin Initialization -----
// ---------------------------------

void PluginEntry (void)
{
	REALRegisterClass (&classAndCtrlAnalClass);
	REALRegisterClass (&ctrlAnalClass);
	REALRegisterClass (&databaseAnalClass);
	REALRegisterClass (&dbEngineAnalClass);
	REALRegisterClass (&dbTableAnalClass);
	REALRegisterClass (&dbCursorAnalClass);
	REALRegisterClass (&analClass);
	for (int i = 0; i < sizeof(pluginMethodArray) / sizeof(REALmethodDefinition); ++i) {
		REALRegisterMethod (&pluginMethodArray[i]);
	}
}
