
/*
 * Plugin for RB 2.1 or higher
 *
 * Alfred Van Hoek is the mainainer of this code. Whoever makes changes to it
 * is asked to FIRST contact Alfred in order to make sure he has the most
 * current version, especially if he wants to contribute his changes later!
 * His email address is: <mailto:vanhoek@mac.com>
 *
 * Change history
 * --------------
 *  - Originally developed by Thomas Tempelmann (rb@tempel.org), 13 July 2000
 *
 *  - Enhanced July 14-20 by Alfred Van Hoek and Bjrn Eirksson
 *
 *  - Carbonized by Alfred Van Hoek, 2 October 2000. version 1.3
 *  - little additions, i.e., resolverCalls that the plugin did not handle: Febr/Mar 2001. version 1.4
 *  - added a PL68 resource call, and made it compatible? with 68k: Febr/Mar 2001. version 1.5.
 *
 *  - 68K support added by TT, 19 March 2001, version 1.6
 *
 *  - fixed a bug with respect to obtaining information on implementation of methods and properties.
 *	  now correct getter/setter/editor.  AvH, 25 March 2001, version 1.7
 */


/*
 * How it works
 *--------------
 *
 *  Well, it's not a secret, but we haven't taken the effort to
 *  document it here, either. Perhaps you can understand it by
 *  looking at this source code. Start with browsePlugin() and
 *  loadPlugin().
 */

//----------------------
// common includes here
//----------------------
#include <string.h>
#include <setjmp.h>
#include <CodeFragments.h>
#include <MacMemory.h>
#if TARGET_CPU_68K	
	#include <SetupA4.h>
#else
	#define PrepareCallback() 
	#define EnterCallback()
	#define ExitCallback()
#endif

#include "DConPrefix.h"

//-------------------------
// RB header includes here
//-------------------------
#include "rb_plugin.h"

//-----------------------
// Some type definitions
//-----------------------

typedef long RBinteger;
typedef Boolean RBboolean;

//*********************************************************************************

enum ClassDefnTypes {
	classType, controlType, dbEngineType, dbTableType, dbCursorType
};

enum {
	kVersion1 = 1,
	kVersion2,
	kVersion3,
	kVersion4,
	kVersion5
};


//------------------------------------------------------------
// Class Analyser Class (also supports Control Analyer Class)
//------------------------------------------------------------
extern REALclassDefinition globalAnalClass;
extern REALclassDefinition classAndCtrlAnalClass;
extern REALclassDefinition ctrlAnalClass;

struct 	globalAnalClassData
{	
	long					*version;
	REALmethodDefinition	*rbmethod;
};

static RBinteger getGlobalStructVersion (REALobject instance)
{
	ClassData (globalAnalClass, instance, globalAnalClassData, me);
	return *me->version;
}

static REALstring getglobalMethod (REALobject instance)
{
	ClassData (globalAnalClass, instance, globalAnalClassData, me);
	const char* name = nil;	
	if (me->rbmethod != nil) {
		name = me->rbmethod->declaration;
	}
	return REALBuildString (name, name?strlen(name):0);
}

static RBboolean tstglobalMethodProcsAsBoolean (REALobject instance, long param)
{
	ClassData (globalAnalClass, instance, globalAnalClassData, me);
	REALproc proc = nil;	
	if (me->rbmethod != nil) {
		proc = (REALproc) *((char*)me->rbmethod + param);
	} 
	return proc != nil;
}


static REALmethodDefinition globalanalClMethods[] = {
	{ (REALproc) getGlobalStructVersion, REALnoImplementation, "GlobalVersion() as Integer" },
	{ (REALproc) getglobalMethod, REALnoImplementation, "Method() as String" },
};	
	
static REALproperty globalanalClProperties[] = {
	{ nil, "hasMethodGetter", "Boolean", 0, (REALproc)tstglobalMethodProcsAsBoolean, REALnoImplementation, FieldOffset(REALmethodDefinition, function) },
	{ nil, "hasMethodSetter", "Boolean", 0, (REALproc)tstglobalMethodProcsAsBoolean, REALnoImplementation, FieldOffset(REALmethodDefinition, setterFunction) },
};	
	
static REALclassDefinition globalAnalClass = {
	kCurrentREALControlVersion,
	"PluginGlobalsAnalyser", nil,
	sizeof(globalAnalClassData), 0,
	REALnoImplementation, REALnoImplementation,
	globalanalClProperties, sizeof(globalanalClProperties) / sizeof(REALproperty),
	globalanalClMethods, sizeof(globalanalClMethods) / sizeof(REALmethodDefinition),
};
	
	
	
	
//------------------------------------------------------------
struct classAndCtrlAnalClassData
{
	union {
		long					*version;	// the version is always the fist field in all plugin-defined RB structs
		REALcontrol				*rbcntl;
		REALclassDefinition		*rbclass;
	};
	ClassDefnTypes	type;
	RBboolean		isExt;
};


//------------------------------------------------------------
static RBinteger getStructVersion (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	return *me->version;
}


static REALstring getSuper (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	const char* name = nil;	
	if (me->type == classType) {
		name = me->rbclass->superName;
	}
	return REALBuildString (name, name?strlen(name):0);
}

static RBinteger methodCnt (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	int rbint = 0;
	if (me->type == classType) {
		rbint = me->rbclass->methodCount;
	} else if (me->type == controlType) {
		rbint = me->rbcntl->methodCount;
	}
	return rbint;
}

static REALstring getMethod (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	const char* name = nil;	
	if (me->type == classType) {
		name = me->rbclass->methods[idx-1].declaration;
	} else if (me->type == controlType) {
		name = me->rbcntl->methods[idx-1].declaration;
	}
	return REALBuildString (name, name?strlen(name):0);
}

static RBinteger propertyCnt (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	int rbint = 0;
	if (me->type == classType) {
		rbint = me->rbclass->propertyCount;
	} else if (me->type == controlType) {
		rbint = me->rbcntl->propertyCount;
	}
	return rbint;
}

static REALstring getPropertyName (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	const char* name = nil;
	if (me->type == classType) {
		name = me->rbclass->properties[idx-1].name;
	} else if (me->type == controlType) {
		name = me->rbcntl->properties[idx-1].name;
	}
	return REALBuildString (name, name?strlen(name):0);
}

static REALstring getPropertyType (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	const char* name = nil;
	if (me->type == classType) {
		name = me->rbclass->properties[idx-1].type;
	} else if (me->type == controlType) {
		name = me->rbcntl->properties[idx-1].type;
	}
	return REALBuildString (name, name?strlen(name):0);
}

static RBinteger eventCnt (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	int rbint = 0;
	if (me->type == classType) {
		rbint = me->rbclass->eventCount;
	} else if (me->type == controlType) {
		rbint = me->rbcntl->eventCount;
	}
	return rbint;
}

static REALstring getEvent (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	const char* name;	
	if (me->type == classType) {
		name = me->rbclass->events[idx-1].declaration;
	} else if (me->type == controlType) {
		name = me->rbcntl->events[idx-1].declaration;
	}
	return REALBuildString (name, name?strlen(name):0);
}

static RBinteger eventInstanceCnt (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	int rbint = 0;
	if (me->type == classType) {
		if (me->rbclass->version >= kVersion3) {
			rbint = me->rbclass->eventInstanceCount;
		}
	} else if (me->type == controlType) {
		if (me->rbcntl->version >= kVersion4) {
			rbint = me->rbcntl->eventInstanceCount;
		}
	}
	return rbint;
}

static REALstring getEventInstance (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	const char* name = nil;
	if (me->type == classType) {
		if (me->rbclass->version >= kVersion3) {
			name = me->rbclass->eventInstances[idx-1].name;
		}
	} else if (me->type == controlType) {
		if (me->rbcntl->version >= kVersion4) {
			name = me->rbcntl->eventInstances[idx-1].name;
		}
	}
	return REALBuildString (name, name?strlen(name):0);
}


static RBinteger bindDescriptionCnt (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	int rbint = 0;
	if (me->type == classType) {
		if (me->rbclass->version >= kVersion4) {
			rbint = me->rbclass->bindDescriptionCount;
		}
	} else if (me->type == controlType) {
		if (me->rbcntl->version >= kVersion4) {
			rbint = me->rbcntl->bindDescriptionCount;
		}
	}
	return rbint;
}

static REALstring getbindInterface (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	const char* name = nil;
	if (me->type == classType) {
		if (me->rbclass->version >= kVersion5) {
			name = me->rbclass->bindDescriptions[idx-1].szInterface;
		}
	} else if (me->type == controlType) {
		if (me->rbcntl->version >= kVersion5) {
			name = me->rbcntl->bindDescriptions[idx-1].szInterface;
		}
	}
	return REALBuildString (name, name?strlen(name):0);
}

static REALstring getbindDescription (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	const char* name = nil;
	if (me->type == classType) {
		if (me->rbclass->version >= kVersion5) {
			name = me->rbclass->bindDescriptions[idx-1].szDescription;
		}
	} else if (me->type == controlType) {
		if (me->rbcntl->version >= kVersion5) {
			name = me->rbcntl->bindDescriptions[idx-1].szDescription;
		}
	}
	return REALBuildString (name, name?strlen(name):0);
}

static REALstring getbindPart (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	const char* name = nil;
	if (me->type == classType) {
		if (me->rbclass->version >= kVersion5) {
			name = me->rbclass->bindDescriptions[idx-1].szPart;
		}
	} else if (me->type == controlType) {
		if (me->rbcntl->version >= kVersion5) {
			name = me->rbcntl->bindDescriptions[idx-1].szPart;
		}
	}
	return REALBuildString (name, name?strlen(name):0);
}

static RBinteger propertyFlags (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	int rbint = 0;
	if (me->type == classType) {
		rbint = me->rbclass->properties[idx-1].flags;
	} else if (me->type == controlType) {
		rbint = me->rbcntl->properties[idx-1].flags;
	}
	return rbint;
}

static REALstring getInterfaces (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	const char* name = nil;
	if (me->type == classType && me->rbclass->version >= kVersion3) {
		name = me->rbclass->interfaces;
	} else if (me->type == controlType && me->rbcntl->version >= kVersion5) {
		name = me->rbcntl->interfaces;
	}
	return REALBuildString (name, name?strlen(name):0);
}

static RBboolean tstConstructor (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	REALproc proc = nil;	
	if (me->type == controlType && me->rbcntl->behaviour != nil) {
		proc = (REALproc) me->rbcntl->behaviour->constructorFunction;
	} else if (me->type == classType) {
		proc = me->rbclass->constructor;
	}
	return proc != nil;
}

static RBboolean tstDestructor (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	REALproc proc = nil;	
	if (me->type == controlType && me->rbcntl->behaviour != nil) {
		proc = (REALproc) me->rbcntl->behaviour->destructorFunction;
	} else if (me->type == classType) {
		proc = me->rbclass->destructor;
	}
	return proc != nil;
}

//------------------------------------------------------------

static RBinteger controlFlags (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	int rbint = 0;
	if (me->type == controlType) {
		rbint = me->rbcntl->flags;
	}
	return rbint;
}

static RBboolean tstLongAsBoolean (REALobject instance, long param)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	REALproc proc = nil;	
	if (me->type == controlType && me->rbcntl->behaviour != nil) {
		proc = (REALproc) *((char*)me->rbcntl->behaviour + param);
	}
	return proc != nil;
}

static RBinteger getParamAsInt (REALobject instance, long param)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	int rbint = 0;
	if (me->type == controlType) {
		rbint = (int)*((char*)me->rbcntl+param);
	}
	return rbint;
}

static RBinteger getToolBarPict (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	int rbint = 0;
	if (me->type == controlType) {
		rbint = me->rbcntl->toolbarPICT;
	}
	return rbint;
}

static RBinteger getToolBarDownPict (REALobject instance)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	int rbint = 0;
	if (me->type == controlType) {
		rbint = me->rbcntl->toolbarDownPICT;
	}
	return rbint;
}

/*
static RBboolean tstPropertyProcsAsBoolean (REALobject instance, long param)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	REALproc proc = nil;	
	if (me->type == classType && me->rbclass->properties != nil) {
		proc = (REALproc) *((char*)me->rbclass->properties + param);
	} 
	if (me->type == controlType && me->rbcntl->properties != nil) {
		proc = (REALproc) *((char*)me->rbcntl->properties + param);
	} 
	return proc != nil;
}
*/

static RBboolean tstPropertyGetterProcsAsBoolean (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	REALproc proc = nil;	
	if (me->type == classType && me->rbclass->properties != nil) {
		proc = me->rbclass->properties[idx-1].getter;
	} 
	if (me->type == controlType && me->rbcntl->properties != nil) {
		proc = me->rbcntl->properties[idx-1].getter;
	} 
	return proc != nil;
}

static RBboolean tstPropertySetterProcsAsBoolean (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	REALproc proc = nil;	
	if (me->type == classType && me->rbclass->properties != nil) {
		proc = me->rbclass->properties[idx-1].setter;
	} 
	if (me->type == controlType && me->rbcntl->properties != nil) {
		proc = me->rbcntl->properties[idx-1].setter;
	} 
	return proc != nil;
}

static RBboolean tstPropertyEditorProcsAsBoolean (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	REALproc proc = nil;	
	if (me->type == classType && me->rbclass->properties != nil) {
		proc = me->rbclass->properties[idx-1].editor;
	} 
	if (me->type == controlType && me->rbcntl->properties != nil) {
		proc = me->rbcntl->properties[idx-1].editor;
	} 
	return proc != nil;
}

/*
static RBboolean tstMethodProcsAsBoolean (REALobject instance, long param)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	REALproc proc = nil;	
	if (me->type == classType && me->rbclass->methods != nil) {
		proc = (REALproc) *((char*)me->rbclass->methods + param);
	} 
	if (me->type == controlType && me->rbcntl->methods != nil) {
		proc = (REALproc) *((char*)me->rbcntl->methods + param);
	} 
	return proc != nil;
}
*/

static RBboolean tstMethodFunctionProcsAsBoolean (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	REALproc proc = nil;	
	if (me->type == classType && me->rbclass->methods != nil) {
		proc = me->rbclass->methods[idx-1].function;
	} 
	if (me->type == controlType && me->rbcntl->methods != nil) {
		proc = me->rbcntl->methods[idx-1].function;
	} 
	return proc != nil;
}

static RBboolean tstMethodSetterFunctionProcsAsBoolean (REALobject instance, RBinteger idx)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	REALproc proc = nil;	
	if (me->type == classType && me->rbclass->methods != nil) {
		proc = me->rbclass->methods[idx-1].setterFunction;
	} 
	if (me->type == controlType && me->rbcntl->methods != nil) {
		proc = me->rbcntl->methods[idx-1].setterFunction;
	} 
	return proc != nil;
}

static RBboolean tstExtension (REALobject instance, long param)
{
	ClassData (classAndCtrlAnalClass, instance, classAndCtrlAnalClassData, me);
	
	return me->isExt;
}


//------------------------------------------------------------

//  --- methods for PluginClassAnalyser and PluginControlAnalyser ---
static REALmethodDefinition analClMethods[] = {
	{ (REALproc) getStructVersion, REALnoImplementation, "Version() as Integer" },
	{ (REALproc) getSuper, REALnoImplementation, "Super() as String" },
	{ (REALproc) methodCnt, REALnoImplementation, "MethodCount() as Integer" },
	{ (REALproc) getMethod, REALnoImplementation, "Method(idx as Integer) as String" },
	{ (REALproc) propertyCnt, REALnoImplementation, "PropertyCount() as Integer" },
	{ (REALproc) getPropertyName, REALnoImplementation, "PropertyName(idx as Integer) as String" },
	{ (REALproc) getPropertyType, REALnoImplementation, "PropertyType(idx as Integer) as String" },
	{ (REALproc) propertyFlags, REALnoImplementation, "PropertyFlags(idx as Integer) as Integer" },
	{ (REALproc) eventCnt, REALnoImplementation, "EventCount() as Integer" },
	{ (REALproc) getEvent, REALnoImplementation, "Event(idx as Integer) as String" },
	{ (REALproc) eventInstanceCnt, REALnoImplementation, "EventInstanceCount() as Integer" },
	{ (REALproc) getEventInstance, REALnoImplementation, "EventInstance(idx as Integer) as String" },
	{ (REALproc) getInterfaces, REALnoImplementation, "Interfaces() as String" },
	{ (REALproc) bindDescriptionCnt, REALnoImplementation, "BindDescriptionCount() as Integer" },
	{ (REALproc) getbindInterface, REALnoImplementation, "BindInterface(idx as Integer) as String" },
	{ (REALproc) getbindDescription, REALnoImplementation, "BindDescription(idx as Integer) as String" },
	{ (REALproc) getbindPart, REALnoImplementation, "BindPart(idx as Integer) as String" },

	{ (REALproc) tstConstructor, REALnoImplementation, "ImplementsConstructor() as Boolean" },
	{ (REALproc) tstDestructor, REALnoImplementation, "ImplementsDestructor() as Boolean" },
	
	{ (REALproc) tstPropertyGetterProcsAsBoolean, REALnoImplementation, "ImplementsPropertyGetter(idx as integer) as Boolean" },
	{ (REALproc) tstPropertySetterProcsAsBoolean, REALnoImplementation, "ImplementsPropertySetter(idx as integer) as Boolean" },
	{ (REALproc) tstPropertyEditorProcsAsBoolean, REALnoImplementation, "ImplementsPropertyEditor(idx as integer) as Boolean"  },
	{ (REALproc) tstMethodFunctionProcsAsBoolean, REALnoImplementation, "ImplementsMethodGetter(idx as integer) as Boolean" },
	{ (REALproc) tstMethodSetterFunctionProcsAsBoolean, REALnoImplementation, "ImplementsMethodSetter(idx as integer) as Boolean" },
	
};

//  --- properties for PluginClassAnalyser and PluginControlAnalyser ---

static REALproperty analClProperties[] = {
	{ nil, "IsClassExtension", "Boolean", 0, (REALproc)tstExtension, REALnoImplementation, 0 },
/*	{ nil, "ImplementsPropertyGetter", "Boolean", 0, (REALproc)tstPropertyProcsAsBoolean, REALnoImplementation, FieldOffset(REALproperty, getter) },
	{ nil, "ImplementsPropertySetter", "Boolean", 0, (REALproc)tstPropertyProcsAsBoolean, REALnoImplementation, FieldOffset(REALproperty, setter) },
	{ nil, "ImplementsPropertyEditor", "Boolean", 0, (REALproc)tstPropertyProcsAsBoolean, REALnoImplementation, FieldOffset(REALproperty, editor) },
	{ nil, "ImplementsMethodGetter", "Boolean", 0, (REALproc)tstMethodProcsAsBoolean, REALnoImplementation, FieldOffset(REALmethodDefinition, function) },
	{ nil, "ImplementsMethodSetter", "Boolean", 0, (REALproc)tstMethodProcsAsBoolean, REALnoImplementation, FieldOffset(REALmethodDefinition, setterFunction) },
*/};

//  --- methods for PluginControlAnalyser only ---
static REALmethodDefinition analCtMethods[] = {
	{ (REALproc) controlFlags, REALnoImplementation, "ControlFlags() as Integer" },
};

//  --- properties for PluginControlAnalyser only ---
static REALproperty analCtProperties[] = {
	{ nil, "ImplementsConstructorFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, constructorFunction) },
	{ nil, "ImplementsDestructorFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, destructorFunction) },
	{ nil, "ImplementsRedrawFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, redrawFunction) },
	{ nil, "ImplementsClickFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, clickFunction) },
	{ nil, "ImplementsMouseDragFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, mouseDragFunction) },
	{ nil, "ImplementsMouseUpFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, mouseUpFunction) },
	{ nil, "ImplementsGainedFocusFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, gainedFocusFunction) },
	{ nil, "ImplementsLostFocusFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, lostFocusFunction) },
	{ nil, "ImplementsKeyDownFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, keyDownFunction) },
	{ nil, "ImplementsOpenFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, openFunction) },
	{ nil, "ImplementsCloseFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, closeFunction) },
	{ nil, "ImplementsBackgroundIdleFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, backgroundIdleFunction) },
	{ nil, "ImplementsDrawOffscreenFunction", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, drawOffscreenFunction) },
	{ nil, "ImplementsSetSpecialBackground", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, setSpecialBackground) },
	{ nil, "ImplementsConstantChanging", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, constantChanging) },
	{ nil, "ImplementsDroppedNewInstance", "Boolean", 0, (REALproc)tstLongAsBoolean, REALnoImplementation, FieldOffset(REALcontrolBehaviour, droppedNewInstance) },
	{ nil, "toolbarPICT", "Integer", 0, (REALproc)getToolBarPict, REALnoImplementation, 0/*FieldOffset(REALcontrol, toolbarPICT)*/ },
	{ nil, "toolbarDownPICT", "Integer", 0, (REALproc)getToolBarDownPict, REALnoImplementation, 0/*FieldOffset(REALcontrol, toolbarDownPICT)*/ },
};

static REALclassDefinition classAndCtrlAnalClass = {
	kCurrentREALControlVersion,
	"PluginClassAnalyser", "PluginGlobalsAnalyser",
	sizeof(classAndCtrlAnalClassData), 0,
	REALnoImplementation, REALnoImplementation,
	analClProperties, sizeof(analClProperties) / sizeof(REALproperty),
	analClMethods, sizeof(analClMethods) / sizeof(REALmethodDefinition),
};

static REALclassDefinition ctrlAnalClass = {
	kCurrentREALControlVersion,
	"PluginControlAnalyser", "PluginClassAnalyser",
	nil, 0,
	REALnoImplementation, REALnoImplementation,
	analCtProperties, sizeof(analCtProperties) / sizeof(REALproperty),
	analCtMethods, sizeof(analCtMethods) / sizeof(REALmethodDefinition),
};


//*********************************************************************************

//---------------------
// DB Analyser Classes
//---------------------

extern REALclassDefinition databaseAnalClass;

struct databaseAnalClassData
{
	union {
		long					*version;	// the version is always the fist field in all plugin-defined RB structs
		char					*structData;
		REALdbEngineDefinition	*rbDBngn;
		REALdbTableDefinition	*rbDBtbl;
		REALdbCursorDefinition	*rbDBcrsr;
	};
	ClassDefnTypes	type;
};

static RBinteger getDBStructVersion (REALobject instance)
{
	ClassData (databaseAnalClass, instance, databaseAnalClassData, me);
	return *me->version;
}

static RBboolean dbLongAsBoolean (REALobject instance, long param)
{
	ClassData (databaseAnalClass, instance, databaseAnalClassData, me);
	REALproc proc = (REALproc) *(me->structData + param);
	return proc != nil;
}

//----------------------------------------------------------------

// methods common to all DB classes
static REALmethodDefinition analDBMethods[] = {
	{ (REALproc) getDBStructVersion, REALnoImplementation, "Version() as Integer" },
};

// properties of PluginDBEngineAnalyser class
static REALproperty analDBEngineProperties[] = {
	{ nil, "ImplementsCloseDatabase", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,closeDatabase) },
	{ nil, "ImplementsGetTableSchemaCrsr", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,getTableSchemaCursor) },
	{ nil, "ImplementsGetFieldSchemaCrsr", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,getFieldSchemaCursor) },
	{ nil, "ImplementsDirectSQLSelect", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,directSQLSelect) },
	{ nil, "ImplementsDirectSQLExecute", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,directSQLExecute) },
	{ nil, "ImplementsCreateTable", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,createTable) },
	{ nil, "ImplementsAddTableRecord", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,addTableRecord) },
	{ nil, "ImplementsGetTableCursor", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,getTableCursor) },
	{ nil, "ImplementsUpdateFields", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,updateFields) },
	{ nil, "ImplementsAddTableColumn", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,addTableColumn) },
	{ nil, "ImplementsGetDatabaseIndexes", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,getDatabaseIndexes) },
	{ nil, "ImplementsGetLastErrorCode", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,getLastErrorCode) },
	{ nil, "ImplementsGetLastErrorString", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,getLastErrorString) },
	{ nil, "ImplementsCommit", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,commit) },
	{ nil, "ImplementsRollback", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,rollback) },
	{ nil, "ImplementsGetProperty", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbEngineDefinition,getProperty) },
};

// properties of PluginDBTableAnalyser class
static REALproperty analDBTableProperties[] = {
	{ nil, "ImplementsCloseTable", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbTableDefinition,closeTable) },
	{ nil, "ImplementsTableCursor", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbTableDefinition,tableCursor) },
};

// properties of PluginDBCursorAnalyser class
static REALproperty analDBCursorProperties[] = {
	{ nil, "ImplementsCloseCursor", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,closeCursor) },
	{ nil, "ImplementsColumnCount", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorColumnCount) },
	{ nil, "ImplementsColumnName", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorColumnName) },
	{ nil, "ImplementsRowCount", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorRowCount) },
	{ nil, "ImplementsColumnValue", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorColumnValue) },
	{ nil, "ImplementsReleaseValue", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorReleaseValue) },
	{ nil, "ImplementsNextRow", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorNextRow) },
	{ nil, "ImplementsDelete", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorDelete) },
	{ nil, "ImplementsDeleteAll", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorDeleteAll) },
	{ nil, "ImplementsFieldKey", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorFieldKey) },
	{ nil, "ImplementsUpdate", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorUpdate) },
	{ nil, "ImplementsEdit", "Boolean", 0, (REALproc)dbLongAsBoolean, REALnoImplementation, FieldOffset(REALdbCursorDefinition,cursorEdit) },
};


static REALclassDefinition databaseAnalClass = {
	kCurrentREALControlVersion,
	"PluginDatabaseAnalyser", "PluginGlobalsAnalyser",
	sizeof(databaseAnalClassData), 0,
	REALnoImplementation, REALnoImplementation,
	nil, 0,
	analDBMethods, sizeof(analDBMethods) / sizeof(REALmethodDefinition),
};

static REALclassDefinition dbEngineAnalClass = {
	kCurrentREALControlVersion,
	"PluginDBEngineAnalyser", "PluginDatabaseAnalyser",
	nil, 0,
	REALnoImplementation, REALnoImplementation,
	analDBEngineProperties, sizeof(analDBEngineProperties) / sizeof(REALproperty),
	nil, 0,
};

static REALclassDefinition dbTableAnalClass = {
	kCurrentREALControlVersion,
	"PluginDBTableAnalyser", "PluginDatabaseAnalyser",
	nil, 0,
	REALnoImplementation, REALnoImplementation,
	analDBTableProperties, sizeof(analDBTableProperties) / sizeof(REALproperty),
	nil, 0,
};

static REALclassDefinition dbCursorAnalClass = {
	kCurrentREALControlVersion,
	"PluginDBCursorAnalyser", "PluginDatabaseAnalyser",
	nil, 0,
	REALnoImplementation, REALnoImplementation,
	analDBCursorProperties, sizeof(analDBCursorProperties) / sizeof(REALproperty),
	nil, 0,
};


//*********************************************************************************

//---------------------------------------
// The Plugin Analyser Interface (class)
//---------------------------------------


#define UnknownRequestName "UnknownRequest"
#define RegisterVersionName "RegisterVersion"
#define RegisterMethodName "RegisterMethod"
#define RegisterClassName "RegisterClass"
#define RegisterControlName "RegisterControl"
#define RegisterDBEngineName "RegisterDBEngine"
#define RegisterDBTableName "RegisterDBTable"
#define RegisterDBCursorName "RegisterDBCursor"
#define PluginLoadErrName "LoadError"
#define PluginOpenName "Open"
#define PluginCloseName "Close"
#define PluginVersionControl "CurrentVersion"

#define RegisterDataSourceInterfaceName "RegisterDataSourceInterface"
#define RegisterDataSourceName "RegisterDataSource"

static REALevent analyserEvents[] = {
	{ RegisterDataSourceName "(name as String)" },
	{ RegisterDataSourceInterfaceName "(name as String)" },
	{ RegisterDBCursorName"(defn as PluginDBCursorAnalyser)" },
	{ RegisterDBTableName"(defn as PluginDBTableAnalyser)" },
	{ RegisterDBEngineName"(defn as PluginDBEngineAnalyser)" },
	{ RegisterControlName"(name as String, defn as PluginControlAnalyser)" },
	{ RegisterClassName"(name as String, defn as PluginClassAnalyser)" },
	{ RegisterMethodName"(name as String, defn as PluginGlobalsAnalyser)" },
	{ RegisterVersionName"(v as Integer)" },
	{ UnknownRequestName"(name as String)" },
	{ PluginLoadErrName"(rsrcName as String, rsrcID as Integer, errMsg as String)" },
	{ PluginCloseName"(successful as Boolean)" },
	{ PluginOpenName"(rsrcName as String, rsrcID as Integer, cFragConnID as Integer)" },
	{ PluginVersionControl"(versionControl as integer, url as String)"	}
};


static REALclassDefinition analClass = {
	kCurrentREALControlVersion,
	"PluginAnalyser", nil,
	0, 0,
	REALnoImplementation, REALnoImplementation,
	nil, 0,
	nil, 0,
	analyserEvents, sizeof(analyserEvents) / sizeof(REALevent),
};

//---------------------------------------------------------
// definitions for the (now obsolete) plugin API of RB 1.x
//---------------------------------------------------------

struct REALmethodDefinitionV1
{
	int exportIndex;
	const char *declaration;
};

struct REALexportV1
{
	const char *entryName;	// this is not set up yet when registerPluginExports() gets called!
	void *proc;
	int flags;
};


//-----------------------
// global RB methods
//-----------------------

typedef void (*PluginVersionControlProc) (REALobject instance, unsigned long numVersion, REALstring linkURL);

typedef void (*StringProc) (REALobject instance, REALstring str);
typedef void (*IntProc) (REALobject instance, RBinteger value);
typedef void (*ClassProc) (REALobject instance, REALstring name, REALobject classAnalyser);
typedef void (*ControlProc) (REALobject instance, REALstring name, REALobject classAnalyser);
typedef void (*DBEngineProc) (REALobject instance, REALobject classAnalyser);
typedef void (*DBTableProc) (REALobject instance, REALobject classAnalyser);
typedef void (*DBCursorProc) (REALobject instance, REALobject classAnalyser);
typedef void (*GlobalsProc) (REALobject, REALstring, REALobject);
typedef void (*LoadErrProc) (REALobject instance, REALstring rsrcName, RBinteger rsrcID, REALstring errMsg);
typedef void (*OpenProc) (REALobject instance, REALstring rsrcName, RBinteger rsrcID, RBinteger cFragConnID);
typedef void (*BoolProc) (REALobject instance, RBboolean successful);
typedef void (*DtSrcIntProc) (REALobject instance, REALstring str);
typedef void (*DtSrcProc) (REALobject instance, REALstring str);

struct browserData
{
	REALobject	instance;
	long		pluginVersion;

	
	StringProc		unknownRequestEvent;
	IntProc			registerVersionEvent;
	GlobalsProc		registerMethodEvent;
	ClassProc		registerClassEvent;
	ControlProc		registerControlEvent;
	DBEngineProc 	registerDBEngineEvent;
	DBTableProc 	registerDBTableEvent;
	DBCursorProc 	registerDBCursorEvent;
	LoadErrProc		pluginLoadErr;
	OpenProc		pluginOpen;
	BoolProc		pluginClose;
	DtSrcIntProc	dataSrcInterface;
	DtSrcProc		dataSource;
	PluginVersionControlProc	pluginNumVer;
};

static browserData *currentBrowser;


static void registerMethod (REALmethodDefinition *defn)
{
	EnterCallback();
	REALobject classAnal = REALnewInstance (globalAnalClass.name);
	if (classAnal) {
		ClassData (globalAnalClass, classAnal, globalAnalClassData, ca);
		const char *name;
		
		if (currentBrowser->pluginVersion <= 1) {
			name = ((REALmethodDefinitionV1*)defn)->declaration;
			ca->rbmethod->declaration = name;
			ca->rbmethod->setterFunction = nil;
		} else {
			name = defn->declaration;
			ca->rbmethod = defn;
		}
		REALstring s = REALBuildString (name, strlen(name));
		
		if (currentBrowser->registerMethodEvent) currentBrowser->registerMethodEvent (currentBrowser->instance, s, classAnal);
		REALUnlockObject (classAnal);
		REALUnlockString (s);			
	}
	ExitCallback();
}

static void registerClass (REALclassDefinition *defn)
{
	EnterCallback();
	REALstring s = REALBuildString (defn->name, strlen(defn->name));
	REALobject classAnal = REALnewInstance (classAndCtrlAnalClass.name);
	if (classAnal) {
		ClassData (classAndCtrlAnalClass, classAnal, classAndCtrlAnalClassData, ca);
		ca->rbclass = defn;
		ca->type = classType;
		ca->isExt = false;
		if (currentBrowser->registerClassEvent) currentBrowser->registerClassEvent (currentBrowser->instance, s, classAnal);
		REALUnlockObject (classAnal);
	}
	if (s) REALUnlockString (s);
	ExitCallback();
}

static void registerClassExtension (REALclassDefinition *defn)
{
	EnterCallback();
	REALstring s = REALBuildString (defn->name, strlen(defn->name));
	REALobject classAnal = REALnewInstance (classAndCtrlAnalClass.name);
	if (classAnal) {
		ClassData (classAndCtrlAnalClass, classAnal, classAndCtrlAnalClassData, ca);
		ca->rbclass = defn;
		ca->type = classType;
		ca->isExt = true;
		if (currentBrowser->registerClassEvent) currentBrowser->registerClassEvent (currentBrowser->instance, s, classAnal);
		REALUnlockObject (classAnal);
	}
	if (s) REALUnlockString (s);
	ExitCallback();
}

static void registerControl (REALcontrol *defn)
{
	EnterCallback();
	REALstring s = REALBuildString (defn->name, strlen(defn->name));
	REALobject ctrlAnal = REALnewInstance (ctrlAnalClass.name);
	if (ctrlAnal) {
		ClassData (classAndCtrlAnalClass, ctrlAnal, classAndCtrlAnalClassData, ca);
		ca->rbcntl = defn;
		ca->type = controlType;
		ca->isExt = false;
		if (currentBrowser->registerControlEvent) currentBrowser->registerControlEvent (currentBrowser->instance, s, ctrlAnal);
		REALUnlockObject (ctrlAnal);
	}
	if (s) REALUnlockString (s);
	ExitCallback();
}

static void registerDBEngine (REALdbEngineDefinition *defn)
{
	EnterCallback();
	REALobject dbAnal = REALnewInstance (dbEngineAnalClass.name);
	if (dbAnal) {
		ClassData (databaseAnalClass, dbAnal, databaseAnalClassData, db);
		db->type = dbEngineType;
		db->rbDBngn = defn;
		if (currentBrowser->registerDBEngineEvent) currentBrowser->registerDBEngineEvent (currentBrowser->instance, dbAnal);
		REALUnlockObject (dbAnal);
	}
	ExitCallback();
}

static void registerDBTable (REALdbTableDefinition *defn)
{
	EnterCallback();
	REALobject dbAnal = REALnewInstance (dbTableAnalClass.name);
	if (dbAnal) {
		ClassData (databaseAnalClass, dbAnal, databaseAnalClassData, db);
		db->type = dbTableType;
		db->rbDBtbl = defn;
		if (currentBrowser->registerDBTableEvent) currentBrowser->registerDBTableEvent (currentBrowser->instance, dbAnal);
		REALUnlockObject (dbAnal);
	}
	ExitCallback();
}

static void registerDBCursor (REALdbCursorDefinition *defn)
{
	EnterCallback();
	REALobject dbAnal = REALnewInstance (dbCursorAnalClass.name);
	if (dbAnal) {
		ClassData (databaseAnalClass, dbAnal, databaseAnalClassData, db);
		db->type = dbCursorType;
		db->rbDBcrsr = defn;
		if (currentBrowser->registerDBCursorEvent) currentBrowser->registerDBCursorEvent (currentBrowser->instance, dbAnal);
		REALUnlockObject (dbAnal);
	}
	ExitCallback();
}

static void registerPluginExports (REALexportV1 *defn)
{
	// we can ignore this one (called by RB v1.0 plugins to report the method entry point table)
}

static void registerPluginVersion (long v)
{
	EnterCallback();
	currentBrowser->pluginVersion = v;
	if (currentBrowser->registerVersionEvent) currentBrowser->registerVersionEvent (currentBrowser->instance, v);
	ExitCallback();
}


static void	registerPluginMarkSocketUsage (void)
{
	// there's no interesting user information in this call
}

static void	registerDataSourceInterface(const char *szMenuName/*, REALDataSourceInterfaceProc proc*/)
{
	EnterCallback();
	REALstring s = REALBuildString (szMenuName, strlen(szMenuName));
	if (currentBrowser->dataSrcInterface) currentBrowser->dataSrcInterface (currentBrowser->instance, s);
	if (s) REALUnlockString (s);
	ExitCallback();
}

static void	registerDataSource(const char *szDatasourceName/*, REALDataSourceProc proc*/)
{
	EnterCallback();
	REALstring s = REALBuildString (szDatasourceName, strlen(szDatasourceName));
	if (currentBrowser->dataSource) currentBrowser->dataSource (currentBrowser->instance, s);
	if (s) REALUnlockString (s);
	ExitCallback();
}

static void RegisterPluginVersion(void)
{
	
}

static void registerREALenterMovies (void)	
{
	// there's no interesting user information in this call
}

static void registerREALinRuntime (void)	
{
	// there's no interesting user information in this call
}

static void registerREALQDGlobals (void)	
{
	// there's no interesting user information in this call
}

static void registerallocateMenuID (void)	
{
	// there's no interesting user information in this call
}

static void registerreleaseMenuID (int id)	
{
	// there's no interesting user information in this call
}

static void	registerEventFilter (REALEventCallback callback, long param)
{
	// there's no interesting user information in this call
}

static Ptr registergetA4stackReference (void)
{
	EnterCallback();
	static long a4Stack[100];
	Ptr res = ((Ptr)a4Stack + sizeof(a4Stack));
	ExitCallback();
	return res;
}

#if TARGET_68K
#define REALGetRBVersion() 0.0
#endif

static double registerREALGetRBVersion (void)
{
	EnterCallback();
	ExitCallback();
	return REALGetRBVersion();
}

static void registerKnownEntries (void)	
{
	// there's no interesting user information in this call
}


static jmp_buf gJmpBuf;

static void abortPlugin ()
{
	EnterCallback();
	longjmp (gJmpBuf, 1);
}

static void* resolver (const char *entryName)
{
	EnterCallback();
	MONGO(entryName)
	if (strcmp (entryName, "ResolverPPC") == 0) {
		ExitCallback();
		return (void*) resolver;
	}

	if (strcmp (entryName, "RegisterPluginVersion") == 0) {
		ExitCallback();
		return (void*) registerPluginVersion;
	}
	if (strcmp (entryName, "PluginRegisterMethod") == 0) {
		ExitCallback();
		return (void*) registerMethod;
	}
	if (strcmp (entryName, "PluginRegisterClass") == 0) {
		ExitCallback();
		return (void*) registerClass;
	}
	if (strcmp (entryName, "PluginRegisterClassExtension") == 0) {
		ExitCallback();
		return (void*) registerClassExtension;
	}
	if (strcmp (entryName, "PluginRegisterControl") == 0) {
		ExitCallback();
		return (void*) registerControl;
	}
	if (strcmp (entryName, "PluginRegisterDBEngine") == 0) {
		ExitCallback();
		return (void*) registerDBEngine;
	}
	if (strcmp (entryName, "PluginRegisterDBTable") == 0) {
		ExitCallback();
		return (void*) registerDBTable;
	}
	if (strcmp (entryName, "PluginRegisterDBCursor") == 0) {
		ExitCallback();
		return (void*) registerDBCursor;
	}
	if (strcmp (entryName, "RegisterPluginExports") == 0) {
		ExitCallback();
		return (void*) registerPluginExports;
	}
	if (strcmp (entryName, "PluginRegisterEventFilterPPC") == 0) {
		ExitCallback();
		return (void*) registerEventFilter;
	}
	if (strcmp (entryName, "PluginMarkSocketUsage") == 0) {
		ExitCallback();
		return (void*) registerPluginMarkSocketUsage;
	}	
	if (strcmp (entryName, "RegisterDataSourceInterface") == 0) {
		ExitCallback();
		return (void*) registerDataSourceInterface;
	}	
	if (strcmp (entryName, "RegisterDataSource") == 0) {
		ExitCallback();
		return (void*) registerDataSource;
	}	
	if (strcmp (entryName, "REALenterMovies") == 0) {
		ExitCallback();
		return (void*) registerREALenterMovies;
	}	

	if (strcmp (entryName, "REALinRuntime") == 0) {
		ExitCallback();
		return (void*) registerREALinRuntime;
	}	

	if (strcmp (entryName, "REALQDGlobals") == 0) {
		ExitCallback();
		return (void*) registerREALQDGlobals;
	}

	if (strcmp (entryName, "allocateMenuID") == 0) {
		ExitCallback();
		return (void*) registerallocateMenuID;
	}

	if (strcmp (entryName, "releaseMenuID") == 0) {
		ExitCallback();
		return (void*) registerreleaseMenuID;
	}

	if (strcmp (entryName, "getA4stackReference") == 0) {
		ExitCallback();
		return (void*) registergetA4stackReference;
	}

	if (strcmp (entryName, "RuntimeGraphicsDrawLine") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "StringGetCString") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "StringGetPString") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "PluginInterpretConstantValue") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "PluginDefaultControlFont") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "PluginDefaultControlCaption") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "PluginDefaultControlFontSize") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALBuildString") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALLockObject") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALUnlockObject") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALLockString") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALUnlockString") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "GetInterfaceRoutine") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALBuildPictureFromPicHandle") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALBuildPictureFromGWorld") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALBuildPictureFromPictureDescription") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "lockPictureDescription") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "unlockPictureDescription") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALPictureClearCache") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "drawPicturePrimitive") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "GetEventInstancePPC") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "GetEventInstance") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "GetControlBounds") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "GetControlVisible") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "controlEnabledGetter") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "SetControlVisible") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALGetControlGraphics") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "FolderItemFromFSSpec") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALFSSpecFromFolderItem") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALpathFromFolderItem") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "SelectGraphics") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "DrawOffscreenMacControl") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALInvalidateControl") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALInvalidateControlRect") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALSetSpecialBackground") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "getControlWindow") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALBuildSoundFromHandle") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALBuildAppleEvent") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALBuildAEDescList") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALBuildAEObjSpecifier") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALAccessAppleEvent") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALAccessAppleEventReply") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALGetWindowHandle") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALGetControlHandle") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALGetPopupMenuHandle") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "getMoviePlayerController") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "getMovieMovie") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "buildMovie") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "markMovieDirty") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "PluginNewInstance") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "PluginMarkSocketUsage") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "socketDirectConnect") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "socketClose") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "pluginSocketReadAll") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "pluginSocketRead") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "SocketWrite") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "socketLastErrorCode") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "pluginSocketLookahead") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "socketLookahead") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "socketPoll") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "socketGetEvents") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "DesignAddDataSource") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "RuntimeMsgBox") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "controlEnabledSetter") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "ctlPosGetter") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "ctlPosSetter") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALRefreshWindow") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "pluginPictureGraphicsGetter") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "newPicture") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "newMemoryBlock") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "memoryBlockGetPtr") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "memoryBlockGetSize") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "PtrToMemoryBlock") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	
	//3.5
	if (strcmp (entryName, "REALGetRBVersion") == 0) {
		ExitCallback();
		return (void*) registerREALGetRBVersion;
	}
	if (strcmp (entryName, "RuntimeRaiseException") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "RuntimeUBound") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "RuntimeDirectReadIntArray") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "RuntimeDirectReadStringArray") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "RuntimeDirectReadObjectArray") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "GetTabPanelVisible") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	
	//4.0
	if (strcmp (entryName, "REALGetControlFocus") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALSetControlFocus") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "REALYieldToRB") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "pluginObjectType") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "CreateInstance") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	if (strcmp (entryName, "PluginRegisterInterface") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	
	
	if (strcmp (entryName, "") == 0) {
		ExitCallback();
		return (void*) registerKnownEntries;
	}
	// Error - we got an unhandled entryName!
	{
		REALstring s = REALBuildString (entryName, strlen(entryName));
		MSG(s->CString())
		if (currentBrowser->unknownRequestEvent) currentBrowser->unknownRequestEvent (currentBrowser->instance, s);
		REALUnlockString (s);
	}
	ExitCallback();
	return (void*) abortPlugin;	// will return from the plugin when this requested routine is called
}

static UniversalProcPtr gOldExitToShell, gNewExitToShell;
static Boolean gSimulatingExitToShell;

#if TARGET_PPC || TARGET_68K
	static void deInstallExitToShellInterceptor ()
	{
		if (gOldExitToShell) SetToolTrapAddress (gOldExitToShell, _ExitToShell);
		gOldExitToShell = nil;
	}

	static pascal void interceptExitToShell (void)
	{
		EnterCallback();
		if (!gSimulatingExitToShell) {
			deInstallExitToShellInterceptor ();
			ExitCallback();
			ExitToShell ();
		}
		ExitCallback();
	}

	static void installExitToShellInterceptor ()
	{
		gSimulatingExitToShell = false;
		gNewExitToShell = NewRoutineDescriptor ((ProcPtr)interceptExitToShell, kPascalStackBased, GetCurrentISA());
		gOldExitToShell = NGetTrapAddress (_ExitToShell, 1);
		SetToolTrapAddress (gNewExitToShell, _ExitToShell);
	}
#endif



#if TARGET_PPC
	typedef void (*PluginMainProcPtr) (RoutineDescriptor*);
#else
	typedef void (*PluginMainProcPtr)(ProcPtr);
#endif

enum { 
	uppResolverPPCProcInfo =  
		kThinkCStackBased | 
		RESULT_SIZE(SIZE_CODE(4)) | 
		STACK_ROUTINE_PARAMETER(1, SIZE_CODE(4)) 
}; 

static void loadPlugin (Handle plHdl)
{
	HLockHi (plHdl);
	CFragConnectionID myConnID;
	Str255 errMsg;
	
	PluginMainProcPtr plMain;
	Boolean ok = false;

	short rsrcID;
	Str255 rsrcName2;
	OSType rsrcType;

	GetResInfo (plHdl, &rsrcID, &rsrcType, rsrcName2);

	REALstring rsrcName = REALBuildString ((Ptr)&rsrcName2[1], rsrcName2[0]);

	#if TARGET_PPC || TARGET_68K
		installExitToShellInterceptor ();
	#endif

	OSErr err;
	#if TARGET_68K
		myConnID = (CFragConnectionID) plHdl;	// for 68K, the "connection ID" is simply the handle to the code resource
		plMain = (PluginMainProcPtr) *plHdl; // 68K plugins' main() entries are at the very first position of the code resource
		errMsg[0] = 0;
		err = 0;
	#else
		long ofs = *(long*)(*plHdl+0x14);
		// this call leads to the call of "__initialize" inside the plugin:
		err = GetMemFragment (*plHdl+ofs, GetHandleSize(plHdl)-ofs, "\ploaded plugin", kPrivateCFragCopy, &myConnID, (Ptr*)&plMain, errMsg);
	#endif

	if (!err) {
		if (currentBrowser->pluginOpen) currentBrowser->pluginOpen (currentBrowser->instance, rsrcName, rsrcID, (long)myConnID);
		
		#if TARGET_PPC || TARGET_68K
			#if TARGET_68K
				ProcPtr plugin_resolver = (ProcPtr) resolver; 	
			#else
				// Classic PPC
				static RoutineDescriptor resolver_desc = BUILD_ROUTINE_DESCRIPTOR (uppResolverPPCProcInfo, resolver);
				RoutineDescriptor* plugin_resolver = &resolver_desc;
			#endif
		#elif TARGET_CARBON
			// Carbon PPC
			ProcPtr plugin_resolver = (ProcPtr) (uppResolverPPCProcInfo, resolver);
		#else
			-- oops --
		#endif

		if (setjmp (gJmpBuf) == 0) {
			// call the plugin's main() function
			plMain (plugin_resolver);
			ok = true;
		} else {
			// we get here if the plugin called a resolved function that was unknown to us.
			ok = false;
		}

		// It is possible that the plugin mainEntry code has installed a trap patch to ExitToShell in order to
		// let go of some resources it might have permanently opened. To make the plugin release these resources,
		// we simulate a ExitToShell call here now:

		#if TARGET_PPC || TARGET_68K
			gSimulatingExitToShell = true;
			ExitToShell ();	// this exits the current plugin's "process", not your plunger process, and will return
			deInstallExitToShellInterceptor ();
		#endif
		#if not TARGET_68K
			// this call leads to the call of "__terminate" inside the plugin (only if it's a Carbon, not Classic, plugin, though)
			CloseConnection (&myConnID);
		#endif

		if (currentBrowser->pluginClose) currentBrowser->pluginClose (currentBrowser->instance, ok);
	} else {
		#if TARGET_PPC || TARGET_68K
			deInstallExitToShellInterceptor ();
		#endif
		REALstring s = REALBuildString ((Ptr)&errMsg[1], errMsg[0]);
		if (currentBrowser->pluginLoadErr) currentBrowser->pluginLoadErr (currentBrowser->instance, rsrcName, rsrcID, s);
		REALUnlockString (s);
	}

	REALUnlockString (rsrcName);
}

enum {
	kNoErr = 0,
	kCouldNotOpen,
	kLowMem,
	kNoPlugFile
};


static long browsePlugin (REALfolderItem plugFile, REALobject browserInstance)
{
	FSSpec		plSpec;
	Boolean		ok;
	long		returnFlag = kNoPlugFile;

	#if TARGET_CARBON
		OSType		type = 'PLCN';
		MSG(PLCN)
	#elif TARGET_PPC
		OSType		type = 'PLPC';
		MSG(PLPC)
	#elif TARGET_68K
		OSType		type = 'PL68';
		MSG(PL68)
	#else
		// what's this?
		-- oops --
	#endif
	
	PrepareCallback ();
	
	ok = REALFSSpecFromFolderItem(&plSpec, plugFile);

	if (ok) {
		returnFlag = kNoErr;
		browserData data;

		data.pluginVersion = 0;
		data.instance = browserInstance;
		data.pluginNumVer = (PluginVersionControlProc)REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[13]);
		data.unknownRequestEvent = (StringProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[9]);
		data.registerVersionEvent = (IntProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[8]);
		data.registerMethodEvent = (GlobalsProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[7]);
		data.registerClassEvent = (ClassProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[6]);
		data.registerControlEvent = (ControlProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[5]);
		data.registerDBEngineEvent = (DBEngineProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[4]);
		data.registerDBTableEvent = (DBTableProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[3]);
		data.registerDBCursorEvent = (DBCursorProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[2]);
		data.pluginLoadErr = (LoadErrProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[10]);
		data.pluginOpen = (OpenProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[12]);
		data.pluginClose = (BoolProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[11]);
		data.dataSrcInterface = (DtSrcIntProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[1]);
		data.dataSource = (DtSrcProc) REALGetEventInstance((REALcontrolInstance)browserInstance, &analyserEvents[0]);
		
		short prevRF = CurResFile ();
		short f = FSpOpenResFile (&plSpec, fsRdPerm);
		if (f == -1) {
			// failed to open file
			returnFlag = kCouldNotOpen;
		} else {
			MSG(opening file)
			UseResFile (f);
			for (int i = 1; i <= Count1Resources(type); ++i) {
				Handle h = Get1IndResource (type, i);

				if (h) {
					browserData *prevBrowser = currentBrowser;	// this provides for recursions into this method
					currentBrowser = &data;
					
					VersRecHndl vrh = (VersRecHndl)Get1Resource('vers', 1);
					Handle linkURL = Get1Resource('CSTR', 128);
					if (vrh) {
						HLock((Handle) vrh);
						NumVersionVariant numver = *(NumVersionVariant*) &(*(vrh))->numericVersion;
						REALstring linkStr = nil;
						if (linkURL) {
							HLock(linkURL);
							linkStr = REALBuildString(*linkURL, GetHandleSize(linkURL));
							
							HUnlock(linkURL);
							ReleaseResource(linkURL);
						}
						if (currentBrowser->pluginNumVer) currentBrowser->pluginNumVer (currentBrowser->instance, numver.whole, linkStr);
						HUnlock((Handle) vrh);
						ReleaseResource((Handle) vrh);
						REALUnlockString(linkStr);
					}
					
					loadPlugin (h);

					currentBrowser = prevBrowser;
					ReleaseResource (h);
				} else {
					// failed to load a resource that's supposed to be available - this could happen if we're low on memory

					returnFlag = kLowMem;
				}
			}
			CloseResFile (f);
		}
		UseResFile (prevRF);
	}

	return returnFlag;
}

static REALmethodDefinition pluginMethodArray[] = {
	{ (REALproc) browsePlugin, REALnoImplementation, "BrowsePlugin(plugin as FolderItem, browser as PluginAnalyser) as Integer"},
};


// ---------------------------------
// ----- Plugin Initialization -----
// ---------------------------------

void PluginEntry (void)
{
	REALRegisterClass (&globalAnalClass);
	REALRegisterClass (&classAndCtrlAnalClass);
	REALRegisterClass (&ctrlAnalClass);
	REALRegisterClass (&databaseAnalClass);
	REALRegisterClass (&dbEngineAnalClass);
	REALRegisterClass (&dbTableAnalClass);
	REALRegisterClass (&dbCursorAnalClass);
	REALRegisterClass (&analClass);
	for (int i = 0; i < sizeof(pluginMethodArray) / sizeof(REALmethodDefinition); ++i) {
		REALRegisterMethod (&pluginMethodArray[i]);
	}
}
