/*
 * ResourceForkStream plugin
 *
 * provides the ResourceForkStream class which is a subclass of BinaryStream
 *
 * Written by Thomas Tempelmann (rb@tempel.org), 20 Mar 2003
 *
 * -> unfortunately, it does not work in RB 4.5: once an instance of this class is
 *    assigned to a BinaryStream object, the methods in this plugin won't get called
 *    any more :(
 */


//--------------------
// Your includes here
//--------------------

	// These and other includes should come _before_ the include of "rb_plugin.h"
	// in order to avoid compilation errors.

#include <math.h>
#include <string.h>

#if TARGET_OS_WIN32
	static void inline DebugStr(unsigned char* p) {}
	#define log(x) OutputDebugString(x)
	#define PrepareCallback() 
	#define EnterCallback()
	#define ExitCallback()
#else
	#if TARGET_CPU_68K
		#include <SetupA4.h>
	#endif
	#include <MacMemory.h>
	#include "dcon.h"
	#define log(x) dprintf(x)
#endif

//-------------------------
// RB header includes here
//-------------------------

	// Note: these include should come last, after all other includes of yours

#include "TT's Plugin Support.h"
#include "rb_plugin.h"


//-----------------------
// Some type definitions
//-----------------------

typedef long RBinteger;
typedef Boolean	RBboolVar;	// use this for Boolean variables stored in this plugin
typedef Boolean	RBboolPar;	// use this to receive Boolean parameters (both by value and by reference)
typedef long	RBboolRet;	// use this when returning a Boolean as a function result


//--------------------------------------------
// Raising exceptions from within this plugin
//--------------------------------------------

static void RaiseNILPtrException ()
{
	REALobject exc = REALnewInstance ("NILObjectException");
	REALRaiseException (exc);
}

static void RaiseOutOfBoundsException ()
{
	REALobject exc = REALnewInstance ("OutOfboundsException");
	REALRaiseException (exc);
}

static void RaiseOutOfMemoryException ()
{
	REALobject exc = REALnewInstance ("OutOfMemoryException");
	REALRaiseException (exc);
}



//--------------------------
// ResourceForkStream class
//--------------------------


extern REALclassDefinition rsrcClass;	// forward declaration

struct rsrcClassData
{
	long aInteger;
};

static void rsrcClassConstructor (REALobject instance)
{
	ClassData (rsrcClass, instance, rsrcClassData, me);
	me->aInteger = 12345;
}

static void rsrcClassDestructor (REALobject instance)
{
	ClassData (rsrcClass, instance, rsrcClassData, me);
}

static RBinteger lengthGet (REALobject instance, long param)
{
	ClassData (rsrcClass, instance, rsrcClassData, me);
	return me->aInteger;
}

static void lengthSet (REALobject instance, long param, RBinteger value)
{
	ClassData (rsrcClass, instance, rsrcClassData, me);
	me->aInteger = value;
}


static REALstring rsrcRead (REALobject instance, RBinteger aValue)
{
	ClassData (rsrcClass, instance, rsrcClassData, me);
	return REALBuildString("abcde",5);
}

static REALproperty rsrcClassProperties[] = {
	{ nil, "Length", "Integer",            0, (REALproc) lengthGet,    (REALproc) lengthSet,    0 /* param */ },
};

static REALmethodDefinition rsrcClassMethods[] = {
	{ (REALproc) rsrcRead, REALnoImplementation, "Read(count as Integer) as String"},
};




static REALclassDefinition rsrcClass = {
	kCurrentREALControlVersion,
	"ResourceForkStream",
	"BinaryStream",
	sizeof(rsrcClassData), 0,
	(REALproc) rsrcClassConstructor, (REALproc) rsrcClassDestructor,
	rsrcClassProperties,
	sizeof(rsrcClassProperties) / sizeof(REALproperty),
	rsrcClassMethods,
	sizeof(rsrcClassMethods) / sizeof(REALmethodDefinition),
	nil, 0
};


//----------------------------
// FolderItem class extension
//----------------------------

static REALobject openAsRsrc (REALobject fi, RBboolPar write)
{
	// !!!
	return REALnewInstance ("ResourceForkStream");
}

static REALmethodDefinition folderClassMethods[] = {
	{ (REALproc) openAsRsrc, REALnoImplementation, "OpenAsResourceForkStream(write as Boolean) as ResourceForkStream"},
};

static REALclassDefinition folderExtClass = {
	kCurrentREALControlVersion,
	"FolderItem",
	nil,
	0, 0,
	nil, nil,
	nil, 0,
	folderClassMethods,
	sizeof(folderClassMethods) / sizeof(REALmethodDefinition),
	nil, 0
};

// ---------------------------------
// ----- Plugin Initialization -----
// ---------------------------------

void PluginExit (void)
{
	// !!! close open files here
}

void PluginEntry (void)
{
	InitTTsPluginSupport ();
	
	REALRegisterClass (&rsrcClass);
	REALRegisterClassExtension (&folderExtClass);
}
