//
// this is a "uuencode" decoder, modified to be used in a RB plugin by Thomas Tempelmann Mar 19, 2003
//

/*
 *	uudecode [input]
 *
 * create the specified file, decoding as you go.
 * used with uuencode.
 *
 * modified for MSDOS by Kenneth J. Hendrickson
 */

#include <ctype.h> 
#include <string.h> 

#include "decoders.h"


/* single character decode */
#define DEC(c)	((c - 0x20) & 0x3F)


/*
 * output a group of 3 bytes (4 input characters).
 * the input chars are pointed to by p, they are to
 * be output to file f.  n is used to tell us not to
 * output all of them at the end of the file.
 */
static int outdec (char *p, int in_len, int n, byte* buf_out_ptr)
{
	int c1, c2, c3, p0, p1, p2, p3;
	
	p0 = *p++;
	if (--in_len > 0) {
		p1 = *p++;
		if (--in_len > 0) {
			p2 = *p++;
			if (--in_len > 0) {
				p3 = *p;
				--in_len;
			}
		}
	}

	c1 = DEC(p0) << 2 | DEC(p1) >> 4;
	c2 = DEC(p1) << 4 | DEC(p2) >> 2;
	c3 = DEC(p2) << 6 | DEC(p3);

	if (((p0-0x20) & 0xC0) != 0) {
		if (p0 != 0x60) {
			return -6; // illegal char in line
		}
	}
	if (((p1-0x20) & 0xC0) != 0) {
		if (p1 != 0x60) {
			return -6; // illegal char in line
		}
	}
	buf_out_ptr[0] = c1;
	if (n >= 2) {
		if (((p2-0x20) & 0xC0) != 0) {
			if (p2 != 0x60) {
				return -6; // illegal char in line
			}
		}
		buf_out_ptr[1] = c2;
		if (n >= 3) {
			if (((p3-0x20) & 0xC0) != 0) {
				if (p3 != 0x60) {
					return -6; // illegal char in line
				}
			}
			buf_out_ptr[2] = c3;
			return 3;
		}
	}
	return n;
}


long uudecode (byte* buf_out_ptr, long buf_out_len)
{
	char *bp;
	int n, origlen = buf_out_len;
	byte *line;
	long len;

	for (;;) {
		/* for each input line */
		if (not getline(&line, &len)) {
			//printf("Short file\n");
			return -4;
		}

		if (len == 0 or line[0] == 'e') {
			// the end has been reached
			if (len == 0) getline(&line, &len);
			if (strncmp((char*)line, "end", 3) != 0) {
				return -2;	// end marker missing
			}
			return origlen - buf_out_len;
		}
		
		n = line[0] - 0x20;
		--len;

		if (n == 64) n = 0;
		if ((n+2) / 3 != (len+3) / 4) {
			// line has wrong length
			return -7;
		}

		if (len > 0) {
			bp = (char*)&line[1];
			while (n > 0 and buf_out_len >= n) {
				int n2 = outdec(bp, len, n, buf_out_ptr);
				if (n2 < 0) {
					return n2; // error
				}
				buf_out_ptr += n2;
				buf_out_len -= n2;
				bp += 4;
				n -= 3;
			}
			if (buf_out_len < n) {
				return -3; // outbuf too small
			}
		}
	}
	return -1;
}
