/*
 * QuitHandler.cp
 *
 * Shows how to cleanup when your app unexpectedly quits
 *
 * First version by Thomas Tempelmann (macdev@tempel.org), 4 Jan 2001
 * Updated May 5, 2002, with help from Peter Robinson: MPW support added, some includes enhanced
 */

#include <ConditionalMacros.h>

#if not TARGET_OS_MAC
	#error This file (QuitHandler.cp) can currently only be used for MacOS targets!
#endif

#if not TARGET_CARBON

#if !defined(MPW_CPLUS) && !defined(MPW_C) // --- mod PR ...
	#include <SetupA4.h>
#else
	#if TARGET_CPU_68K
		#error MPW cannot currently be used (with this starter) to build 68k plug-ins!
	#else
		#define EnterCallback()
		#define ExitCallback()
		#define PrepareCallback()
	#endif
#endif // --- ... mod PR

#include <Traps.h>
#include <OSUtils.h>
#include <Processes.h>
#include "QuitHandler.h"

#if not TARGET_CPU_68K
#define far
#endif

far static UniversalProcPtr gOldExitToShell = nil, gNewExitToShell;
far static QuitProcPtr gHandleQuit;

static void deInstallQuitHandler ()
// uninstalls the ExitToShell patch
{
	if (gOldExitToShell) SetToolTrapAddress (gOldExitToShell, _ExitToShell);
	gOldExitToShell = nil;
}

static pascal void quitHandler (void)
// gets called when ExitToShell() is executed
{
	EnterCallback();
	deInstallQuitHandler ();
	gHandleQuit ();
	ExitCallback();
	ExitToShell ();
}

void InstallQuitHandler (QuitProcPtr callAtQuit)
// installs the ExitToShell patch
{
	PrepareCallback ();
	gHandleQuit = callAtQuit;
	gNewExitToShell = NewRoutineDescriptor ((ProcPtr)quitHandler, kPascalStackBased, GetCurrentISA());
	gOldExitToShell = NGetTrapAddress (_ExitToShell, 1);
	SetToolTrapAddress (gNewExitToShell, _ExitToShell);
}

#endif

// EOF
