/******************************************************************************\
* simplec.c - Simple TCP/UDP client using Winsock 1.1
* 
*       This is a part of the Microsoft Source Code Samples.
*       Copyright 1996 - 1998 Microsoft Corporation.
*       All rights reserved.
*       This source code is only intended as a supplement to
*       Microsoft Development Tools and/or WinHelp documentation.
*       See these sources for detailed information regarding the
*       Microsoft samples programs.
\******************************************************************************/

#define WIN32_LEAN_AND_MEAN
#include <stdio.h>
#include <stdlib.h>
#include <winsock2.h>

//#define CONNECTED_UDP //if commented out, use unconnected semantic; if uncommented, use connected semantics

#define DEFAULT_PORT 5001
//#define DEFAULT_PROTO SOCK_STREAM // TCP
#define DEFAULT_PROTO SOCK_DGRAM // UDP

void Usage(char *progname) {
	fprintf(stderr,"Usage\n%s -p [protocol] -n [server] -e [endpoint] \
	-l [iterations]\n",
		progname);
	fprintf(stderr,"Where:\n\tprotocol is one of TCP or UDP\n");
	fprintf(stderr,"\tserver is the IP address or name of server\n");
	fprintf(stderr,"\tendpoint is the port to listen on\n");
	fprintf(stderr,"\titerations is the number of loops to execute\n");
	fprintf(stderr,"\t(-l by itself makes client run in an infinite loop,");
	fprintf(stderr," Hit Ctrl-C to terminate it)\n");
	fprintf(stderr,"Defaults are TCP , localhost and 5001\n");
	WSACleanup();
	exit(1);
}


SOCKET OpenQoSSocket(INT iSocketType);
BOOL SetQoS (SOCKET s, INT iSocketType, SOCKADDR_IN *remoteaddr);
void DisplayError (LPTSTR title, DWORD error)
{
    printf ("%s, Error# %d\n\n", title,  error);
}

int main(int argc, char **argv) {

	char Buffer[128];
	char *server_name= "localhost";
	//char *server_name= "192.168.1.77";
	unsigned short port = DEFAULT_PORT;
	int retval, loopflag=0;
	int i, loopcount,maxloop=-1;
	unsigned int addr;
	int socket_type = DEFAULT_PROTO;
	struct sockaddr_in server;
	struct hostent *hp;
	WSADATA wsaData;
	SOCKET  conn_socket;

	if (argc >1) {
		for(i=1;i <argc;i++) {
			if ( (argv[i][0] == '-') || (argv[i][0] == '/') ) {
				switch(tolower(argv[i][1])) {
					case 'p':
						if (!strcmp(argv[i+1], "TCP") || !strcmp(argv[i+1], "tcp") )
							socket_type = SOCK_STREAM;
						else if (!strcmp(argv[i+1], "UDP") || !strcmp(argv[i+1], "udp") )
							socket_type = SOCK_DGRAM;
						else
							Usage(argv[0]);
						i++;
						break;

					case 'n':
						server_name = argv[++i];
						break;
					case 'e':
						port = atoi(argv[++i]);
						break;
					case 'l':
						loopflag =1;
						if (argv[i+1]) {
							if (argv[i+1][0] != '-') 
								maxloop = atoi(argv[i+1]);
						}
						else
							maxloop = -1;
						i++;
						break;
					default:
						Usage(argv[0]);
						break;
				}
			}
			else
				Usage(argv[0]);
		}
	}
	
	if ((retval = WSAStartup(0x202,&wsaData)) != 0) {
		fprintf(stderr,"WSAStartup failed with error %d\n",retval);
		WSACleanup();
		return -1;
	}
	
	if (port == 0){
		Usage(argv[0]);
	}

	//
	// Attempt to detect if we should call gethostbyname() or
	// gethostbyaddr()

	if (isalpha(server_name[0])) {   /* server address is a name */
		hp = gethostbyname(server_name);
	}
	else  { /* Convert nnn.nnn address to a usable one */
		addr = inet_addr(server_name);
		hp = gethostbyaddr((char *)&addr,4,AF_INET);
	}
	if (hp == NULL ) {
		fprintf(stderr,"Client: Cannot resolve address [%s]: Error %d\n",
			server_name,WSAGetLastError());
		WSACleanup();
		exit(1);
	}

	//
	// Copy the resolved information into the sockaddr_in structure
	//
	memset(&server,0,sizeof(server));
	memcpy(&(server.sin_addr),hp->h_addr,hp->h_length);
	server.sin_family = hp->h_addrtype;
	server.sin_port = htons(port);

	conn_socket = socket(AF_INET,socket_type,0); /* Open a socket */
	//conn_socket = OpenQoSSocket(socket_type); /* Open a socket */
	if (conn_socket <0 ) {
		fprintf(stderr,"Client: Error Opening socket: Error %d\n",
			WSAGetLastError());
		WSACleanup();
		return -1;
	}

	//
	// Notice that nothing in this code is specific to whether we 
	// are using UDP or TCP.
	// We achieve this by using a simple trick.
	//    When connect() is called on a datagram socket, it does not 
	//    actually establish the connection as a stream (TCP) socket
	//    would. Instead, TCP/IP establishes the remote half of the
	//    ( LocalIPAddress, LocalPort, RemoteIP, RemotePort) mapping.
	//    This enables us to use send() and recv() on datagram sockets,
	//    instead of recvfrom() and sendto()

    if (socket_type == SOCK_STREAM)
	{
		printf("Client connecting to: %s\n",hp->h_name);
		if (connect(conn_socket,(struct sockaddr*)&server,sizeof(server))
			== SOCKET_ERROR) {
			fprintf(stderr,"connect() failed: %d\n",WSAGetLastError());
			WSACleanup();
			return -1;
		}
	}

///////////////////////////////////////

#ifdef CONNECTED_UDP

	if (socket_type != SOCK_STREAM)
		printf ("connected UDP\n");
	printf("Client connecting to: %s\n",hp->h_name);
	if (socket_type != SOCK_STREAM && connect(conn_socket,(struct sockaddr*)&server,sizeof(server))
		== SOCKET_ERROR) {
		fprintf(stderr,"connect() failed: %d\n",WSAGetLastError());
		WSACleanup();
		return -1;
	}
/*
    if (! SetQoS (conn_socket, socket_type, NULL) )
		printf ("SetQoS failed \n");
*/

	Sleep (200); //Wait a little while to allow PATH, RESV exchange

	// cook up a string to send
	//
	loopcount =0;
	while(1) {
		wsprintf(Buffer,"This is a small test message [number %d]",loopcount++);
		retval = send(conn_socket,Buffer,sizeof(Buffer),0);
		if (retval == SOCKET_ERROR) {
			fprintf(stderr,"send() failed: error %d\n",WSAGetLastError());
			WSACleanup();
			return -1;
		}
		printf("Sent Data [%s]\n",Buffer);

		Sleep (200);
		if (!loopflag){
			printf("Terminating connection\n");
			break;
		}
		else {
			if ( (loopcount >= maxloop) && (maxloop >0) )
				break;
		}

	}

#else
////////////////////////////////////////////////////
	if (socket_type != SOCK_STREAM)
		printf ("unconnected UDP\n");
/*
    if (! SetQoS (conn_socket, socket_type, &server) )
		printf ("SetQoS failed \n");
*/
	Sleep (200); //Wait a little while to allow PATH, RESV exchange

	// cook up a string to send
	//
	loopcount =0;
	while(1) {
		wsprintf(Buffer,"This is a small test message [number %d]",loopcount++);
		if (socket_type != SOCK_STREAM) 
		{
			retval = sendto(conn_socket,Buffer,sizeof (Buffer),0,
				(struct sockaddr *)&server,sizeof (server) );
		}
		else
		{
			retval = send(conn_socket,Buffer,sizeof(Buffer),0);
		}

		if (retval == SOCKET_ERROR) {
			fprintf(stderr,"send() failed: error %d\n",WSAGetLastError());
			WSACleanup();
			return -1;
		}
		printf("Sent Data [%s]\n",Buffer);

		Sleep (200);
		if (!loopflag){
			printf("Terminating connection\n");
			break;
		}
		else {
			if ( (loopcount >= maxloop) && (maxloop >0) )
				break;
		}

	}
#endif
	
	closesocket(conn_socket);
	WSACleanup();

	return 0;
}

/*
SOCKET OpenQoSSocket(INT iSocketType)
{
    DWORD bufferSize = 0;
    DWORD numProtocols, i;
    LPWSAPROTOCOL_INFO installedProtocols, qosProtocol; 
    SOCKET fd;
    
    //
    // Call WSAEnumProtocols to determine buffer size required
    //
    numProtocols = WSAEnumProtocols(NULL, NULL, &bufferSize);
    if((numProtocols != SOCKET_ERROR) && (WSAGetLastError() != WSAENOBUFS)){
        DisplayError ( "WSAEnumProtocols Failed", WSAGetLastError() ); 
        return(INVALID_SOCKET);
    } else {
        //
        // Allocate a buffer to hold the list of protocol info structures
        //
        installedProtocols = (LPWSAPROTOCOL_INFO)malloc(bufferSize);
   
        //
        // Enumerate the protocols, find the QoS enabled one
        //
        numProtocols = WSAEnumProtocols(NULL,
                                        installedProtocols,
                                        &bufferSize);
        if(numProtocols == SOCKET_ERROR){
            DisplayError ( "WSAEnumProtocols Failed", WSAGetLastError() ); 
            return(INVALID_SOCKET);
        } 
        else 
        {
            qosProtocol = installedProtocols;

            for( i=0 ; i < numProtocols ; qosProtocol++, i++)
            {
                if  ((qosProtocol->dwServiceFlags1 & XP1_QOS_SUPPORTED) &&
                    (qosProtocol->iSocketType    == iSocketType) &&
                    (qosProtocol->iAddressFamily == AF_INET))
                {
                    break;
                }
                     
            }
        }
    
        //
        // Now open the socket.
        //
        fd = WSASocket(0, 
                       iSocketType, 
                       0, 
                       qosProtocol,          // Use the QoS SP we found
                       0, 
                       WSA_FLAG_OVERLAPPED); // *MUST* be overlapped!
    
        //
        // De-allocate protocol info buffer
        //
        free(installedProtocols);

        return(fd);
    }        
}


BOOL SetQoS (SOCKET s, INT iSocketType, SOCKADDR_IN *remoteaddr)
{
    QOS         Qos;
	int   result ;
	DWORD BytesRet;   

        
	memset ( &Qos, QOS_NOT_SPECIFIED, sizeof(QOS) );
    
    Qos.ReceivingFlowspec.ServiceType           = SERVICETYPE_NOTRAFFIC;
    Qos.ReceivingFlowspec.TokenRate             = QOS_NOT_SPECIFIED; 
    Qos.ReceivingFlowspec.TokenBucketSize       = QOS_NOT_SPECIFIED; 
    Qos.ReceivingFlowspec.PeakBandwidth         = QOS_NOT_SPECIFIED;
    Qos.ReceivingFlowspec.MinimumPolicedSize    = QOS_NOT_SPECIFIED;
    Qos.ReceivingFlowspec.MaxSduSize            = QOS_NOT_SPECIFIED;
    
	//g711
    Qos.SendingFlowspec.ServiceType             = SERVICETYPE_GUARANTEED; 
    Qos.SendingFlowspec.TokenRate               = 9250; 
    Qos.SendingFlowspec.TokenBucketSize         = 680; 
    Qos.SendingFlowspec.PeakBandwidth           = 13875; 
	Qos.SendingFlowspec.Latency			        = QOS_NOT_SPECIFIED;
	Qos.SendingFlowspec.DelayVariation          = QOS_NOT_SPECIFIED; 
    Qos.SendingFlowspec.MinimumPolicedSize      = 340; 
    Qos.SendingFlowspec.MaxSduSize              = 340; 

    Qos.ProviderSpecific.len = 0;
    Qos.ProviderSpecific.buf = NULL;
    if ( iSocketType == SOCK_DGRAM && remoteaddr != NULL)
	{
        QOS_DESTADDR QosDestaddr;
		struct sockaddr_in local;

		//if SIO_SET_QOS is used to set QOS with QOS_DESTADDR for unconnected UDP, 
		//the socket must be locally bound first, other SIO_SET_QOS will fail with 10022
		local.sin_family = AF_INET;
		local.sin_addr.s_addr = INADDR_ANY; 

		// Port MUST be in Network Byte Order
		local.sin_port = htons(6666);
		
		if (bind(s,(struct sockaddr*)&local,sizeof(local) ) 
			== SOCKET_ERROR) {
			fprintf(stderr,"bind() failed with error %d\n",WSAGetLastError());
			return FALSE;
		}
		
		ZeroMemory((char *)&QosDestaddr, sizeof(QosDestaddr));
        QosDestaddr.ObjectHdr.ObjectType   = QOS_OBJECT_DESTADDR;
        QosDestaddr.ObjectHdr.ObjectLength = sizeof(QosDestaddr);
        QosDestaddr.SocketAddress = (SOCKADDR *)remoteaddr;
        QosDestaddr.SocketAddressLength = sizeof(SOCKADDR);

		Qos.ProviderSpecific.len = QosDestaddr.ObjectHdr.ObjectLength;
        Qos.ProviderSpecific.buf = (char *)&QosDestaddr;


    }
    result = 
            WSAIoctl(s,SIO_SET_QOS, &Qos,sizeof(QOS),NULL, 0,&BytesRet, NULL,NULL);

	if (result == SOCKET_ERROR) 
	{	
		printf ("SetQoS Error %d\n", GetLastError());
		return FALSE;
	}
	
	return TRUE;
}
*/
