/******************************************************************************\
* simples.c - Simple TCP/UDP server using Winsock 1.1
*       This is a part of the Microsoft Source Code Samples.
*       Copyright 1996 - 1998 Microsoft Corporation.
*       All rights reserved.
*       This source code is only intended as a supplement to
*       Microsoft Development Tools and/or WinHelp documentation.
*       See these sources for detailed information regarding the
*       Microsoft samples programs.
\******************************************************************************/

#define WIN32_LEAN_AND_MEAN
#include <stdio.h>
#include <stdlib.h>
#include <winsock2.h>
/*
#include <Qos.h>
#include <qossp.h> 
#include <qos.h>
#include <qospol.h>
*/

#define DEFAULT_PORT 5001
//#define DEFAULT_PROTO SOCK_STREAM // TCP
#define DEFAULT_PROTO SOCK_DGRAM // UDP

void Usage(char *progname) {
	fprintf(stderr,"Usage\n%s -p [protocol] -e [endpoint] -i [interface]\n",
		progname);
	fprintf(stderr,"Where:\n\tprotocol is one of TCP or UDP\n");
	fprintf(stderr,"\tendpoint is the port to listen on\n");
	fprintf(stderr,"\tinterface is the ipaddr (in dotted decimal notation)");
	fprintf(stderr," to bind to\n");
	fprintf(stderr,"Defaults are TCP,5001 and INADDR_ANY\n");
	WSACleanup();
	exit(1);
}

//SOCKET OpenQoSSocket(INT iSocketType);
//BOOL SetQoS (SOCKET s, INT iSocketType, SOCKADDR_IN *remoteaddr);

void DisplayError (LPTSTR title, DWORD error)
{
    printf ("%s, Error# %d\n\n", title,  error);
}


 
int main(int argc, char **argv) {

	char Buffer[128];
	char *interface= NULL;
	unsigned short port=DEFAULT_PORT;
	int retval;
	int fromlen;
	int i;
	int socket_type = DEFAULT_PROTO;
	struct sockaddr_in local, from;
	WSADATA wsaData;
	SOCKET listen_socket, msgsock;
	int bOnce = 0;

	/* Parse arguments */
	if (argc >1) {
		for(i=1;i <argc;i++) {
			if ( (argv[i][0] == '-') || (argv[i][0] == '/') ) {
				switch(tolower(argv[i][1])) {
					case 'p':
						if (!strcmp(argv[i+1], "TCP") || !strcmp(argv[i+1], "tcp") )
							socket_type = SOCK_STREAM;
						else if (!strcmp(argv[i+1], "UDP") || !strcmp(argv[i+1], "udp") )
							socket_type = SOCK_DGRAM;
						else
							Usage(argv[0]);
						i++;
						break;

					case 'i':
						interface = argv[++i];
						break;
					case 'e':
						port = atoi(argv[++i]);
						break;
					default:
						Usage(argv[0]);
						break;
				}
			}
			else
				Usage(argv[0]);
		}
	}
	
	if ((retval = WSAStartup(0x202,&wsaData)) != 0) {
		fprintf(stderr,"WSAStartup failed with error %d\n",retval);
		WSACleanup();
		return -1;
	}

	if (port == 0){
		Usage(argv[0]);
	}

	local.sin_family = AF_INET;
	local.sin_addr.s_addr = (!interface)?INADDR_ANY:inet_addr(interface); 

	/* 
	 * Port MUST be in Network Byte Order
	 */
	local.sin_port = htons(port);

	//listen_socket = OpenQoSSocket ( socket_type);
	listen_socket = socket(AF_INET, socket_type,0); // TCP socket

	if (listen_socket == INVALID_SOCKET){
		fprintf(stderr,"socket() failed with error %d\n",WSAGetLastError());
		WSACleanup();
		return -1;
	}

	//
	// bind() associates a local address and port combination with the
	// socket just created. This is most useful when the application is a 
	// server that has a well-known port that clients know about in advance.
	//
	if (bind(listen_socket,(struct sockaddr*)&local,sizeof(local) ) == SOCKET_ERROR) {
		fprintf(stderr,"bind() failed with error %d\n",WSAGetLastError());
		WSACleanup();
		return -1;
	}

	// show our own IP addr
	/* this does not work :(
	{
		unsigned char *p;
		unsigned int addr;
		addr = inet_addr("localhost");
		p = (unsigned char*)&addr;
		printf ("localhost is [%d.%d.%d.%d]\n", p[0],p[1],p[2],p[3]);
		exit(0);
	}
	*/
	
	//
	// So far, everything we did was applicable to TCP as well as UDP.
	// However, there are certain steps that do not work when the server is
	// using UDP.
	//

	// We cannot listen() on a UDP socket.

	if (socket_type != SOCK_DGRAM) {
		if (listen(listen_socket,5) == SOCKET_ERROR) {
			fprintf(stderr,"listen() failed with error %d\n",WSAGetLastError());
			WSACleanup();
			return -1;
		}
	}
	printf("%s: 'Listening' on port %d, protocol %s\n",argv[0],port,
		(socket_type == SOCK_STREAM)?"TCP":"UDP");
	while(1) {
		bOnce = 0;
		fromlen =sizeof(from);
		//
		// accept() doesn't make sense on UDP, since we do not listen()
		//
		if (socket_type != SOCK_DGRAM) {
			msgsock = accept(listen_socket,(struct sockaddr*)&from, &fromlen);
			if (msgsock == INVALID_SOCKET) {
				fprintf(stderr,"accept() error %d\n",WSAGetLastError());
				WSACleanup();
				return -1;
			}
			printf("accepted connection from %s, port %d\n", 
						inet_ntoa(from.sin_addr),
						htons(from.sin_port)) ;
			
		}
		else
			msgsock = listen_socket;

/*
    if (!SetQoS (msgsock, socket_type, NULL) )
		printf ("SetQoS failed \n");
*/

		//
		// In the case of SOCK_STREAM, the server can do recv() and 
		// send() on the accepted socket and then close it.

		// However, for SOCK_DGRAM (UDP), the server will do
		// recvfrom() and sendto()  in a loop.

Loop:
		if (socket_type != SOCK_DGRAM)
		{
			retval = recv(msgsock,Buffer,sizeof (Buffer),0 );
		}
		else {
			retval = recvfrom(msgsock,Buffer,sizeof (Buffer),0,
				(struct sockaddr *)&from,&fromlen);
			printf("Received datagram from %s\n",inet_ntoa(from.sin_addr));
		}
			

		bOnce = 1;
		
		Sleep(200);

		if (retval == SOCKET_ERROR) {
			fprintf(stderr,"recv() failed: error %d\n",WSAGetLastError());
			closesocket(msgsock);
			continue;
		}
		if (retval == 0) {
			printf("Client closed connection\n");
			closesocket(msgsock);
			continue;
		}
		printf("Received %d bytes, data [%s] from client\n",retval,Buffer);
		
		if (socket_type != SOCK_DGRAM){
			//printf("Terminating connection\n");
			//closesocket(msgsock);
			goto Loop;
		}
		else 
			printf("UDP server looping back for more requests\n");
		continue;
	}
}

/*
SOCKET OpenQoSSocket(INT iSocketType)
{
    DWORD bufferSize = 0;
    DWORD numProtocols, i;
    LPWSAPROTOCOL_INFO installedProtocols, qosProtocol; 
    SOCKET fd;
    
    //
    // Call WSAEnumProtocols to determine buffer size required
    //
    numProtocols = WSAEnumProtocols(NULL, NULL, &bufferSize);
    if((numProtocols != SOCKET_ERROR) && (WSAGetLastError() != WSAENOBUFS)){
        DisplayError ( "WSAEnumProtocols Failed", WSAGetLastError() ); 
        return(INVALID_SOCKET);
    } else {
        //
        // Allocate a buffer to hold the list of protocol info structures
        //
        installedProtocols = (LPWSAPROTOCOL_INFO)malloc(bufferSize);
   
        //
        // Enumerate the protocols, find the QoS enabled one
        //
        numProtocols = WSAEnumProtocols(NULL,
                                        installedProtocols,
                                        &bufferSize);
        if(numProtocols == SOCKET_ERROR){
            DisplayError ( "WSAEnumProtocols Failed", WSAGetLastError() ); 
            return(INVALID_SOCKET);
        } 
        else 
        {
            qosProtocol = installedProtocols;

            for( i=0 ; i < numProtocols ; qosProtocol++, i++)
            {
                if  ((qosProtocol->dwServiceFlags1 & XP1_QOS_SUPPORTED) &&
                    (qosProtocol->iSocketType    == iSocketType) &&
                    (qosProtocol->iAddressFamily == AF_INET))
                {
                    break;
                }
                     
            }
        }
    
        //
        // Now open the socket.
        //
        fd = WSASocket(0, 
                       iSocketType, 
                       0, 
                       qosProtocol,          // Use the QoS SP we found
                       0, 
                       WSA_FLAG_OVERLAPPED); // *MUST* be overlapped!
    
        //
        // De-allocate protocol info buffer
        //
        free(installedProtocols);

        return(fd);
    }        
}


BOOL SetQoS (SOCKET s, INT iSocketType, SOCKADDR_IN *remoteaddr)
{
    QOS         Qos;
	int   result ;
	DWORD BytesRet;   

        
	memset ( &Qos, QOS_NOT_SPECIFIED, sizeof(QOS) );
    
    Qos.SendingFlowspec.ServiceType           = SERVICETYPE_NOTRAFFIC;
    Qos.SendingFlowspec.TokenRate             = QOS_NOT_SPECIFIED; 
    Qos.SendingFlowspec.TokenBucketSize       = QOS_NOT_SPECIFIED; 
    Qos.SendingFlowspec.PeakBandwidth         = QOS_NOT_SPECIFIED;
    Qos.SendingFlowspec.MinimumPolicedSize    = QOS_NOT_SPECIFIED;
    Qos.SendingFlowspec.MaxSduSize            = QOS_NOT_SPECIFIED;
    Qos.SendingFlowspec.MinimumPolicedSize    = QOS_NOT_SPECIFIED; 
    Qos.SendingFlowspec.MaxSduSize            = QOS_NOT_SPECIFIED; 

	//Default to Sender's FLOWSPEC, only ServiceType is needed
    Qos.ReceivingFlowspec.ServiceType         = SERVICETYPE_GUARANTEED; 
    Qos.ReceivingFlowspec.TokenRate           = QOS_NOT_SPECIFIED; 
    Qos.ReceivingFlowspec.TokenBucketSize     = QOS_NOT_SPECIFIED; 
    Qos.ReceivingFlowspec.PeakBandwidth       = QOS_NOT_SPECIFIED; 
	Qos.ReceivingFlowspec.Latency			  = QOS_NOT_SPECIFIED;
	Qos.ReceivingFlowspec.DelayVariation      = QOS_NOT_SPECIFIED; 
    Qos.ReceivingFlowspec.MinimumPolicedSize  = QOS_NOT_SPECIFIED; 
    Qos.ReceivingFlowspec.MaxSduSize          = QOS_NOT_SPECIFIED; 

    Qos.ProviderSpecific.len = 0;
    Qos.ProviderSpecific.buf = NULL;
    if ( iSocketType == SOCK_DGRAM && remoteaddr != NULL)
	{
        QOS_DESTADDR QosDestaddr;
 		struct sockaddr_in local;

		//if SIO_SET_QOS is used to set QOS with QOS_DESTADDR for unconnected UDP, 
		//the socket must be locally bound first, other SIO_SET_QOS will fail with 10022
		local.sin_family = AF_INET;
		local.sin_addr.s_addr = INADDR_ANY; 

		// Port MUST be in Network Byte Order
		local.sin_port = htons(6666);
		
		if (bind(s,(struct sockaddr*)&local,sizeof(local) ) 
			== SOCKET_ERROR) {
			fprintf(stderr,"bind() failed with error %d\n",WSAGetLastError());
			return FALSE;
		}
		
		ZeroMemory((char *)&QosDestaddr, sizeof(QosDestaddr));
        QosDestaddr.ObjectHdr.ObjectType   = QOS_OBJECT_DESTADDR;
        QosDestaddr.ObjectHdr.ObjectLength = sizeof(QosDestaddr);
        QosDestaddr.SocketAddress = (SOCKADDR *)remoteaddr;
        QosDestaddr.SocketAddressLength = sizeof(SOCKADDR);


		Qos.ProviderSpecific.len = QosDestaddr.ObjectHdr.ObjectLength;
        Qos.ProviderSpecific.buf = (char *)&QosDestaddr;


    }
    result = 
            WSAIoctl(s,SIO_SET_QOS, &Qos,sizeof(QOS),NULL, 0,&BytesRet, NULL,NULL);

	if (result == SOCKET_ERROR) 
	{	
		printf ("SetQoS Error %d\n", GetLastError());
		return FALSE;
	}
	
	return TRUE;
}
*/
