/*
 * Intercepts messages to windows on Win9x/NT
 *
 * Made by TT 11Oct05 in order to work around a crash that happens with the RB-only version by Aaron Ballman on NT4
 * Based on "Win32EventMonitor" code from James Milne, 2001
 */

//--------------------
// Your includes here
//--------------------

#include <string.h>
#if TARGET_OS_WIN32
	static void inline DebugStr(unsigned char* p) {}
	#define log(x) OutputDebugString(x)
#else
	#include <MacMemory.h>
	#include "dcon.h"
	#define log(x) dprintf(x)
#endif

//-------------------------
// RB header includes here
//-------------------------

#include "TT's Plugin Support.h"
#include "rb_plugin.h"


//-----------------------
// Some type definitions
//-----------------------

typedef long RBinteger;
typedef Boolean RBboolVar;	// use this for Boolean variables stored in this plugin
typedef Boolean RBboolPar;	// use this to receive Boolean parameters (both by value and by reference)
typedef long	RBboolRet;	// use this when returning a Boolean as a function result


//-----------------------
// Add global RB methods
//-----------------------


#if not TARGET_OS_WIN32
	typedef long HWND;
	typedef long WNDPROC;
	typedef long LONG;
#endif

typedef struct EventProcessorData *Link;
typedef Link* LinkRef;

struct EventProcessorData {
	Link	next;
	REALobject instance;
	HWND	hWnd;
	LONG	oldWindowProc;
};

extern REALclassDefinition eventProcessorObj;

far Link gFirst = nil;

static LinkRef find_hWnd (HWND hWnd)
{
	Link	*ptr = &gFirst;
	while (*ptr and ((*ptr)->hWnd != hWnd)) {
		ptr = &(*ptr)->next;
	}
	return ptr;
}

typedef RBboolRet (*WndFilterProc) (REALobject theObjInst, void* v1, RBinteger v2, RBinteger v3, RBinteger v4, RBinteger* res);

static REALevent sampleClassEvents[] = {
	{ "WndProcFilter(hwnd as Integer, msg as Integer, wPar as Integer, lPar as Integer, ByRef result as Integer) as Boolean" },
};


#if TARGET_OS_WIN32

	static LRESULT CALLBACK ProcessorWndProc (HWND hWnd, UINT message, UINT wParam, LONG lParam)
	{
		long result = 0;
		
		Link ptr = *find_hWnd (hWnd);
		if (!ptr) {
			log("Win32WndProcFilter: filter - lnk not found");
		} else {
			EventProcessorData *data = (EventProcessorData*)ptr;
		
			static WndFilterProc fp = (WndFilterProc) -1L;
			if (fp == (WndFilterProc) -1L) {
				fp = (WndFilterProc) REALGetEventInstance ((REALcontrolInstance)data->instance, &sampleClassEvents[0]);
			}
			if (!fp) {
				log("Win32WndProcFilter: no event handler");
			} else {
				Boolean handled = fp (data->instance, hWnd, message, wParam, lParam, &result);
				if (handled) {
					return result;
				}
			}
			
			// Note: originally, this code was restoring the proc to the previous one for the call, but
			// tests show that we would then not get certain events that get created e.g. by mouse drag,
			// such as WM_MOVING and WM_SIZING for move and resize of the window.
			// LONG curWindowProc = SetWindowLongA (hWnd, GWL_WNDPROC, data->oldWindowProc);
			result = CallWindowProcA ((WNDPROC)data->oldWindowProc, hWnd, message, wParam, lParam);
			// SetWindowLongA (hWnd, GWL_WNDPROC, curWindowProc);
		}
		return result;
	}

#endif

static void remove0 (REALobject instance)
{
	ClassData(eventProcessorObj, instance, EventProcessorData, data);

	if (data->hWnd) {
		HWND hWnd = data->hWnd;

		LinkRef ptr = find_hWnd (hWnd);
		if (*ptr) {
			*ptr = (*ptr)->next; // unlink
			data->hWnd = 0;
		} else {
			log("Win32WndProcFilter: remove - lnk not found");
		}

		#if TARGET_OS_WIN32
			if (GetWindowLongA (hWnd, GWL_WNDPROC) != (LONG)ProcessorWndProc) {
				// this happens when window gets closed, because RB has altered it already
				//log("Win32WndProcFilter: remove - wndProc has changed");
			} else {
				SetWindowLongA (hWnd, GWL_WNDPROC, data->oldWindowProc);
			}
		#endif
	}
}

static RBboolRet check0 (REALobject instance)
{
	ClassData(eventProcessorObj, instance, EventProcessorData, data);
	if (data->hWnd) {
		return true;
	}
	return false;
}


static void install (REALobject instance, HWND hWnd)
{
	ClassData(eventProcessorObj, instance, EventProcessorData, data);

	remove0 (instance);

	data->hWnd = hWnd;
	if (hWnd) {
		#if TARGET_OS_WIN32
			data->oldWindowProc = SetWindowLongA (data->hWnd, GWL_WNDPROC, (LONG)ProcessorWndProc);
		#endif
		data->next = gFirst;
		gFirst = data;
	} else {
		log("Win32WndProcFilter: install - hWnd is null");
	}
}

static void constructor (REALobject instance)
{
	ClassData(eventProcessorObj, instance, EventProcessorData, data);
	data->instance = instance;
}

static void constructor2 (REALobject instance, HWND hWnd)
{
	constructor (instance);
	install (instance, hWnd);
}

static void destructor (REALobject instance)
{
	ClassData(eventProcessorObj, instance, EventProcessorData, data);
	remove0 (instance);
}

REALmethodDefinition EventProcessorMethods[] = {
	{ (REALproc) constructor2, nil, "Constructor(hwnd as Integer)" },
	{ (REALproc) install, nil, "Install(hwnd as Integer)" },
	{ (REALproc) remove0, nil, "Remove()" },
	{ (REALproc) check0, nil, "Installed() as Boolean" },
};


REALclassDefinition eventProcessorObj = {
	kCurrentREALControlVersion,
	"Win32WndProcFilter", nil,
	sizeof(EventProcessorData), 0,
	(REALproc)constructor,
	(REALproc)destructor,
	nil, 0,
	EventProcessorMethods, sizeof(EventProcessorMethods) / sizeof(REALmethodDefinition),
	sampleClassEvents, sizeof(sampleClassEvents) / sizeof(REALevent),
	nil, 0
};



// ---------------------------------
// ----- Plugin Initialization -----
// ---------------------------------
	
void PluginInit (void)
{
}

void PluginExit (void)
{
}

void PluginEntry (void)
{
	InitTTsPluginSupport ();

	REALRegisterClass (&eventProcessorObj);
}
